with Ada.Text_IO;    use Ada.Text_IO;
with PolyORB.Parameters;
with Core_Listener; use Core_Listener;
with Billboard_Output; use Billboard_Output;
with Input_Types; use Input_Types;
with Output_Types; use Output_Types;

with YAMI.Agents;
with YAMI.Incoming_Messages; use  YAMI.Incoming_Messages;
with YAMI.Parameters; use  YAMI.Parameters;

with Ada.Exceptions;
with Ada.Text_IO;
with Util_Text; use Util_Text;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Simulation_Exceptions;
with Load_Configuration; use Load_Configuration;

procedure Billboard_Input_Main is

   SimulatorGUI_Connecteds,
   LocalManagerGUI_Connecteds,
   VisitorManagerGUI_connecteds,
   SpectatorGUI_connecteds : Connections_Counter;
   --counters used to notify SimulatorGUI the alive connections
   SimulatorGUI_Address,
   LocalManagerGUI_Address,
   VisitorManagerGUI_Address : Unbounded_String := Null_Unbounded_String;
   --These address identify each GUI connected to the Simulation
--   Input_Address : constant String := "tcp://127.0.0.1:8081";
   Input_Address : Unbounded_String := Null_Unbounded_String;
   Input_Channel  : constant String := "controller";
   --This should load from configuration file
  procedure Initialize_Controller is
     type Incoming_Message_Handler is new YAMI.Incoming_Messages.Message_Handler
       with null record;
     overriding procedure Call (H : in out Incoming_Message_Handler;
       Message : in out YAMI.Incoming_Messages.Incoming_Message'Class) is
         procedure Dispatch
           (Content : in out YAMI.Parameters.Parameters_Collection)
         is
         begin
            declare
              Action : constant String := Message_Name (Message);
              Caller :  Input_Types.Caller := spectatorGUI;
              Request : Input_Types.GUI_Request := connect;
              Caller_Address : constant String := Source (Message);
              Success : Boolean := False;
            begin
              Util_Text.Split_Action(Action, Caller, Request);
              Put_Line ("Caller=" &Caller'img);
              Put_Line ("Request=" &Request'img);
              case Caller is
--------------------------------------------------------------------------------
--Manage spectatorGUI requests
--------------------------------------------------------------------------------
                when spectatorGUI =>
                    case SpectatorGUI_Request(Request)  is
                      when connect =>
                          spectatorGUI_Connecteds.Increment;
                          Billboard_Output.Notify_Response_Inquiry
                                              (simulatorGUI,
                                              "updateSpectators",
                                              True,
                                              spectatorGUI_Connecteds.Get);
                          --@response to OK to SimulatorGUI
                      when disconnect =>
                          spectatorGUI_Connecteds.Decrement;
                          Billboard_Output.Notify_Response_Inquiry
                                              (simulatorGUI,
                                              "updateSpectators",
                                              True,
                                              spectatorGUI_Connecteds.Get);
                          --@response to OK to spectatorGUI
                  end case; --Spectator_Request
                  -- This case manage all Spectator Request
                  -- No request is forwarded to Core
                  -- each response is directed to SimulatorGUI
--------------------------------------------------------------------------------
--Manage simulatorGUI requests
--------------------------------------------------------------------------------
                when simulatorGUI =>
                  case simulatorGUI_Request(Request) is
                    when connect =>
                      SimulatorGUI_Connecteds.Increment_Limited(Success);
                      If Success then
                        SimulatorGUI_Address := To_Unbounded_String(
                                                      Caller_Address);
                        Reply (Message,0);
                        -- it response ok to the connect request
                        Delay 1.0;
                        -- give time to simulatorGUI to subscribe
                        -- to response_inquery Channel
                        Core_Listener.Get_Initial_configuration(
                              LocalManagerGUI_Connecteds.Get,
                              VisitorManagerGUI_connecteds.Get,
                              SpectatorGUI_connecteds.Get);
                      --Create a Core request to get Initial Configuration

                      else
                        Reject (Message,
                            "Another SimulaltorGUI is already connected in: "
                            & To_String(SimulatorGUI_Address),
                            0);
                      --@reject the original SimulatorGUI.connection request
                      end If;
                    when disconnect =>
                      SimulatorGUI_Connecteds.Decrement;
                      SimulatorGUI_Address := To_Unbounded_String("");
                      Billboard_Output.Notify_Response_Inquiry
                                            (simulatorGUI,
                                            "disconnect",
                                            True,
                                            SimulatorGUI_Connecteds.Get);
                    --@response to OK to SimulatorGUI
                    --this request always response OK
                    when startMatch =>
                      declare
                        Period_Duration : constant YAMI.Parameters.YAMI_Integer
                                    :=  Content.Get_Integer("periodDuration");
                        Break_Duration : constant YAMI.Parameters.YAMI_Integer:=
                               Content.Get_Integer("breakDuration");
                        Substitutions_Allowed :constant
                               YAMI.Parameters.YAMI_Integer :=
                               Content.Get_Integer("substitutionsAllowed");
                      begin
                           Core_Listener.Start_Match (
                             Integer'Value(Period_Duration'Img),
                             Integer'Value(Break_Duration'Img),
                             Integer'Value(Substitutions_Allowed'Img));
                      end;--closer begin
                      -- forward the Request to Core
                    when stopMatch =>
                      Core_Listener.Stop_Match;
                      -- forward the Request to Core
                    when startSecondHalf =>
                      Core_Listener.Start_Second_Half;
                      -- forward the Request to Core
                  end case;
                  --SimulatorGUI_Request
                  -- This case manage all Simulator Request
                  -- Some request are forwarded to Core
                  -- Connect and disconnect response directly to SimulatorGUI
--------------------------------------------------------------------------------
--Manage managerGUI requests
--------------------------------------------------------------------------------
                when localManagerGUI | visitorManagerGUI =>
                  case Request is
                    when connect =>
                      If Caller = localManagerGUI  then
                        localManagerGUI_Connecteds.Increment_Limited(Success);
                        If Success then
                          localManagerGUI_Address := To_Unbounded_String(
                                                      Caller_Address);
                          --@response to OK to localManagerGUI
                          Reply (Message,0);
                          -- it response ok to the connect request
                        Delay 1.0;
                        -- give time to LocalManagerGUI to subscribe
                        -- to response_inquery Channel
                          Billboard_Output.Notify_Response_Inquiry
                                              (simulatorGUI,
                                               "updateLocalManagerGUI",
                                              True,
                                              localManagerGUI_Connecteds.Get);
                          --@update localManagerGUI counter on SimulatorGUI
                          Core_Listener.Get_Manager_Configuration
                                       (localManagerGUI);
                          --forward the request to Core
                        else
                        Reject (Message,
                            "Another localManagerGUI is already connected in: "
                            & To_String(localManagerGUI_Address),
                            0);
                          --@response to FAIL to localManagerGUI.connect
                        end If;
                      -- Manage localManagerGUI connection requests
                      elsif Caller = visitorManagerGUI then
                        visitorManagerGUI_Connecteds.Increment_Limited(Success);
                        If Success then
                          visitorManagerGUI_Address := To_Unbounded_String(
                                                      Caller_Address);
                          --@response to OK to visitorManagerGUI
                          Reply (Message,0);
                          -- it response ok to the connect request
                        Delay 1.0;
                        -- give time to VisitorManagerGUI to subscribe
                        -- to response_inquery Channel
                          Billboard_Output.Notify_Response_Inquiry
                                              (simulatorGUI,
                                               "updateVisitorManagerGUI",
                                               True,
                                              visitorManagerGUI_Connecteds.Get);
                          --@update visitorManagerGUI counter on SimulatorGUI
                          Core_Listener.Get_Manager_Configuration
                                       (visitorManagerGUI);
                          --forward the request to Core
                        else
                          Reject (Message,
                           "Another visitorManagerGUI is already connected in: "
                            & To_String(visitorManagerGUI_Address),
                            0);
                          --@response to FAIL to visitorManagerGUI.connect
                        end If;
                        --response FAIL to visitorManagerGUI
                      end If;
                      -- Manage localManagerGUI connection requests
                    when disconnect =>
                      If Caller = localManagerGUI  then
                        localManagerGUI_Connecteds.Decrement;
                        Billboard_Output.Notify_Response_Inquiry
                                              (simulatorGUI,
                                               "updateLocalManagerGUI",
                                               True,
                                              localManagerGUI_Connecteds.Get);
                          --@update localManagerGUI counter on SimulatorGUI
                          Billboard_Output.Notify_Response_Inquiry
                                          (localManagerGUI,
                                           "disconnect",
                                           True,
                                          localManagerGUI_Connecteds.Get);
                          --@response to OK to localManagerGUI
                      -- manage all localManager Diconnect requests
                      elsif  Caller = visitorManagerGUI  then
                          visitorManagerGUI_Connecteds.Decrement;
                          Billboard_Output.Notify_Response_Inquiry
                                              (simulatorGUI,
                                               "updateVisitorManagerGUI",
                                               True,
                                              visitorManagerGUI_Connecteds.Get);
                        --update visitorManagerGUI counter on simulatorGUI
                          Billboard_Output.Notify_Response_Inquiry
                                          (visitorManagerGUI,
                                           "disconnect",
                                            True,
                                          visitorManagerGUI_Connecteds.Get);
                      end If;
                      -- manage all visitorManager Diconnect requests
                     when saveTeamConfig =>
                        declare
                           Team_Id : Team_Id_T;
                           Configuration : constant String := Content.Get_String
                             ("configuration");
                        begin
                           If Caller = localManagerGUI  then
                              Team_Id := Local;
                           elsif  Caller = visitorManagerGUI  then
                              Team_Id := Visitor;
                           end if;
                           Core_Listener.Save_Team_Configuration(Team_Id,
                                                                 Configuration);
                           -- forward the Request to Core
                        end;--closer begin
                        -- manage all saveTeamConfig requests
                     when tacticChange =>
                        declare
                           Team_Id : Team_Id_T;
                           Line_Up : constant YAMI.Parameters.YAMI_Integer  :=
                             Content.Get_Integer("Line_Up");
                           Pressing_Level : constant YAMI.Parameters.YAMI_Integer  :=
                             Content.Get_Integer("Pressing_Level");
                        begin
                           If Caller = localManagerGUI  then
                              Team_Id := Local;
                           elsif  Caller = visitorManagerGUI  then
                              Team_Id := Visitor;
                           end if;

                           Core_Listener.Tactic_Change
                             (Team_Id,
                              Integer'Value(Line_Up'Img),
                              Integer'Value(Pressing_Level'Img));
                           --forward request to Core
                        end;--closer begin
                        --manage all tacticChange requests
                     when playerSubstitution =>
                        declare
                           Team_Id : Team_Id_T;
                           In_Player_Id : constant YAMI.Parameters.YAMI_Integer  :=
                             Content.Get_Integer("In_Player_Id");
                           Out_Player_Id : constant YAMI.Parameters.YAMI_Integer  :=
                             Content.Get_Integer("Out_Player_Id");
                        begin
                           If Caller = localManagerGUI  then
                              Team_Id := Local;
                           elsif  Caller = visitorManagerGUI  then
                              Team_Id := Visitor;
                           end if;
                           Put_Line ("playerSubstitution="&Request'Img);
                           Core_Listener.Player_Substitution
                             (Team_Id,
                              Integer'Value(In_Player_Id'Img),
                              Integer'Value(Out_Player_Id'Img));
--                           Put_Line ("playerSubstitution request has been sent");
                           --forward request to core
                        end;
                        -- manage all playerSubstitution requests
                     when others =>
                        Put_Line ("invalid ManagerGUI_Request="&Request'Img);
                     end case;--ManagerGUI_Request
               end case; --Caller
              exception
                 when Constraint_Error =>
                    Put_Line ("Caller, Request or Parameters are invalid: "
                               &Action);
           end;
         end Dispatch;
     begin
        Message.Process_Content (Dispatch'Access);
     end Call;
     Handler : aliased Incoming_Message_Handler;
  begin
     declare
        Server_Agent : YAMI.Agents.Agent := YAMI.Agents.Make_Agent;
        Resolved_Address : String (1 .. YAMI.Agents.Max_Target_Length);
        Resolved_Address_Last : Natural;
     begin
        Input_Address  := "tcp://"& Load_Configuration.Get_Billboard_Input_Host;
        Server_Agent.Add_Listener (
           Ada.Strings.Unbounded.To_String(Input_Address),
           Resolved_Address,
           Resolved_Address_Last);
        Ada.Text_IO.Put_Line
          ("The Controller is listening on " & Resolved_Address
             (1 .. Resolved_Address_Last));
        Server_Agent.Register_Object (Input_Channel, Handler'Unchecked_Access);
        loop
           delay 1.1;
        end loop;
     end;
  exception
     when E : others =>
        Ada.Text_IO.Put_Line (Ada.Exceptions.Exception_Message (E));
  end Initialize_Controller;

begin
     Load_Configuration.Read_Configuration;
     Initialize_Controller;

    exception -- Start of exception handlers
    when Simulation_Exceptions.Configuration_Error =>
      Ada.Text_IO.Put("Problems with config/DISTRIBUTION.txt content");

    when Error : others => 
      -- Handle all others
      Ada.Text_IO.Put("Exception: ");
      Ada.Text_IO.Put_Line(Ada.Exceptions.Exception_Name(Error));
      Ada.Text_IO.Put_Line(Ada.Exceptions.Exception_Message(Error));

end Billboard_Input_Main;
