with Ada.Text_IO;    use Ada.Text_IO;
with YAMI.Agents;
with YAMI.Parameters;
with YAMI.Value_Publishers;

with Util; use Util;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Load_Configuration; use Load_Configuration;

package body Billboard_Output is
   --<summary>declare procedures that may be called remotly <summary>

   Output_Address : Unbounded_String := Null_Unbounded_String;
   -- this should be loaded by properties file
   -- Yami Publisher adress, in this location will publish all Notifications
   Output_Agent : aliased YAMI.Agents.Agent := YAMI.Agents.Make_Agent;
   -- Yami Agent, all notification are using this agent for publish
   Resolved_Address : String (1 .. YAMI.Agents.Max_Target_Length);
   Resolved_Address_Last : Natural;
   -- needed to publish using yami agent

   Luxe_Fotogram_Publisher,
   Regular_Fotogram_Publisher,
   Sporadic_Fotogram_Publisher,
   Local_Team_Players_Statistics_Publisher,
   Visitor_Team_Players_Statistics_Publisher,
   Teams_Statistics_Publisher,
   Local_Messages_Publisher,
   Visitor_Messages_Publisher,
   Spectator_Messages_Publisher,
   Local_Response_Inquiry_Publisher,
   Visitor_Response_Inquiry_Publisher,
   Simulator_Response_Inquiry_Publisher,
   Match_Status_Publisher :
   YAMI.Value_Publishers.Value_Publisher :=
     YAMI.Value_Publishers.Make_Value_Publisher;
   -- Declare Yami publishers for GUI notifications

   Luxe_Fotogram_Frame_Channel      : constant String := "Fotogram_Frame";
   Regular_Fotogram_Frame_Channel   : constant String :=
     "Regular_Fotogram_Frame";
   Sporadic_Fotogram_Frame_Channel  : constant String :=
     "Sporadic_Fotogram_Frame";
   Local_Players_Statistics_Channel : constant String :=
     "Local_Team_Players_Statistics";
   Visitor_Players_Stats_Channel   : constant String :=
     "Visitor_Team_Players_Statistics";
   Teams_Statistics_Channel         : constant String := "Teams_Statistics";
   Local_Messages_Channel           : constant String := "Local_Messages";
   Visitor_Messages_Channel         : constant String := "Visitor_Messages";
   Spectator_Messages_Channel       : constant String := "Spectator_Messages";
   Local_Response_Channel           : constant String :=
     "Local_Response_Inquiry";
   Visitor_Response_Channel         : constant String :=
     "Visitor_Response_Inquiry";
   Simulator_Response_Channel       : constant String :=
     "Simulator_Response_Inquiry";
   Match_Status_Channel             : constant String := "Match_Status";
   -- these should be load by properties file




   -----------------------------------------------------------------------------
   -- Television Billboard_Output implementation
   -----------------------------------------------------------------------------


   protected body Protected_Television_Frame is

      procedure Set(New_Frame : in Output_Types.Television_Frame_T) is
      begin
         Frame := New_Frame;
      end Set;

      function Get return Output_Types.Television_Frame_T is
      begin
         return Frame;
      end Get;

   end Protected_Television_Frame;

   procedure Send_Television_Frame(
                                   Television_Frame : in Output_Types.Television_Frame_T)
   is
      Content : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;
   begin
      -- save the new television frame into the protected object
      Protected_Television_Frame.Set(Television_Frame);

      -- populate the Yami content and publish it to the HQ publisher
      Populate_Television(Television_Frame, Content);
      Luxe_Fotogram_Publisher.Publish(Content);
   end Send_Television_Frame;

   procedure Notify_MQ_Television_Frame is
      Current_Frame : Output_Types.Television_Frame_T;
      Content : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;
   begin
      -- get the current television frame from protected object
      Current_Frame := Protected_Television_Frame.Get;
      -- populate the Yami content and publish it to the MQ publisher
      Populate_Television(Current_Frame, Content);
      Regular_Fotogram_Publisher.Publish(Content);
   end Notify_MQ_Television_Frame;

   procedure Notify_LQ_Television_Frame is
      Current_Frame : Output_Types.Television_Frame_T;
      Content : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;
   begin
      -- get the current television frame from protected object
      Current_Frame := Protected_Television_Frame.Get;
      -- populate the Yami content and publish it to the LQ publisher
      Populate_Television(Current_Frame, Content);
      Sporadic_Fotogram_Publisher.Publish(Content);
   end Notify_LQ_Television_Frame;

   -----------------------------------------------------------------------------
   -- Statistics Billboard_Output implementation
   -----------------------------------------------------------------------------


   procedure Send_Statistics_Frame(
                                   Statistics_Frame : in Output_Types.Statistics_Frame_T)
   is
      Local_Players_Content : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;
      Visitor_Players_Content : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;
      Teams_Content : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;
   begin
      -- populate the Yami content for each type of statistics
      Populate_Players_Statistics(
                                  Statistics_Frame => Statistics_Frame,
                                  Team_Id          => Output_Types.Local,
                                  Content          => Local_Players_Content);
      Populate_Players_Statistics(
                                  Statistics_Frame => Statistics_Frame,
                                  Team_Id          => Output_Types.Visitor,
                                  Content          => Visitor_Players_Content);
      Populate_Teams_Statistics(
                                Statistics_Frame => Statistics_Frame,
                                Content          => Teams_Content);
      -- publish every content to the corresponding publisher
      Local_Team_Players_Statistics_Publisher.Publish(
                                                      Local_Players_Content);
      Visitor_Team_Players_Statistics_Publisher.Publish(
                                                        Visitor_Players_Content);
      Teams_Statistics_Publisher.Publish(
                                         Teams_Content);
   end Send_Statistics_Frame;


   -----------------------------------------------------------------------------
   -- Message Billboard_Output implementation
   -----------------------------------------------------------------------------
   procedure Send_Message(Message : in String;
                          Receiver : in Output_Types.Message_Receiver_T) is
      Content : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;
   begin
      -- set the message to the Yami content
      Content.Set_String("Message", Message);
      -- publish the Yami content to the correct publisher
      case Receiver is
         when Output_Types.Local_Manager =>
            Local_Messages_Publisher.Publish(Content);
         when Output_Types.Visitor_Manager =>
            Visitor_Messages_Publisher.Publish(Content);
         when Output_Types.Spectator =>
            Spectator_Messages_Publisher.Publish(Content);
         when Output_Types.All_Receivers =>
            Local_Messages_Publisher.Publish(Content);
            Visitor_Messages_Publisher.Publish(Content);
            Spectator_Messages_Publisher.Publish(Content);
      end case;
   end Send_Message;


   --@Receive Notification and add Local Response Inquery to Buffer
   procedure Notify_Response_Inquiry
     (Destination    : in Input_Types.Caller;
      Reply_Type     : in String;
      Reply_Status   : in Boolean;
      Reply_Numeric_Detail  : in Integer) is
      Content : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;
   begin
      Put_Line ("Billboard_Output notified: Notify_Response_Inquiry");
      Put_Line ("Reply_Type="&Reply_Type);
      Populate_Response_Inquery(
                                Reply_Type       => Reply_Type,
                                Reply_Status      => Reply_Status,
                                Reply_Numeric_Detail    => Reply_Numeric_Detail,
                                Content       => Content);
      case Destination is
         when Input_Types.LocalManagerGUI =>
            Local_Response_Inquiry_Publisher.Publish (Content);
         when Input_Types.VisitorManagerGUI =>
            Visitor_Response_Inquiry_Publisher.Publish (Content);
         when Input_Types.SimulatorGUI =>
            Simulator_Response_Inquiry_Publisher.Publish (Content);
         when others =>
            Put_Line ("invalid Destination");
            --Should raise an exception
      end case;
      --@Transmit the notification to GUI's using Yami
   end Notify_Response_Inquiry;


   procedure Response_Manager_Save
     (Destination    : in Input_Types.Caller;
      Reply_Type     : in String;
      Reply_Status   : in Output_Types.Configuration_Code_T;
      Team_Id        : in String;
      Formation_Id   : in Natural;
      Pressing_Level : in Integer)
   is
      Content : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;
   begin
      Put_Line ("Billboard_Output notified: Response_Manager_Save");
      Put_Line ("Reply_Type="&Reply_Type);
      Populate_Response_Manager_Save(
                                     Reply_Type      => Reply_Type,
                                     Reply_Status    => Reply_Status,
                                     Team_Id         => Team_Id,
                                     Formation_Id    => Formation_Id,
                                     Pressing_Level  => Pressing_Level,
                                     Content         => Content);
      case Destination is
         when Input_Types.LocalManagerGUI =>
            Local_Response_Inquiry_Publisher.Publish (Content);
         when Input_Types.VisitorManagerGUI =>
            Visitor_Response_Inquiry_Publisher.Publish (Content);
         when others =>
            Put_Line ("invalid Destination");
            --Should raise an exception
      end case;  end Response_Manager_Save;







   procedure Notify_Match_Status
     (Match_Status      : in Output_Types.Match_Status)
   is
      Content : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;

   begin
      Put_Line ("Billboard_Output notified: Notify_Match_Status");
      Populate_Match_Status(
                            Match_Status  => Match_Status,
                            Content       => Content);
      Match_Status_Publisher.Publish (Content);
      --@Transmit the notification to remote Procedures using Yami
   end Notify_Match_Status;


   procedure Notify_Started_Lineup
     (ManagerGUI         : in Input_Types.Caller;
      Configuration_Xml  : in String;
      Match_Status       : in Output_Types.Match_Status;
      Team_Id            : in String;
      Team_Formation     : in Natural;
      Pressing_Level     : in Natural)
   is
      Content : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;
      use Input_Types;
   begin
      Put_Line ("Billboard_Output notified: Notify_Started_Lineup");
      Populate_Started_Lineup(
                              Configuration_Xml  => Configuration_Xml,
                              Match_Status       => Match_Status,
                              Team_Id            => Team_Id,
                              Team_Formation     => Team_Formation,
                              Pressing_Level     => Pressing_Level,
                              Content            => Content);

      If ManagerGUI = localManagerGUI then
         Local_Response_Inquiry_Publisher.Publish (Content);
      elsif ManagerGUI = visitorManagerGUI then
         Visitor_Response_Inquiry_Publisher.Publish (Content);
      end If;
      --Notify_Match_Status(Match_Status);
      --@Transmit the notification to remote Procedures using Yami
   end Notify_Started_Lineup;

   procedure Notify_Initial_Configuration
     (LocalManagerGUI_connecteds   : in integer;
      VisitorManagerGUI_connecteds : in integer;
      SpectatorGUI_connecteds      : in integer;
      Current_Status               : in Output_Types.Match_Status;
      Configuration_Xml            : in String;
      Period_Duration		          : in Positive;
      Break_Duration		            : in Positive;
      Substitution_Allowed	        : in Positive) is

      Content     : YAMI.Parameters.Parameters_Collection :=
        YAMI.Parameters.Make_Parameters;
      use Input_Types;

   begin
      Put_Line ("Billboard_Output notified : Notify_Initial_Configuration");
      delay 1.0;
      Populate_Initial_Configuration(
                                     LocalManagerGUI_connecteds   => LocalManagerGUI_connecteds,
                                     VisitorManagerGUI_connecteds   => VisitorManagerGUI_connecteds,
                                     SpectatorGUI_connecteds => SpectatorGUI_connecteds,
                                     Configuration_Xml       => Configuration_Xml,
                                     Current_Status          => Current_Status,
                                     Period_Duration => Period_Duration,
                                     Break_Duration => Break_Duration,
                                     Substitution_Allowed => Substitution_Allowed,
                                     Reply_Type       => "connect",
                                     Reply_Status   => True,
                                     Content       => Content);
      Simulator_Response_Inquiry_Publisher.Publish(Content);
      --Notify_SimulatorGUI (Match_Status);
      --@Transmit the notification to remote Procedures using Yami
   end Notify_Initial_Configuration;

   -----------------------------------------------------------------------------
   --  Procedure called by outside
   -----------------------------------------------------------------------------
   procedure Initialize_Publishers is
   begin
      Output_Address  := "tcp://"& Load_Configuration.Get_Billboard_Output_Host;

      Output_Agent.Add_Listener
        (Ada.Strings.Unbounded.To_String(Output_Address),
         Resolved_Address,
         Resolved_Address_Last);
      -- setup the only listener

      Ada.Text_IO.Put_Line
        (Luxe_Fotogram_Frame_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Luxe_Fotogram_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Luxe_Fotogram_Frame_Channel);
      --register Luxe_Fotogram_Frame Publisher Channel

      Ada.Text_IO.Put_Line
        (Regular_Fotogram_Frame_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Regular_Fotogram_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Regular_Fotogram_Frame_Channel);
      --register Regular_Fotogram_Frame Publisher Channel

      Ada.Text_IO.Put_Line
        (Sporadic_Fotogram_Frame_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Sporadic_Fotogram_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Sporadic_Fotogram_Frame_Channel);
      --register  Publisher Sporadic_Fotogram_Frame Channel

      Ada.Text_IO.Put_Line
        (Local_Players_Statistics_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Local_Team_Players_Statistics_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Local_Players_Statistics_Channel);
      --register  Publisher Local_Players_Statistics Channel

      Ada.Text_IO.Put_Line
        (Visitor_Players_Stats_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Visitor_Team_Players_Statistics_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Visitor_Players_Stats_Channel);
      --register  Publisher Visitor_Players_Stats Channel

      Ada.Text_IO.Put_Line
        (Teams_Statistics_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Teams_Statistics_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Teams_Statistics_Channel);
      --register Teams_Statistics Publisher Channel

      Ada.Text_IO.Put_Line
        (Local_Messages_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Local_Messages_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Local_Messages_Channel);
      --register Local_Messages Publisher Channel

      Ada.Text_IO.Put_Line
        (Visitor_Messages_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Visitor_Messages_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Visitor_Messages_Channel);
      --register Visitor_Messages Publisher Channel

      Ada.Text_IO.Put_Line
        (Spectator_Messages_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Spectator_Messages_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Spectator_Messages_Channel);
      --register Spectator_Messages Publisher Channel

      Ada.Text_IO.Put_Line
        (Local_Response_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Local_Response_Inquiry_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Local_Response_Channel);
      --register Local_Response_Inquiry Publisher Channel

      Ada.Text_IO.Put_Line
        (Visitor_Response_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Visitor_Response_Inquiry_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Visitor_Response_Channel);
      --register Visitor_Response_Inquiry Publisher Channel

      Ada.Text_IO.Put_Line
        (Simulator_Response_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Simulator_Response_Inquiry_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Simulator_Response_Channel);
      --register Simulator_Response_Inquiry Publisher Channel

      Ada.Text_IO.Put_Line
        (Match_Status_Channel &" is listening on " &
         Resolved_Address (1 .. Resolved_Address_Last));
      Match_Status_Publisher.Register_At
        (Output_Agent'Unchecked_Access, Match_Status_Channel);
      --register Match_Status Publisher Channel

   end Initialize_Publishers;
   -- <summary> just called one time on initialization phase </summary>


end Billboard_Output;
