with Input_Types;
with Output_Types;
package Billboard_Output is
--  <description>
--    This billboard package, contain all Remote Procedures
--    these procedures are called by Core  to notify events.
--  </description>
--  <group>latinada</group>
--  <c_version>1.0.0</c_version>

   pragma Remote_Call_Interface;

   procedure Send_Television_Frame(
       Television_Frame : in Output_Types.Television_Frame_T);
   --  <summary> a remote procedure used by core to send a television frame to
   --  Billboard_Output</summary>
   --  <parameter name="Television_Frame">Television_Frame</parameter>
   procedure Notify_MQ_Television_Frame;
   --  <summary>a public procedure used by MQ_Television_Manager_Task to get and
   --  send television frame to medium quality SpectatorGUI </summary>
   procedure Notify_LQ_Television_Frame;
   --  <summary> a public procedure used by LQ_Television_Manager_Task to get
   --  and send television frame to low quality SpectatorGUI</summary>


   procedure Send_Statistics_Frame(
       Statistics_Frame : in Output_Types.Statistics_Frame_T);
   --  <summary>a remote procedure used by core to send a statistics frame to
   --  Billboard_Output </summary>
   --  <parameter name="Statistics_Frame">Statistics_Frame</parameter>

   procedure Send_Message(Message : in String;
                          Receiver : in Output_Types.Message_Receiver_T);
   --  <summary>a procedure used by core to send messages to Billboard_Output;
   --  </summary>
   --  <parameter name="Message">Message to send</parameter>
   --  <parameter name="Receiver">says who is the receiver of the message (local
   --   team ManagerGUI, visitor team ManagerGUI or SpectatorGUIs)</parameter>

   procedure Initialize_Publishers;
   --  <summary>Exec all required yami4 initialization; prepare all channels
   --   get ready to submit messages.
   --   The next channels are initialized:
   --      Luxe_Fotogram_Frame_Channel
   --      Regular_Fotogram_Frame_Channel
   --      Sporadic_Fotogram_Frame_Channel
   --      Local_Players_Statistics_Channel
   --      Visitor_Players_Stats_Channel
   --      Teams_Statistics_Channel
   --      Local_Messages_Channel
   --      Visitor_Messages_Channel
   --      Spectator_Messages_Channel
   --      Local_Response_Channel
   --      Visitor_Response_Channel
   --      Spectator_Response_Channel </summary>
   --  <seealso></seealso>

   procedure Notify_Response_Inquiry
      (Destination    : in Input_Types.Caller;
       Reply_Type     : in String;
       Reply_Status   : in Boolean;
       Reply_Numeric_Detail  : in Integer);
   --  <summary>This procedure Notify inquiry responses;
   --  <parameter name="Destination">The original Caller</parameter>
   --  <parameter name="Reply_Type">The original Request</parameter>
   --  <parameter name="Reply_Status">Response status</parameter>
   --  <parameter name="Reply_Numeric_Detail">Numeric_Detail </parameter>
   --  <exception>No exception</exception>
   --  <seealso></seealso>

   procedure Response_Manager_Save
      (Destination    : in Input_Types.Caller;
       Reply_Type     : in String;
       Reply_Status   : in Output_Types.Configuration_Code_T;
       Team_Id        : in String;
       Formation_Id   : in Natural;
       Pressing_Level : in Integer);
   --  <summary>This procedure Notify the reply for save requests;
   --  <parameter name="Destination">The original Caller</parameter>
   --  <parameter name="Reply_Type">The original Request</parameter>
   --  <parameter name="Reply_Status">Save status</parameter>
   --  <parameter name="Team_Id">Numeric_Detail </parameter>
   --  <parameter name="Formation_Id">Formation identifier </parameter>
   --  <parameter name="Pressing_Level">Pressing level </parameter>
   --  <exception>No exception</exception>
   --  <seealso></seealso>


  procedure Notify_Match_Status
    (Match_Status      : in Output_Types.Match_Status);
   --  <summary>This procedure Notify to GUI's the different Match Status changes
   --  <parameter name="Match_Status">Match Status</parameter>

  procedure Notify_Started_Lineup
    (ManagerGUI         : in Input_Types.Caller;
     Configuration_Xml  : in String;
     Match_Status       : in Output_Types.Match_Status;
     Team_Id            : in String;
     Team_Formation     : in Natural;  
     Pressing_Level     : in Natural);
   --  <summary>This procedure Notify to Manager GUI's Initial Started
   --   Team Configuration </summary>
   --  <parameter name="ManagerGUI">Destinatary ManagerGUI</parameter>
   --  <parameter name="Configuration_Xml">loaded configuration</parameter>
   --  <parameter name="Current_Status">Current Match Status</parameter>
   --  <parameter name="Team_Id">Team linked to this ManagerGUI</parameter>
   --  <parameter name="Team_Formation">Current team formation</parameter>
   --  <parameter name="Pressing_Level">Current team pressing level</parameter>


  procedure Notify_Initial_Configuration
    (LocalManagerGUI_connecteds   : in integer;
     VisitorManagerGUI_connecteds : in integer;
     SpectatorGUI_connecteds      : in integer;
     Current_Status               : in Output_Types.Match_Status;
     Configuration_Xml            : in String;
     Period_Duration		          : in Positive;
     Break_Duration		            : in Positive;
     Substitution_Allowed	        : in Positive);
   --  <summary>This procedure Notify to SimulatorGUI the Initial Configuration
   --   and the current Match State </summary>
   --  <parameter name="ManagerGUI_connecteds">Number of connected ManagerGUI's
   --  </parameter>
   --  <parameter name="SpectatorGUI_connecteds">Number of connected
   --  spectatorGUI's </parameter>
   --  <parameter name="Current_State">match state</parameter>
   --  <parameter name="Configuration_Xml">loaded configuration</parameter>
   --  <parameter name="Period_Duration">default period duration</parameter>
   --  <parameter name="Break_Duration">default break duration</parameter>
   --  <parameter name="Substitution_Allowed">default substitution allowed</parameter>
   --  <exception>No exception</exception>
   --  <seealso>SimulatorGUI</seealso>

private

   protected Protected_Television_Frame is
      procedure Set(New_Frame : in Output_Types.Television_Frame_T);
   --  <summary>a procedure to set a new frame </summary>

      function Get return Output_Types.Television_Frame_T;
   --  <summary>a function to get the current frame </summary>
   private
      Frame : Output_Types.Television_Frame_T;
   end Protected_Television_Frame;
   --  <summary>a protected object to store the current television frame
   --  </summary>

end Billboard_Output;
