package Output_Types is
   pragma Remote_Types;
   -----------------------------------------------------------------------------
   -- shared types
   -----------------------------------------------------------------------------

   type Player_State_T is (Inactive, Active, Injured, Substitution, Sent_Off);
   --  <summary>an enum type to say the player status</summary>
   subtype Player_Index_T is Positive range 1..11;
   --  <summary>a positive range subtype to store player task id</summary>
   subtype Player_Number_T is Positive range 1..17;
   --  <summary>a positive range subtype to store player number </summary>
   type Team_Id_T is (Local, Visitor);
   --  <summary> an enum type to identify a team in the simulation</summary>
   type Coordinates_T is tagged record
      X: Integer; -- X_Range_T;
      Y: Integer; -- Y_Range_T;
   end record;
   --  <summary>a record type to store 2D coordinates </summary>
   --  <parameter name="X_Range_T">X_Range_T</parameter>
   --  <parameter name="Y_Range_T">Y_Range_T</parameter>
   type Coordinates_3D_T is new Coordinates_T with record
      Z: Float;
   end record;
   --  <summary>a record type to store 3D coordinates </summary>


   type Configuration_Code_T is (Success,
                                 Team_Already_Chosen,
                                 Configuration_Already_Present,
                                 Configuration_Not_Allowed);
   -- <summary>an enum type to represent configuration status code </summary>


   subtype Length is Natural range 0 .. 100;
   -- maximum string length for DSA
   -- This should be used to delimit the strig longitud

   -----------------------------------------------------------------------------
   -- Television Billboard_Output implementation
   -----------------------------------------------------------------------------

   type Player_Frame_T is record
      Team_Id : Team_Id_T;
      Shirt_Number: Positive;
      -- player position
      Player_Position : Coordinates_T;
   end record;
   --  <summary>a struct type to store the players info to send within
   --  a television frame </summary>
   --  <parameter name="Player_Position">player position</parameter>
   type Players_Frame_Array is array (Team_Id_T, Player_Index_T)
     of Player_Frame_T;
   --  <summary> </summary>
   type Television_Frame_T is record
      -- two natural to store the minutes and seconds passed from current period
      -- start time
      Minutes : Natural;
      Seconds : Natural;
      -- the ball position
      Ball_Position : Coordinates_3D_T;
      Players_Array : Players_Frame_Array;
      -- TODO: add time information!
   end record;
   --  <summary>a struct type to implement the television frame </summary>
   --  <parameter name="Minutes">store the minutes from current period</parameter>
   --  <parameter name="Seconds">store the Seconds from current period</parameter>
   --  <parameter name="Ball_Position"></parameter>
   --  <parameter name="Players_Array"></parameter>

   type Television_Type is (MQ, LQ);
   --  <summary>an enum type to say if a television is medium or low quality </summary>
   --  <parameter name="MQ">Medium Quality</parameter>
   --  <parameter name="LQ">Low Quality</parameter>

   -----------------------------------------------------------------------------
   -- Statistics Billboard_Output implementation
   -----------------------------------------------------------------------------

   type Player_Statistics_T is record
      Shirt_Number		: Positive;
      Catches_Number          	: Natural := 0;
      Goals_Number            	: Natural := 0;
      Autogoals_Number        	: Natural := 0;
      Shootings_Number        	: Natural := 0;
      Passings_Number         	: Natural := 0;
      Headings_Number         	: Natural := 0;
      Tackles_Done_Number     	: Natural := 0;
      Tackles_Suffered_Number 	: Natural := 0;
      Fouls_Done_Number       	: Natural := 0;
      Fouls_Suffered_Number   	: Natural := 0;
      Current_Fitness         	: Float := 100.0;
      Yellow_Cards_Number     	: Natural := 0;
      Red_Cards_Number        	: Natural := 0;
      Current_Status 		: Player_State_T;
   end record;
   --  <summary>a record type to store the statistics of a particular player </summary>
   type Team_Statistics_T is record
      Name			: String(1..30) := (others => ' ');
      Score                   	: Natural := 0;
      Catches_Number          	: Natural := 0;
      Autogoals_Number        	: Natural := 0;
      Shootings_Number        	: Natural := 0;
      Substitutions_Number	: Natural := 0;
      Throw_In_Number         	: Natural := 0;
      Free_Kicks_Number       	: Natural := 0;
      Penalty_Kicks_Number    	: Natural := 0;
      Goal_Kicks_Number       	: Natural := 0;
      Kick_Off_Number         	: Natural := 0;
      Corner_Kicks_Number     	: Natural := 0;
      Passings_Number         	: Natural := 0;
      Headings_Number         	: Natural := 0;
      Tackles_Done_Number     	: Natural := 0;
      Tackles_Suffered_Number 	: Natural := 0;
      Fouls_Done_Number       	: Natural := 0;
      Fouls_Suffered_Number   	: Natural := 0;
      Yellow_Cards_Number     	: Natural := 0;
      Red_Cards_Number        	: Natural := 0;
   end record;
   --  <summary> </summary>
   --  parameter name="Score">Score</parameter>
   --  parameter name="Catches_Number">Catches_Number</parameter>
   --  parameter name="Autogoals_Number">Autogoals_Number</parameter>
   --  parameter name="Shootings_Number">Shootings_Number</parameter>
   --  parameter name="Substitutions_Number">Substitutions_Number</parameter>
   --  parameter name="Throw_In_Number">Throw_In_Number</parameter>
   --  parameter name="Free_Kicks_Number">Free_Kicks_Number</parameter>
   --  parameter name="Penalty_Kicks_Number">Penalty_Kicks_Number</parameter>
   --  parameter name="Goal_Kicks_Number">Goal_Kicks_Number</parameter>
   --  parameter name="Kick_Off_Number">Kick_Off_Number</parameter>
   --  parameter name="Corner_Kicks_Number">Corner_Kicks_Number</parameter>
   --  parameter name="Passings_Number">Passings_Number</parameter>
   --  parameter name="Headings_Number">Headings_Number</parameter>
   --  parameter name="Tackles_Done_Number">Tackles_Done_Number</parameter>
   --  parameter name="Tackles_Suffered_Number">Tackles_Suffered_Number</parameter>
   --  parameter name="Fouls_Done_Number">Fouls_Done_Number</parameter>
   --  parameter name="Fouls_Suffered_Number">Fouls_Suffered_Number</parameter>
   --  parameter name="Yellow_Cards_Number">Yellow_Cards_Number</parameter>
   --  parameter name="Red_Cards_Number">Red_Cards_Number</parameter>


   type Players_Statistics_Array is array (Team_Id_T, Player_Number_T) of Player_Statistics_T;
   --  <summary> </summary>
   type Teams_Statistics_Array is array (Team_Id_T) of Team_Statistics_T;
   --  <summary> </summary>
   type Statistics_Frame_T is record
      -- a bidimensional array to store all the player statistics
      Players_Statistics : Players_Statistics_Array;
      -- an array to store all the teams statistics
      Teams_Statistics : Teams_Statistics_Array;
   end record;
   --  <summary>a record type to store the simulation statistics </summary>

   -----------------------------------------------------------------------------
   -- Message Billboard_Output implementation
   -----------------------------------------------------------------------------

   type Message_Receiver_T is (Local_Manager, Visitor_Manager,
                               Spectator, All_Receivers);
   --  <summary> </summary>

   type Match_Status is (
                         Warm_Up,
                         Match_Started,
                         Second_Half_Started,
                         First_Half_Finished,
                         Second_Half_Finished,
                         Match_Aborted
                        );
   --  <summary> Match Game States used to identify the current Match period
   --  </summary>

end Output_Types;
--  <summary>This package define all Types used in the RCI Billboard_Output
--           also are defined other types used also by Core.gpr</summary>
