with Output_Types; use Output_Types;
with Ada.Strings.Fixed;

package body Util is

   procedure Populate_Television(Television_Frame : in Television_Frame_T;
                      Content : out YAMI.Parameters.Parameters_Collection)
    is
      -- a Yami array (of integer) to represent the ball 3D coordinates
      Ball_Position : Yami_Int_Array:= (0,0,0);
      -- a Yami bisimensional array to store players frames
      Players_Frames : Yami_Players_Array;

      -- a procedure to set a Yami integer array in the Yami content
      procedure Yami_Set_Integer_Array is new YAMI.Parameters.Set_Integer_Array
        (Integer_Type => Yami_Int_Type,
         Index_Type => Yami_Index_Type,
         Integer_Array_Type => Yami_Int_Array);

      Yami_Minutes : constant YAMI.Parameters.YAMI_Integer :=
        YAMI.Parameters.YAMI_Integer(Television_Frame.Minutes);
      Yami_Seconds : constant YAMI.Parameters.YAMI_Integer :=
        YAMI.Parameters.YAMI_Integer(Television_Frame.Seconds);
   begin
      -- set the minutes and seconds in the Yami content
      Content.Set_Integer("Minutes", Yami_Minutes);
      Content.Set_Integer("Seconds", Yami_Seconds);

      -- set the ball position in the Yami content
      Ball_Position := (Yami_Int_Type(Television_Frame.Ball_Position.X),
                        Yami_Int_Type(Television_Frame.Ball_Position.Y),
                        Yami_Int_Type(Television_Frame.Ball_Position.Z*100.0));
      Yami_Set_Integer_Array(Content, "Ball_Position", Ball_Position);

      -- iterate on teams
      for Team in Team_Id_T loop
         -- iterate on team players
         for Player in Player_Index_T loop
            -- set the player frame
            Players_Frames(Team,Player) :=
              (Yami_Int_Type(Television_Frame.Players_Array(Team,Player).
                                        Shirt_Number),
               Yami_Int_Type(Television_Frame.Players_Array(Team,Player).
                                        Player_Position.X),
               Yami_Int_Type(Television_Frame.Players_Array(Team,Player).
                                        Player_Position.Y));
            -- set the player frame in the Yami content
            Yami_Set_Integer_Array
              (Content,
               Team'Img & "_Player" &
               Ada.Strings.Fixed.Trim(Player'Img,Ada.Strings.Both) & "_Frame",
               Players_Frames(Team,Player));
         end loop; -- end loop on team players
      end loop; -- end loop on teams
   end Populate_Television;
    --  <summary>a private function to populate the Yami content for
    --          a television frame</summary>


   procedure Populate_Players_Statistics(
                  Statistics_Frame : in Statistics_Frame_T;
                  Team_Id : in Team_Id_T;
                  Content : out YAMI.Parameters.Parameters_Collection)
   is
      Shirt_Number, Catches_Number, Goals_Number, Autogoals_Number,
      Shootings_Number, Passings_Number, Headings_Number, Tackles_Done_Number,
      Tackles_Suffered_Number, Fouls_Done_Number, Fouls_Suffered_Number,
      Yellow_Cards_Number, Red_Cards_Number :YAMI.Parameters.YAMI_Integer;
      Current_Fitness : YAMI.Parameters.YAMI_Long_Float;
      Current_Status : Player_State_T;
   begin
      -- iterate on team players
      for Player in Player_Number_T loop
         -- for each player get statistics values and add them to the content
         Shirt_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                          .Shirt_Number);
         Catches_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Catches_Number);
         Goals_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Goals_Number);
         Autogoals_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Autogoals_Number);
         Shootings_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Shootings_Number);
         Passings_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Passings_Number);
         Headings_Number := YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Headings_Number);
         Tackles_Done_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Tackles_Done_Number);
         Tackles_Suffered_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Tackles_Suffered_Number);
         Fouls_Done_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Fouls_Done_Number);
         Fouls_Suffered_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Fouls_Suffered_Number);
         Yellow_Cards_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Yellow_Cards_Number);
         Red_Cards_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame
                                        .Players_Statistics(Team_Id,Player)
                                        .Red_Cards_Number);
         Current_Fitness :=
           YAMI.Parameters.YAMI_Long_Float(Statistics_Frame
                                           .Players_Statistics(Team_Id,Player)
                                           .Current_Fitness);
         Current_Status := Statistics_Frame.Players_Statistics(Team_Id,Player)
           .Current_Status;
         -- populate Yami content
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Shirt_Number", Shirt_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Catches_Number", Catches_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Goals_Number", Goals_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Autogoals_Number", Autogoals_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Shootings_Number", Shootings_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Passings_Number", Passings_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Headings_Number", Headings_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Tackles_Done_Number", Tackles_Done_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                           "_Tackles_Suffered_Number", Tackles_Suffered_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Fouls_Done_Number", Fouls_Done_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Fouls_Suffered_Number", Fouls_Suffered_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Yellow_Cards_Number", Yellow_Cards_Number);
         Content.Set_Integer("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                             "_Red_Cards_Number", Red_Cards_Number);
         Content.Set_Long_Float("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                                "_Current_Fitness", Current_Fitness);
         Content.Set_String("Player" & Ada.Strings.Fixed.Trim(
                             Player'Img,Ada.Strings.Both) &
                            "_Current_Status", Current_Status'Img);
      end loop;
   end Populate_Players_Statistics;

   -- a private procedure to populate the Yami content for the teams statistics
   procedure Populate_Teams_Statistics(
              Statistics_Frame : in Statistics_Frame_T;
              Content : out YAMI.Parameters.Parameters_Collection)
   is
      Name : String(1..30);
      Score, Catches_Number, Autogoals_Number, Shootings_Number,
      Substitutions_Number,Throw_In_Number, Free_Kicks_Number,
      Penalty_Kicks_Number, Goal_Kicks_Number, Kick_Off_Number,
      Corner_Kicks_Number, Passings_Number, Headings_Number,
      Tackles_Done_Number, Tackles_Suffered_Number, Fouls_Done_Number,
      Fouls_Suffered_Number, Yellow_Cards_Number,
      Red_Cards_Number :YAMI.Parameters.YAMI_Integer;
   begin
      -- iterate on teams
      for Team in Team_Id_T loop
         -- for each player team get  statistics values and add them to content
         Name := Statistics_Frame.Teams_Statistics(Team).Name;
         Score :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Score);
         Catches_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Catches_Number);
         Autogoals_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Autogoals_Number);
         Shootings_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Shootings_Number);
         Substitutions_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Substitutions_Number);
         Throw_In_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Throw_In_Number);
         Free_Kicks_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Free_Kicks_Number);
         Penalty_Kicks_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Penalty_Kicks_Number);
         Goal_Kicks_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Goal_Kicks_Number);
         Kick_Off_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Kick_Off_Number);
         Corner_Kicks_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Corner_Kicks_Number);
         Passings_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Passings_Number);
         Headings_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Headings_Number);
         Tackles_Done_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Tackles_Done_Number);
         Tackles_Suffered_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Tackles_Suffered_Number);
         Fouls_Done_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Fouls_Done_Number);
         Fouls_Suffered_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Fouls_Suffered_Number);
         Yellow_Cards_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Yellow_Cards_Number);
         Red_Cards_Number :=
           YAMI.Parameters.YAMI_Integer(Statistics_Frame.Teams_Statistics(Team)
                                        .Red_Cards_Number);
         -- populate the yami content
         Content.Set_String(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                            "_Name", Name);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Score", Score);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Catches_Number", Catches_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Autogoals_Number", Autogoals_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Shootings_Number", Shootings_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Substitutions_Number", Substitutions_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Throw_In_Number", Throw_In_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Free_Kicks_Number", Free_Kicks_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Penalty_Kicks_Number", Penalty_Kicks_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Goal_Kicks_Number", Goal_Kicks_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Kick_Off_Number", Kick_Off_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Corner_Kicks_Number", Corner_Kicks_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Passings_Number", Passings_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Headings_Number", Headings_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Tackles_Done_Number", Tackles_Done_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                           "_Tackles_Suffered_Number", Tackles_Suffered_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Fouls_Done_Number", Fouls_Done_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Fouls_Suffered_Number", Fouls_Suffered_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Yellow_Cards_Number", Yellow_Cards_Number);
         Content.Set_Integer(Ada.Strings.Fixed.Trim(Team'Img,Ada.Strings.Both) &
                             "_Red_Cards_Number", Red_Cards_Number);
      end loop;
   end Populate_Teams_Statistics;

   procedure Populate_Response_Inquery(
             Reply_Type            : in String;
             Reply_Status          : in Boolean;
             Reply_Numeric_Detail  : in Integer;
             Content       : out YAMI.Parameters.Parameters_Collection)
    is
      Number :YAMI.Parameters.YAMI_Integer;
      use type YAMI.Parameters.YAMI_Integer;
      begin
        Number := YAMI.Parameters.YAMI_Integer(Reply_Numeric_Detail);
        Content.Set_String ("Reply_Type", Reply_Type);
        Content.Set_Boolean ("Reply_Status", Reply_Status);
        Content.Set_Integer ("Reply_Numeric_Detail", Number);
        --value object assignment
   end Populate_Response_Inquery;


   procedure Populate_Response_Manager_Save(
             Reply_Type     : in String;
             Reply_Status   : in Output_Types.Configuration_Code_T;
             Team_Id        : in String;
             Formation_Id   : in Natural;
             Pressing_Level : in Integer;
             Content        : out YAMI.Parameters.Parameters_Collection)
   is
      Formation, Pressing : YAMI.Parameters.YAMI_Integer;
      use type YAMI.Parameters.YAMI_Integer;
   begin
      Formation := YAMI.Parameters.YAMI_Integer(Formation_Id);
      Pressing  := YAMI.Parameters.YAMI_Integer(Pressing_Level);
      Content.Set_String ("Reply_Type", Reply_Type);
      Content.Set_String ("Reply_Status", Reply_Status'Img);

      case Reply_Status is
         when Configuration_Already_Present | Configuration_Not_Allowed =>
            Content.Set_String ("Team_Id", Team_Id);
            Content.Set_Integer ("Formation_Id", Formation);
            Content.Set_Integer ("Pressing_Level", Pressing);
         when others =>
            null;
      end case;
      --value object assignment
   end Populate_Response_Manager_Save;


   procedure Populate_Match_Status(
             Match_Status  : in Output_Types.Match_Status;
             Content       : out YAMI.Parameters.Parameters_Collection)
    is
      begin
        Content.Set_String ("Match_Status", Match_Status'Img);
        --value object assignment
   end Populate_Match_Status;

   procedure Populate_Started_Lineup(
             Configuration_Xml  : in String;
             Match_Status       : in Output_Types.Match_Status;
             Team_Id            : in String;
             Team_Formation     : in Natural;
             Pressing_Level     : in Natural;
             Content            : out YAMI.Parameters.Parameters_Collection)
    is
        Formation, Pressing: YAMI.Parameters.YAMI_Integer;
        use type YAMI.Parameters.YAMI_Integer;
      begin
        Formation:= YAMI.Parameters.YAMI_Integer(Team_Formation);
        Pressing:= YAMI.Parameters.YAMI_Integer(Pressing_Level);
        Content.Set_String ("Reply_Type", "connect");
        Content.Set_Boolean ("Reply_Status", True);
        Content.Set_String ("Configuration_Xml", Configuration_Xml);
        Content.Set_String ("Match_Status", Match_Status'Img);
        Content.Set_String ("Team_Id", Team_Id);
        Content.Set_Integer ("Team_Formation", Formation);
        Content.Set_Integer ("Pressing_Level", Pressing);
        --value object assignment
   end Populate_Started_Lineup;





   procedure Populate_Initial_Configuration(
       LocalManagerGUI_connecteds   : in Integer;
       VisitorManagerGUI_connecteds : in Integer;
       SpectatorGUI_connecteds      : in Integer;
       Configuration_Xml            : in String;
       Current_Status               : in Output_Types.Match_Status;
       Period_Duration              : in Positive;
       Break_Duration	              : in Positive;
       Substitution_Allowed	        : in Positive;
       Reply_Type                   : in String;
       Reply_Status                 : in Boolean;
       Content                      : out YAMI.Parameters.Parameters_Collection)

   is
      LocalManagers, VisitorManagers, Spectators, Yami_Period_Duration,
      Yami_Break_Duration, Yami_Substitution_Allowed
                    : YAMI.Parameters.YAMI_Integer;
      use type YAMI.Parameters.YAMI_Integer;
   begin
      LocalManagers := YAMI.Parameters.YAMI_Integer(LocalManagerGUI_connecteds);
      VisitorManagers :=
          YAMI.Parameters.YAMI_Integer(VisitorManagerGUI_connecteds);
      Spectators  := YAMI.Parameters.YAMI_Integer(SpectatorGUI_connecteds);
      Yami_Period_Duration := YAMI.Parameters.YAMI_Integer(Period_Duration);
      Yami_Break_Duration := YAMI.Parameters.YAMI_Integer(Break_Duration);
      Yami_Substitution_Allowed :=
          YAMI.Parameters.YAMI_Integer(Substitution_Allowed);
      Content.Set_Integer("LocalManagerGUI_connecteds", LocalManagers);
      Content.Set_Integer("VisitorManagerGUI_connecteds", VisitorManagers);
      Content.Set_Integer("SpectatorGUI_connecteds", Spectators);
      Content.Set_String("Configuration_Xml", Configuration_Xml);
      Content.Set_Integer("Period_Duration", Yami_Period_Duration);
      Content.Set_Integer("Break_Duration", Yami_Break_Duration);
      Content.Set_Integer("Substitution_Allowed", Yami_Substitution_Allowed);
      Content.Set_String("Match_Status", Current_Status'Img);
      Content.Set_String("Reply_Type", Reply_Type);
      Content.Set_Boolean("Reply_Status", Reply_Status);

   end Populate_Initial_Configuration;

end Util;
