with YAMI.Parameters;
with Output_Types;
Package Util is
    type Yami_Int_Type is new Integer;
    --  <summary>Yami type used to represent a integer value</summary>
    type Yami_Index_Type is new Positive;
    --  <summary>Yami type used to represent an array index (positive)</summary>
    type Yami_Int_Array is array (Yami_Index_Type range <>) of Yami_Int_Type;
    --  <summary>Yami type used to represent an array of integer</summary>

    type Yami_Players_Array is array (Output_Types.Team_Id_T,1..11)
              of Yami_Int_Array (1..3);
    --  <summary> Yami type used to represent a bidimensional array to store
    -- the players frame; a cell contains the player shirt number and the player
    --  position</summary>


    procedure Populate_Television(
              Television_Frame : in Output_Types.Television_Frame_T;
              Content : out YAMI.Parameters.Parameters_Collection);
    --  <summary>a private function to populate the Yami content for Television
    --   frame</summary>
    --  <parameter name="Television_Frame">Frame that contain a Television 
    --  Value Object</parameter>
    --  <parameter name="Content">Yami Content, destination</parameter>

    procedure Populate_Players_Statistics(
              Statistics_Frame : in Output_Types.Statistics_Frame_T;
              Team_Id : in Output_Types.Team_Id_T;
              Content : out YAMI.Parameters.Parameters_Collection);
    --  <summary>a private procedure to populate the Yami content for
    --        players statisticsof a particular team</summary>
    --  <parameter name="Statistics_Frame">Contain a Players Statistics Value
    --   Object</parameter>
    --  <parameter name="Team_Id"></parameter>
    --  <parameter name="Content">Yami Content, destination</parameter>

   procedure Populate_Teams_Statistics(
              Statistics_Frame : in Output_Types.Statistics_Frame_T;
              Content : out YAMI.Parameters.Parameters_Collection);
    --  <summary>a private procedure to populate the Yami content
    --           for the teams statistics</summary>
    --  <parameter name="Statistics_Frame">Contain a Teams Statistics Value 
    --  object</parameter>
    --  <parameter name="Content">Yami Content, destination</parameter>

   procedure Populate_Response_Inquery(
             Reply_Type           : in String;
             Reply_Status         : in Boolean;
             Reply_Numeric_Detail : in Integer;
             Content       : out YAMI.Parameters.Parameters_Collection);
   --  <summary>Populate the Yami content for Inquery Responses </summary>
   --  <parameter name="Destination">The original Caller</parameter>
   --  <parameter name="Reply_Type">The original Request</parameter>
   --  <parameter name="Reply_Status">Response status</parameter>
   --  <parameter name="Reply_Numeric_Detail">Numeric_Detail </parameter>
   --  <parameter name="Content">Yami Content, destination</parameter>
   --  <exception>No exception</exception>
   --  <seealso></seealso>

   procedure Populate_Response_Manager_Save(
             Reply_Type     : in String;
             Reply_Status   : in Output_Types.Configuration_Code_T;
             Team_Id        : in String;
             Formation_Id   : in Natural;
             Pressing_Level : in Integer;
             Content        : out YAMI.Parameters.Parameters_Collection);
   --  <summary>Populate the Yami content for Save inquery Response </summary>
   --  <parameter name="Destination">The original Caller</parameter>
   --  <parameter name="Reply_Type">The original Request</parameter>
   --  <parameter name="Reply_Status">Response status</parameter>
   --  <parameter name="Team_Id">Team identifiator </parameter>
   --  <parameter name="Formation_Id">Formation identifier </parameter>
   --  <parameter name="Pressing_Level">Pressing Level </parameter>
   --  <parameter name="Content">Yami Content, destination</parameter>
   --  <exception>No exception</exception>
   --  <seealso></seealso>


   procedure Populate_Match_Status(
             Match_Status  : in Output_Types.Match_Status;
             Content       : out YAMI.Parameters.Parameters_Collection);
   --  <summary>Populate the Yami content for Match Status notifications
   --  <parameter name="Match_Status">Match Status</parameter>
   --  <parameter name="Content">Yami Content, destination</parameter>


   procedure Populate_Started_Lineup(
             Configuration_Xml  : in String;
             Match_Status       : in Output_Types.Match_Status;
             Team_Id            : in String;
             Team_Formation     : in Natural;  
             Pressing_Level     : in Natural;
             Content            : out YAMI.Parameters.Parameters_Collection);
    --  <summary>Populate the Yami content for Started_Lineup
    --  </summary>
   --  <parameter name="Configuration_Xml">loaded configuration</parameter>
   --  <parameter name="Current_Status">Current Match Status</parameter>
   --  <parameter name="Team_Id">Team linked to this ManagerGUI</parameter>
   --  <parameter name="Team_Formation">Current team formation</parameter>
   --  <parameter name="Pressing_Level">Current team pressing level</parameter>
   --  <parameter name="Content">Yami Content, destination</parameter>
   --  <exception>No exception</exception>
   --  <seealso>SimulatorGUI</seealso>


   procedure Populate_Initial_Configuration(
       LocalManagerGUI_connecteds   : in Integer;
       VisitorManagerGUI_connecteds : in Integer;
       SpectatorGUI_connecteds      : in Integer;
       Configuration_Xml            : in String;
       Current_Status               : in Output_Types.Match_Status;
       Period_Duration              : in Positive;
       Break_Duration	              : in Positive;
       Substitution_Allowed	        : in Positive;
       Reply_Type                   : in String;
       Reply_Status                 : in Boolean;
       Content                      : out YAMI.Parameters.Parameters_Collection
      );
   --  <summary>This procedure populate the yami content with tge initial
   --   configuration</summary>
   --  <parameter name="ManagerGUI_connecteds">Number of connected ManagerGUI's
   --  </parameter>
   --  <parameter name="SpectatorGUI_connecteds">Number of connected
   --  spectatorGUI's </parameter>
   --  <parameter name="Current_State">match state</parameter>
   --  <parameter name="Configuration_Xml">loaded configuration</parameter>
   --  <parameter name="Period_Duration">default period duration</parameter>
   --  <parameter name="Break_Duration">default break duration</parameter>
   --  <parameter name="Substitution_Allowed">default substitution allowed
   --  </parameter>
   --  <parameter name="Content">Yami Content, destination</parameter>
   --  <exception>No exception</exception>
   --  <seealso>SimulatorGUI</seealso>

end Util;
