with Output_Types; use Output_Types;
with Ada.Containers.Doubly_Linked_Lists;
use Ada.Containers;
with Ada.Real_Time; use Ada.Real_Time;

package Field_Control is
   -- we istantiate the Doubly_Linked_Lists generic to implement a list of
   -- positions type
   package Positions_List is
     new Ada.Containers.Doubly_Linked_Lists(Coordinates_T);

   -- a protected object used as access protocol to the game field
   protected Barrier is
      -- an entry to access the game field access protocol
      entry Play(Player_Position : in Coordinates_T);
      -- a procedure to leave the game field access protocol
      procedure Leave(Player_Position : in Coordinates_T);
      -- used by the local "television" to print out the execution times
      -- measured between Play and Leave calls (for any given position)
      -- ACET is the measured Average Case Execution Time
      -- WCET is the measured Worst Case Execution Time
      procedure Read_ETs(ACET : out Time_Span; WCET : out Time_Span);
   private
      -- a private entry to requeue not allowed players
      entry Awaiting_Players(Player_Position : in Coordinates_T);
      -- a boolean to store if some player has finished his action and has left
      -- the field access protocol; we evaluate the protocol barrier only if some
      -- player has leaved the protocol
      Someone_Finished : Boolean := False;
      -- a natural to store the number of player who potentially can have access
      -- to the field access protocol when the protocol guard is evaluated; we
      -- evaluate waiting players while this value is greater than zero
      Potentially_Allowable : Natural := 0;
      -- a positions list to store the position of players who has entered the
      -- field access protocol
      Allowed : Positions_List.List;
      -- variables for Execution time measurements
      -- the start time of the last unclosed quantum
      Start : Time := Clock;
      -- no of players already playing (or awaiting)
      Playing : Natural := 0;
      -- time Span since the last opened quantum
      TS : Time_Span := Milliseconds(0);
      -- the worst case execution time already measured
      WCET : Time_Span := Milliseconds(0);
      -- variables used to measure the average case execution time
      TS_Sum : Time_Span := Milliseconds(0); -- sum of all the TSs
      TS_Records : Natural := 0; -- number of TSs counted
   end Barrier;
end Field_Control;
