with Output_Types; use Output_Types;

package Game_Control is
   -- an enumeration type to represent the game state
   type State_T is (Play,
                    Kick_Off, Throw_In, Goal_Kick, Corner_Kick,
                    Free_Kick, Penalty_Kick,
                    Half_Time_Break, End_Match,
                    Not_Started);
   -- a record type to implement the game state
   type State_Record_T is record
      -- game state
      State : State_T := Not_Started;
      -- match status
      Status : Match_Status;
      -- the ball position when the game will restart
      Position : Coordinates_3d_T;
      -- the team which must restart the game
      Offence_Team : Team_Id_T;
   end record;
   -- a protected object to store the game state
   protected Game_State is
      -- getter and setter
      function Get return State_Record_T;
      procedure Set (New_State : in State_Record_T);
      -- a procedure used by referee to end the match
      procedure End_Match;
      -- a procedure used by referee to abort the match
      procedure Abort_Match;
      -- entry used by players to wait for game to restart
      entry Wait_For_Restart;
      -- for players to position themselves before starting the 1st-2nd period
      entry Wait_For_Positioning;
      -- entry used by referee to wait for players to prepare for game restart
      entry Wait_For_Players;
      -- entry used to enable to restart the second half
      entry Enable_Start_2nd_Time_Request;
      -- entry used by injured players to wait for substitution until the game
      -- is over
      entry Wait_The_End;
      -- a procedure used to increase the number of players the referee should
      -- wait before restarting the game (used by substitutes of injured players
      -- when they enter the field
      procedure Increase_Players_Number;
      -- a procedure used to decrease the number of players the referee should
      -- wait before restarting the game (used by injured and sent off players
      -- when they leave the field
      procedure Decrease_Players_Number;
      -- a procedure used by referee to set the game state to half time break
      procedure Half_Time_Break;
      -- a procedure used by referee to restart the game
      procedure Start;
   private
      -- the match state
      State_Record : State_Record_T;
      -- the number of players in game
      Players_Number : Natural range 0..22 := 22;
   end Game_State;
end Game_Control;

