--  <description>
--    This Core package, contain all Remote Procedures
--    these procedures are called by Billboard  to notify requests.
--  </description>
--  <group>latinada</group>
--  <c_version>1.0.0</c_version>
with Input_Types;
with Output_Types; use Output_Types;

package Core_Listener is
   pragma Remote_Call_Interface;

    procedure Get_Initial_Configuration
            (LocalManagerGUI_connecteds   : in integer;
             VisitorManagerGUI_connecteds : in integer;
             SpectatorGUI_connecteds      : in integer);
   --  <summary>This procedure Notify to core that a new SimulatorGUI are
   --  connected and Core should return the initial configuration</summary>
   --  <parameter name="LocalManagerGUI_connecteds"> number of connected
   --  localManager</parameter>
   --  <parameter name="VisitorManagerGUI_connecteds"> number of connected
   --  localManager</parameter>
   --  <parameter name="SpectatorGUI_connecteds">number of connected Spectators
   --  </parameter>
   --  <exception>No exception</exception>
   --  <seealso>SimulatorGUI</seealso>

    procedure Start_Match
            (Period_Duration : in integer;
             Break_Duration : in integer;
             Substitutions_Allowed : in integer);
   --  <summary>This procedure Notify StartMatch request</summary>
   --  <parameter name="Period_Duration">each half game duration time
   --  </parameter>
   --  <parameter name="Break_Duration">break time duration </parameter>
   --  <parameter name="Substitutions_Allowed">Enable/disable manager
   --  substitutions</parameter>
   --  <exception>No exception</exception>
   --  <seealso>SimulatorGUI control panel</seealso>

    procedure Stop_Match;
   --  <summary>This procedure Notify stopMatch Request</summary>
   --  <exception>No exception</exception>
   --  <seealso>SimulatorGUI control panel</seealso>

    procedure Start_Second_Half;
   --  <summary>This procedure Notify startSecondHalf Request</summary>
   --  <exception>No exception</exception>
   --  <seealso>SimulatorGUI control panel</seealso>


    procedure Get_Manager_Configuration
             (Manager_Caller: in Input_Types.Caller);
   --  <summary>This procedure is called when the manager GUI is connected
   --  </summary>
   --  <parameter name="Team_Id">identify Team requester</parameter>
   --  <exception>No exception</exception>
   --  <seealso>ManagerGUI </seealso>


    procedure Save_Team_Configuration
             (Team_Id : in Team_Id_T;
              Configuration : in string);
   --  <summary>This procedure Notify a saveTeamConfiguration request</summary>
   --  <parameter name="Team_Id">identify Team requester</parameter>
   --  <parameter name="Configuration">contain an xml with the entire selected
   --             configuration choosenTeam, choosenPlayers, choosenModel,
   --             choosenPressingLevel  </parameter>
   --  <exception>No exception</exception>
   --  <seealso>ManagerGUI Control Panel</seealso>

    procedure Tactic_Change
             (Team_Id: in Team_Id_T;
              New_Formation_Id : in Positive;
              New_Pressing_Level : in Integer);
   --  <summary>This procedure Notify a tacticChange request</summary>
   --  <parameter name="Team_Id">identify Team requester</parameter>
   --  <parameter name="Line_Up">Describe initial players lineUp </parameter>
   --  <parameter name="Pressing_Level">identify the tactic pressingLevel
   --  </parameter>
   --  <exception>No exception</exception>
   --  <seealso>ManagerGUI Control Panel</seealso>

    procedure player_Substitution
              (Team_Id: in Team_Id_T;
              In_Player_Id : in integer;
              Out_Player_Id : in integer);
   --  <summary>This procedure Notify a playerSubstitution request</summary>
   --  <parameter name="Team_Id">identify Team requester</parameter>
   --  <parameter name="In_Player_Id">Player that will be inserted</parameter>
   --  <parameter name="Out_Player_Id">go out player</parameter>
   --  <exception>No exception</exception>
   --  <seealso>ManagerGUI Control Panel</seealso>




end Core_Listener;

