package body Distribution_Control is

   protected body Simulation_Configuration is
      procedure Init(Default_Configuration : in Simulation_Configuration_T) is
      begin
         Configuration := Default_Configuration;
      end Init;

      entry Wait_For_Simulation_Start when Start_Signal_Has_Arrived is
      begin
         -- entry is used only to wait the start signal
         null;
      end Wait_For_Simulation_Start;

      procedure Stop_Receiving is
      begin
         Can_Accept_Configuration := False;
      end Stop_Receiving;

      procedure Set(New_Configuration : in Simulation_Configuration_T;
                    Success : out Boolean) is
      begin
         -- check if core can accept configuration
         if Can_Accept_Configuration then
            -- set the new configuration and open the core barrier
            Configuration := New_Configuration;
            Start_Signal_Has_Arrived := True;
            Success := True;
         else
            -- delay already expired, return an error
            Success := False;
         end if;
      end Set;

      function Get return Simulation_Configuration_T is
      begin
         return Configuration;
      end Get;
   end Simulation_Configuration;

   protected body Teams_Configuration is
      procedure Set(Team_Id	 	: in Team_Id_T;
                    Team_Configuration  : in Team_Configuration_T;
                    Configuration_Code  : out Configuration_Code_T) is
      begin
         -- check if core can accept configuration
         if Can_Accept_Configuration then
            -- check which manager is
            case Team_Id is
               when Local =>
                  -- check if there is already the configuration for the local team
                  if Teams_Configuration(Output_Types.Local).Team_Id = Null_Unbounded_String then
                     -- check if there is already the visitor team configuration
                     -- and if so check if the team is already choosen
                     if Teams_Configuration(Output_Types.Visitor).Team_Id /= Null_Unbounded_String and
                       Teams_Configuration(Output_Types.Visitor).Team_Id = Team_Configuration.Team_Id then
                        -- return fail
                        Configuration_Code := Team_Already_Chosen;
                     else
                        -- save local team configuration and return success
                        Teams_Configuration(Output_Types.Local) := Team_Configuration;
                        Configuration_Code := Success;
                     end if;
                  else
                     -- configuration already present, return fail
                     Configuration_Code := Configuration_Already_Present;
                  end if;
               when Visitor =>
                  -- check if there is already the configuration for the visitor team
                  if Teams_Configuration(Output_Types.Visitor).Team_Id = Null_Unbounded_String then
                     -- check if there is already the local team configuration
                     -- and if so check if the team is already choosen
                     if Teams_Configuration(Output_Types.Local).Team_Id /= Null_Unbounded_String and
                       Teams_Configuration(Output_Types.Local).Team_Id = Team_Configuration.Team_Id then
                        -- return fail
                        Configuration_Code := Team_Already_Chosen;
                     else
                        -- save visitor team configuration and return success
                        Teams_Configuration(Output_Types.Visitor) := Team_Configuration;
                        Configuration_Code := Success;
                     end if;
                  end if;
               when others =>
                  -- any others values are not possible, do nothing
                  null;
            end case;
         else
            -- manager is not allowed to set configuration, return fail
            Configuration_Code := Configuration_Not_Allowed;
         end if;
      end Set;

      procedure Set_Default(Team_Id		: in Team_Id_T;
                            Xml_Team_Id		: in Unbounded_String;
                            Formation_Id	: in Natural;
                            Pressing_Value	: in Integer) is
      begin
         Teams_Configuration(Team_Id).Team_Id := Xml_Team_Id;
         Teams_Configuration(Team_Id).Formation_Id := Formation_Id;
         Teams_Configuration(Team_Id).Pressing_Balance_Value := Pressing_Value;
      end Set_Default;

      function Get return Teams_Configuration_T is
      begin
         return Teams_Configuration;
      end Get;

      procedure Stop_Receiving is
      begin
         Can_Accept_Configuration := False;
      end Stop_Receiving;
   end Teams_Configuration;
end Distribution_Control;
