with Ada.Real_Time; use Ada.Real_Time;
with Player; use Player;
with Status;
with Motion;
with Utils;
with Billboard_Output; use Billboard_Output;

package body Television is
   Television_Frequency: constant Ada.Real_Time.Time_Span := Milliseconds(30);

   task body Frame_Manager_Task is
      -- local var to store the next time when the frame manager has to send
      -- the frame to the "distribution"
      Next_Television_Quantum : Ada.Real_Time.Time;
      -- local var to store the television frame
      Television_Frame : Television_Frame_T;
      -- a boolean to say if the television timer has started
      Timer_Started : Boolean := False;
      -- a time var to store the initial time of the current period
      Init_Time : Time;
   begin
      -- wait for the start signal; after it we initialize the television frame
      -- with the initial position of the players
      accept Start  do
         -- initialize ball position
         Television_Frame.Ball_Position.X := 1;
         Television_Frame.Ball_Position.Y := 1;
         Television_Frame.Ball_Position.Z := 0.0;
         -- initialize teams players
         for Team in Team_Id_T loop
            for Player in Player_Index_T loop
               Television_Frame.Players_Array(Team, Player).Shirt_Number :=
                 Status.Player_Mapper(Team, Player).Get_Shirt_Number;
               Television_Frame.Players_Array(Team, Player).Player_Position :=
                 Status.Player_Mapper(Team, Player).Get_Position;
            end loop;
         end loop;
         Next_Television_Quantum := Clock; -- i.e. send frame right now
         -- send initial frame
         Send_Television_Frame(Television_Frame);
      end Start;
      -- start the television loop
      loop
         select
            -- wait for a request to reset the television timer
            accept Restart_Timer;
            Init_Time := Clock;
            Timer_Started := True;
         or
            accept Stop_Timer;
            Television_Frame.Minutes := 0;
            Television_Frame.Seconds := 0;
            Timer_Started := False;
         or
            -- wait for a player move
            accept Move_Player(Player_Team_Id	: in Team_Id_T;
                               Player_Task_Id 	: in Player_Index_T;
                               New_Position 	: in Coordinates_T) do
               -- update the player position in the television frame
               Television_Frame.Players_Array(Player_Team_Id, Player_Task_Id)
                 .Player_Position := New_Position;
            end Move_Player;
         or
            -- or a player substitution
            accept Substitute_Player(Player_Identifier : in Identifier_T;
                                     Player_Task_Id : in Player_Index_T) do
               -- update the player shirt number
               Television_Frame.Players_Array(Player_Identifier.Team_Id, Player_Task_Id).Shirt_Number :=
                 Status.Player_Mapper(Player_Identifier.Team_Id, Player_Task_Id).Get_Shirt_Number;
            end Substitute_Player;
         or accept Quit;
            exit;
         or delay until Next_Television_Quantum;
            -- check if the television timer has started
            if Timer_Started then
               -- timer started, compute time passed and add it to the television
               -- frame
               Television_Frame.Minutes := Natural(To_Duration(Clock - Init_Time)) / 60;
               Television_Frame.Seconds := Natural(To_Duration(Clock - Init_Time)) mod 60;
            end if;
            -- here we have to send the television frame, so we get all the
            -- necessary info
            -- get the ball position
            Television_Frame.Ball_Position :=
              Motion.Get_Current_Position(Status.Ball_Status.Get.Motion);
            -- send frame
            Send_Television_Frame(Television_Frame);
            -- set the new Next_Television_Quantum
            Next_Television_Quantum :=
              Next_Television_Quantum + Television_Frequency;
         end select;
      end loop;
   end Frame_Manager_Task;
end Television;
