with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Status; use Status;
with Utils;
with Billboard_Output; use Billboard_Output;

package body Events.Catch is
   procedure Process(Event : in Catch_Event_T) is
      Message : Unbounded_String;
      Player_Name : constant Unbounded_String :=
        Players_Characteristics(Event.Player_Id.Team_Id,
                                Event.Player_Id.Player_Number).Name;
      Team_Name : constant Unbounded_String :=
        Teams_Status(Event.Player_Id.Team_Id).Get_Name;
   begin
      -- check if the catch has been successful
      if Event.Successful then
         -- create the log message
         Message := Player_Name & " (" & Team_Name & ","
           & Players_Characteristics(Event.Player_Id.Team_Id,
                                     Event.Player_Id.Player_Number)
           .Shirt_Number'Img & ") gets the ball.";
         -- update player and team statistics (#of catch)
         Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                       Event.Player_Id.Player_Number)
           .Catches_Number := Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                                            Event.Player_Id.Player_Number)
           .Catches_Number + 1;
         Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Catches_Number :=
           Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Catches_Number + 1;
      else
         -- create the log message
         Message := Player_Name & " (" & Team_Name & ","
           & Players_Characteristics(Event.Player_Id.Team_Id,
                                     Event.Player_Id.Player_Number)
           .Shirt_Number'Img & ") attempts to catch the ball but fails.";
      end if;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;
end Events.Catch;
