with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Status; use Status;
with Utils;
with Billboard_Output; use Billboard_Output;

package body Events.Goal is
   procedure Process(Event : in Goal_Event_T) is
      Message : Unbounded_String;
      Player_Name : constant Unbounded_String :=
        Players_Characteristics(Event.Player_Id.Team_Id,
                                Event.Player_Id.Player_Number).Name;
      Player_Team_Name : constant Unbounded_String :=
        Teams_Status(Event.Player_Id.Team_Id).Get_Name;
      Team_Name : constant Unbounded_String :=
        Teams_Status(Event.Team_Id).Get_Name;
   begin
      -- check if the goal is an autogoal
      if Event.Is_Autogoal then
         -- create the log message
         Message := "Referee assigns an own goal scored by " & Player_Name & " ("
           & Player_Team_Name & ","
           & Players_Characteristics(Event.Player_Id.Team_Id,
                                     Event.Player_Id.Player_Number)
           .Shirt_Number'Img & ") to " & Team_Name & ".";
      else
         -- create the log message
         Message := "Referee assigns a goal to " & Player_Name & " ("
           & Player_Team_Name & ","
           & Players_Characteristics(Event.Player_Id.Team_Id,
                                     Event.Player_Id.Player_Number)
           .Shirt_Number'Img & ") of " & Team_Name & ".";
         -- update the player #of goals
         Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                       Event.Player_Id.Player_Number)
           .Goals_Number := Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                                          Event.Player_Id.Player_Number)
           .Goals_Number + 1;
      end if;
      -- update the team score
      Statistics.Teams_Statistics(Event.Team_Id).Score :=
        Statistics.Teams_Statistics(Event.Team_Id).Score + 1;
      -- test:
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;
end Events.Goal;
