with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Status; use Status;
with Utils;
with Billboard_Output; use Billboard_Output;

package body Events.Kick is
   procedure Process(Event : in Kick_Off_Event_T) is
      Message : Unbounded_String;
      Team_Name : constant Unbounded_String := Teams_Status(Event.Team_Id).Get_Name;
   begin
      -- create the log message
      Message := "Referee assigns a kick-off to " & Team_Name & ".";
      -- update the team statistics (#of kick-off)
      Statistics.Teams_Statistics(Event.Team_Id).Kick_Off_Number :=
        Statistics.Teams_Statistics(Event.Team_Id).Kick_Off_Number + 1;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;

   procedure Process(Event : in Goal_Kick_Event_T) is
      Message : Unbounded_String;
      Team_Name : constant Unbounded_String := Teams_Status(Event.Team_Id).Get_Name;
   begin
      -- create the log message
      Message := "Referee assigns a goal kick to " & Team_Name & ".";
      -- update the team statistics (#of goal kick)
      Statistics.Teams_Statistics(Event.Team_Id).Goal_Kicks_Number :=
        Statistics.Teams_Statistics(Event.Team_Id).Goal_Kicks_Number + 1;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;

   procedure Process(Event : in Corner_Kick_Event_T) is
      Message : Unbounded_String;
      Team_Name : constant Unbounded_String := Teams_Status(Event.Team_Id).Get_Name;
   begin
      -- create the log message
      Message := "Referee assigns a corner kick to " & Team_Name & ".";
      -- update the team statistics (#of corner kick)
      Statistics.Teams_Statistics(Event.Team_Id).Corner_Kicks_Number :=
        Statistics.Teams_Statistics(Event.Team_Id).Corner_Kicks_Number + 1;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;

   procedure Process(Event : in Throw_In_Event_T) is
      Message : Unbounded_String;
      Team_Name : constant Unbounded_String := Teams_Status(Event.Team_Id).Get_Name;
   begin
      -- create the log message
      Message := "Referee assigns a throw in to " & Team_Name & ".";
      -- update the team statistics (#of throw in)
      Statistics.Teams_Statistics(Event.Team_Id).Throw_In_Number :=
        Statistics.Teams_Statistics(Event.Team_Id).Throw_In_Number + 1;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;

   procedure Process(Event : in Free_Kick_Event_T) is
      Message : Unbounded_String;
      Team_Name : constant Unbounded_String := Teams_Status(Event.Team_Id).Get_Name;
   begin
      -- create the log message
      Message := "Referee assigns a free kick to " & Team_Name & ".";
      -- update the team statistics (#of free kicks)
      Statistics.Teams_Statistics(Event.Team_Id).Free_Kicks_Number :=
        Statistics.Teams_Statistics(Event.Team_Id).Free_Kicks_Number + 1;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;

   procedure Process(Event : in Penalty_Kick_Event_T) is
      Message : Unbounded_String;
      Team_Name : constant Unbounded_String := Teams_Status(Event.Team_Id).Get_Name;
   begin
      -- create the log message
      Message := "Referee assigns a penalty kick to " & Team_Name & ".";
      -- update the team statistics (#of penalty kicks)
      Statistics.Teams_Statistics(Event.Team_Id).Penalty_Kicks_Number :=
        Statistics.Teams_Statistics(Event.Team_Id).Penalty_Kicks_Number + 1;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;
end Events.Kick;
