package Events.Kick is
   -- an abstract subtype of Event_T to give a root-type for kick-like events
   type Kick_Event_T is abstract new Event_T with record
      -- the identifier of the team which has to realize the kick
      Team_Id : Team_Id_T;
   end record;

   -- a subtype of Kick_Event_T to implement the kick-off event type
   type Kick_Off_Event_T is new Kick_Event_T with null record;
   -- overriding primitive operation Process to process a kick-off event
   procedure Process(Event : in Kick_Off_Event_T);

   -- a subtype of Kick_Event_T to implement the goal kick event type
   type Goal_Kick_Event_T is new Kick_Event_T with null record;
   -- overriding primitive operation Process to process a goal kick event
   procedure Process(Event : in Goal_Kick_Event_T);

   -- a subtype of Kick_Event_T to implement the corner kick event type
   type Corner_Kick_Event_T is new Kick_Event_T with null record;
   -- overriding primitive operation Process to process a corner kick event
   procedure Process(Event : in Corner_Kick_Event_T);

   -- a subtype of Kick_Event_T to implement the throw in event type
   type Throw_In_Event_T is new Kick_Event_T with null record;
   -- overriding primitive operation Process to process a throw in event
   procedure Process(Event : in Throw_In_Event_T);

   -- a subtype of Kick_Event_T to implement the free kick event type
   type Free_Kick_Event_T is new Kick_Event_T with null record;
   -- overriding primitive operation Process to process a free kick event
   procedure Process(Event : in Free_Kick_Event_T);

   -- a subtype of Kick_Event_T to implement the penalty kick event type
   type Penalty_Kick_Event_T is new Kick_Event_T with null record;
   -- overriding primitive operation Process to process a penalty kick event
   procedure Process(Event : in Penalty_Kick_Event_T);
end Events.Kick;
