with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Status; use Status;
with Utils;
with Billboard_Output; use Billboard_Output;

package body Events.Penalty_Card is
   procedure Process(Event : in Penalty_Card_Event_T) is
      Message : Unbounded_String;
      Player_Name : constant Unbounded_String :=
        Players_Characteristics(Event.Player_Id.Team_Id,
                                Event.Player_Id.Player_Number).Name;
      Team_Name : constant Unbounded_String :=
        Teams_Status(Event.Player_Id.Team_Id).Get_Name;
   begin
      case Event.Category is
         when First_Yellow =>
            -- create the log message
            Message := "Referee assigns a yellow card to "
              & Player_Name & " (" & Team_Name & ","
              & Players_Characteristics(Event.Player_Id.Team_Id,
                                        Event.Player_Id.Player_Number)
              .Shirt_Number'Img & ").";
            -- update player and team statistics
            Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                          Event.Player_Id.Player_Number)
              .Yellow_Cards_Number := Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                                                    Event.Player_Id.Player_Number)
              .Yellow_Cards_Number + 1;
            Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Yellow_Cards_Number :=
              Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Yellow_Cards_Number + 1;
         when Second_Yellow =>
            -- create the log message
            Message := "Referee assigns a second yellow card to "
              & Player_Name & " (" & Team_Name & ","
              & Players_Characteristics(Event.Player_Id.Team_Id,
                                        Event.Player_Id.Player_Number)
              .Shirt_Number'Img & ") and so he has to quit the pitch.";
            -- update player and team statistics and player state
            Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                          Event.Player_Id.Player_Number)
              .Red_Cards_Number := Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                                                 Event.Player_Id.Player_Number)
              .Red_Cards_Number + 1;
            Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                          Event.Player_Id.Player_Number)
              .Current_Status := Sent_Off;
            Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Red_Cards_Number :=
              Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Red_Cards_Number + 1;
         when Red =>
            -- create the log message
            Message := "Referee assigns a red card to "
              & Player_Name & " (" & Team_Name & ","
              & Players_Characteristics(Event.Player_Id.Team_Id,
                                        Event.Player_Id.Player_Number)
              .Shirt_Number'Img & ") and so he has to quit the pitch.";
            -- update player and team statistics
            Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                          Event.Player_Id.Player_Number)
              .Red_Cards_Number := Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                                                 Event.Player_Id.Player_Number)
              .Red_Cards_Number + 1;
            Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                          Event.Player_Id.Player_Number)
              .Current_Status := Sent_Off;
            Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Red_Cards_Number :=
              Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Red_Cards_Number + 1;
      end case;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;

end Events.Penalty_Card;
