with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Status; use Status;
with Utils;
with Billboard_Output; use Billboard_Output;

package body Events.Player_Substitution is
   procedure Process(Event : in Player_Substitution_Event_T) is
      Message : Unbounded_String;
      Incoming_Player_Name : constant Unbounded_String :=
        Players_Characteristics(Event.Incoming_Player_Id.Team_Id,
                                Event.Incoming_Player_Id.Player_Number).Name;
      Outgoing_Player_Name : constant Unbounded_String :=
        Players_Characteristics(Event.Outgoing_Player_Id.Team_Id,
                                Event.Outgoing_Player_Id.Player_Number).Name;
      Team_Name : constant Unbounded_String :=
        Teams_Status(Event.Incoming_Player_Id.Team_Id).Get_Name;
   begin
      -- create the log message
      Message := Incoming_Player_Name & " (" & Team_Name & ","
        & Players_Characteristics(Event.Incoming_Player_Id.Team_Id,
                                  Event.Incoming_Player_Id.Player_Number)
        .Shirt_Number'Img & ") substitutes "
        & Outgoing_Player_Name & " (" & Team_Name & ","
        & Players_Characteristics(Event.Outgoing_Player_Id.Team_Id,
                                  Event.Outgoing_Player_Id.Player_Number)
        .Shirt_Number'Img & ").";
      -- update players status
      Statistics.Players_Statistics(Event.Incoming_Player_Id.Team_Id,
                                    Event.Incoming_Player_Id.Player_Number)
        .Current_Status := Active;
      Statistics.Players_Statistics(Event.Outgoing_Player_Id.Team_Id,
                                    Event.Outgoing_Player_Id.Player_Number)
        .Current_Status := Substitution;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;

end Events.Player_Substitution;
