with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Status; use Status;
with Utils;
with Billboard_Output; use Billboard_Output;
with Input_Types; use Input_Types;
with Ada.Text_IO; use Ada.Text_IO;

package body Events.Substitution is

   procedure Process(Event : in Substitution_Event_T) is
      Message : Unbounded_String;
      Outgoing_Player_Name : constant Unbounded_String :=
        Players_Characteristics(Event.Outgoing_Player_Id.Team_Id,
                                Event.Outgoing_Player_Id.Player_Number).Name;
      Incoming_Player_Name : constant Unbounded_String :=
        Players_Characteristics(Event.Incoming_Player_Id.Team_Id,
                                Event.Incoming_Player_Id.Player_Number).Name;
      Team_Name : constant Unbounded_String :=
        Teams_Status(Event.Incoming_Player_Id.Team_Id).Get_Name;
      Substitution_Status : Boolean;
   begin
      -- check if the referee has agreed the substitution
      if Event.Agreed then
         -- create the log message
         Message := "Referee agrees the substitution of "
           & Outgoing_Player_Name & " (" & Team_Name & ","
           & Players_Characteristics(Event.Outgoing_Player_Id.Team_Id,
                                     Event.Outgoing_Player_Id.Player_Number)
           .Shirt_Number'Img & ") with "
           & Incoming_Player_Name & " (" & Team_Name & ","
           & Players_Characteristics(Event.Incoming_Player_Id.Team_Id,
                                     Event.Incoming_Player_Id.Player_Number)
           .Shirt_Number'Img & ").";
         -- update the #of substitution for the team
         Statistics.Teams_Statistics(Event.Incoming_Player_Id.Team_Id)
           .Substitutions_Number :=
             Statistics.Teams_Statistics(Event.Incoming_Player_Id.Team_Id)
             .Substitutions_Number + 1;
         Substitution_Status := True;
      else
         -- create the log message
         Message := "Referee denies the substitution of "
           & Outgoing_Player_Name & " (" & Team_Name & ","
           & Players_Characteristics(Event.Outgoing_Player_Id.Team_Id,
                                     Event.Outgoing_Player_Id.Player_Number)
           .Shirt_Number'Img & ") with "
           & Incoming_Player_Name & " (" & Team_Name & ","
           & Players_Characteristics(Event.Incoming_Player_Id.Team_Id,
                                     Event.Incoming_Player_Id.Player_Number)
           .Shirt_Number'Img & ").";
         Substitution_Status := False;

      end if;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to player team MagagerGUI)
      case Event.Incoming_Player_Id.Team_Id is
         when Local =>
            Send_Message(To_String(Message), Output_Types.Local_Manager);
            Notify_Response_Inquiry(Destination    => localManagerGUI,
                                    Reply_Type     => "playerSubstitutionReply",
                                    Reply_Status   => Substitution_Status,
                                    Reply_Numeric_Detail => 0);
         when Visitor =>
            Send_Message(To_String(Message), Output_Types.Visitor_Manager);
            Notify_Response_Inquiry(Destination    => visitorManagerGUI,
                                    Reply_Type     => "playerSubstitutionReply",
                                    Reply_Status   => Substitution_Status,
                                    Reply_Numeric_Detail => 0);
      end case;
   end Process;
end Events.Substitution;
