with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Status; use Status;
with Utils;
with Billboard_Output; use Billboard_Output;

package body Events.Substitution_Request is
   procedure Process(Event : in Substitution_Request_Event_T) is
      Message : Unbounded_String;
      Outgoing_Player_Name : constant Unbounded_String :=
        Players_Characteristics(Event.Outgoing_Player_Id.Team_Id,
                                Event.Outgoing_Player_Id.Player_Number).Name;
      Team_Name : constant Unbounded_String :=
        Teams_Status(Event.Outgoing_Player_Id.Team_Id).Get_Name;
   begin
      -- check if the substitution request has been accepted
      if Event.Is_Accepted then
         Message := "Referee accepts substitution request for "
           & Outgoing_Player_Name & " (" & Team_Name & ","
           & Players_Characteristics(Event.Outgoing_Player_Id.Team_Id,
                                     Event.Outgoing_Player_Id.Player_Number)
           .Shirt_Number'Img & ").";
      else
         Message := "Referee denies substitution request for "
           & Outgoing_Player_Name & " (" & Team_Name & ","
           & Players_Characteristics(Event.Outgoing_Player_Id.Team_Id,
                                     Event.Outgoing_Player_Id.Player_Number)
           .Shirt_Number'Img & ").";
      end if;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to player team MagagerGUI)
      case Event.Outgoing_Player_Id.Team_Id is
         when Local =>
            Send_Message(To_String(Message), Output_Types.Local_Manager);
         when Visitor =>
            Send_Message(To_String(Message), Output_Types.Visitor_Manager);
      end case;
   end Process;
end Events.Substitution_Request;
