with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Status; use Status;
with Utils;
with Billboard_Output; use Billboard_Output;

package body Events.Tackle is
   procedure Process(Event : in Tackle_Event_T) is
      Message : Unbounded_String;
      Offending_Player_Name : constant Unbounded_String :=
        Players_Characteristics(Event.Offending_Player_Id.Team_Id,
                                Event.Offending_Player_Id.Player_Number).Name;
      Offending_Team_Name : constant Unbounded_String :=
        Teams_Status(Event.Offending_Player_Id.Team_Id).Get_Name;
      Offended_Player_Name : constant Unbounded_String :=
        Players_Characteristics(Event.Offended_Player_Id.Team_Id,
                                Event.Offended_Player_Id.Player_Number).Name;
      Offended_Team_Name : constant Unbounded_String :=
        Teams_Status(Event.Offended_Player_Id.Team_Id).Get_Name;
   begin
      case Event.Category is
         when Successful =>
            -- create the log message
            Message := Offending_Player_Name & " ("
              & Offending_Team_Name & ","
              & Players_Characteristics(Event.Offending_Player_Id.Team_Id,
                                        Event.Offending_Player_Id.Player_Number)
              .Shirt_Number'Img & ") successfully tackles "
              & Offended_Player_Name & " (" & Offended_Team_Name & ","
              & Players_Characteristics(Event.Offended_Player_Id.Team_Id,
                                        Event.Offended_Player_Id.Player_Number)
              .Shirt_Number'Img & ").";
            -- update #of tackles done in offending player and team statistics
            Statistics.Players_Statistics(Event.Offending_Player_Id.Team_Id,
                                          Event.Offending_Player_Id.Player_Number)
              .Tackles_Done_Number := Statistics.Players_Statistics(Event.Offending_Player_Id.Team_Id,
                                                                    Event.Offending_Player_Id.Player_Number)
              .Tackles_Done_Number + 1;
            Statistics.Teams_Statistics(Event.Offending_Player_Id.Team_Id).Tackles_Done_Number :=
              Statistics.Teams_Statistics(Event.Offending_Player_Id.Team_Id).Tackles_Done_Number + 1;
            -- update #of tackles suffered in offended player and team statistics
            Statistics.Players_Statistics(Event.Offended_Player_Id.Team_Id,
                                          Event.Offended_Player_Id.Player_Number)
              .Tackles_Suffered_Number := Statistics.Players_Statistics(Event.Offended_Player_Id.Team_Id,
                                                                        Event.Offended_Player_Id.Player_Number)
              .Tackles_Suffered_Number + 1;
            Statistics.Teams_Statistics(Event.Offended_Player_Id.Team_Id).Tackles_Suffered_Number :=
              Statistics.Teams_Statistics(Event.Offended_Player_Id.Team_Id).Tackles_Suffered_Number + 1;
         when Unsuccessful =>
            -- create the log message
            Message := Offending_Player_Name & " ("
              & Offending_Team_Name & ","
              & Players_Characteristics(Event.Offending_Player_Id.Team_Id,
                                        Event.Offending_Player_Id.Player_Number)
              .Shirt_Number'Img & ") tackles "
              & Offended_Player_Name & " (" & Offended_Team_Name & ","
              & Players_Characteristics(Event.Offended_Player_Id.Team_Id,
                                        Event.Offended_Player_Id.Player_Number)
              .Shirt_Number'Img & ") " & "but fails.";
         when Foul =>
            -- create the log message
            Message := Offending_Player_Name & " ("
              & Offending_Team_Name & ","
              & Players_Characteristics(Event.Offending_Player_Id.Team_Id,
                                        Event.Offending_Player_Id.Player_Number)
              .Shirt_Number'Img & ") tackles with foul "
              & Offended_Player_Name & " (" & Offended_Team_Name & ","
              & Players_Characteristics(Event.Offended_Player_Id.Team_Id,
                                        Event.Offended_Player_Id.Player_Number)
              .Shirt_Number'Img & ").";
            -- update #of tackles done in offending player and team statistics
            Statistics.Players_Statistics(Event.Offending_Player_Id.Team_Id,
                                          Event.Offending_Player_Id.Player_Number)
              .Tackles_Done_Number := Statistics.Players_Statistics(Event.Offending_Player_Id.Team_Id,
                                                                    Event.Offending_Player_Id.Player_Number)
              .Tackles_Done_Number + 1;
            Statistics.Teams_Statistics(Event.Offending_Player_Id.Team_Id).Tackles_Done_Number :=
              Statistics.Teams_Statistics(Event.Offending_Player_Id.Team_Id).Tackles_Done_Number + 1;
            -- update #of tackles suffered in offended player and team statistics
            Statistics.Players_Statistics(Event.Offended_Player_Id.Team_Id,
                                          Event.Offended_Player_Id.Player_Number)
              .Tackles_Suffered_Number := Statistics.Players_Statistics(Event.Offended_Player_Id.Team_Id,
                                                                        Event.Offended_Player_Id.Player_Number)
              .Tackles_Suffered_Number + 1;
            Statistics.Teams_Statistics(Event.Offended_Player_Id.Team_Id)
              .Tackles_Suffered_Number := Statistics.Teams_Statistics(Event.Offended_Player_Id.Team_Id)
              .Tackles_Suffered_Number + 1;
            -- update #of fouls done in offending player and team statistics
            Statistics.Players_Statistics(Event.Offending_Player_Id.Team_Id,
                                          Event.Offending_Player_Id.Player_Number)
              .Fouls_Done_Number := Statistics.Players_Statistics(Event.Offending_Player_Id.Team_Id,
                                                                  Event.Offending_Player_Id.Player_Number)
              .Fouls_Done_Number + 1;
            Statistics.Teams_Statistics(Event.Offending_Player_Id.Team_Id).Fouls_Done_Number :=
              Statistics.Teams_Statistics(Event.Offending_Player_Id.Team_Id).Fouls_Done_Number + 1;
            -- update #of fouls suffered in offended player and team statistics
            Statistics.Players_Statistics(Event.Offended_Player_Id.Team_Id,
                                          Event.Offended_Player_Id.Player_Number)
              .Fouls_Suffered_Number := Statistics.Players_Statistics(Event.Offended_Player_Id.Team_Id,
                                                                      Event.Offended_Player_Id.Player_Number)
              .Fouls_Suffered_Number + 1;
            Statistics.Teams_Statistics(Event.Offended_Player_Id.Team_Id)
              .Fouls_Suffered_Number := Statistics.Teams_Statistics(Event.Offended_Player_Id.Team_Id)
              .Fouls_Suffered_Number + 1;
      end case;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;

end Events.Tackle;
