with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Status; use Status;
with Utils;
with Billboard_Output; use Billboard_Output;

package body Events.Throw is
   procedure Process(Event : in Throw_Event_T) is
      Message : Unbounded_String;
      Player_Name : constant Unbounded_String :=
        Players_Characteristics(Event.Player_Id.Team_Id,
                                Event.Player_Id.Player_Number).Name;
      Team_Name : constant Unbounded_String :=
        Teams_Status(Event.Player_Id.Team_Id).Get_Name;
   begin
      if Event.Mode = Heading then
         -- player hit the ball with his head; create the log message
         Message := Player_Name & " (" & Team_Name
           & "," & Players_Characteristics(Event.Player_Id.Team_Id,
                                           Event.Player_Id.Player_Number)
           .Shirt_Number'Img & ") gives the ball a header.";
         -- and update #of heading for player and his team
         Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                       Event.Player_Id.Player_Number)
           .Headings_Number := Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                                             Event.Player_Id.Player_Number)
           .Headings_Number + 1;
         Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Headings_Number :=
           Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Headings_Number + 1;
         -- check if the throw is a shooting or a passing
         if Event.Category = Shooting then
            -- update #of shooting for player and his team
            Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                          Event.Player_Id.Player_Number)
              .Shootings_Number := Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                                                 Event.Player_Id.Player_Number)
              .Shootings_Number + 1;
            Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Shootings_Number :=
              Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Shootings_Number + 1;
         else
            -- update #of passing for player and his team
            Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                          Event.Player_Id.Player_Number)
              .Passings_Number := Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                                                Event.Player_Id.Player_Number)
              .Passings_Number + 1;
            Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Passings_Number :=
              Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Passings_Number + 1;
         end if;
      elsif Event.Mode = Throw_In then
         -- player throw in the ball: create the log message
         Message := Player_Name & " (" & Team_Name & ","
           & Players_Characteristics(Event.Player_Id.Team_Id,
                                     Event.Player_Id.Player_Number)
           .Shirt_Number'Img & ") throws in the ball.";
         -- and update #of passing for player and his team
         Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                       Event.Player_Id.Player_Number)
           .Passings_Number := Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                                             Event.Player_Id.Player_Number)
           .Passings_Number + 1;
         Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Passings_Number :=
           Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Passings_Number + 1;
      else
         -- player kicks the ball; check if the throw is a shooting or a passing
         if Event.Category = Shooting then
            -- create the log message
            Message := Player_Name & " (" & Team_Name & ","
              & Players_Characteristics(Event.Player_Id.Team_Id,
                                        Event.Player_Id.Player_Number)
              .Shirt_Number'Img & ") shoots.";
            -- and update the statistics #of shooting for player and his team
            Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                          Event.Player_Id.Player_Number)
              .Shootings_Number := Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                                                 Event.Player_Id.Player_Number)
              .Shootings_Number + 1;
            Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Shootings_Number :=
              Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Shootings_Number + 1;
         else
            -- create the log message
            Message := Player_Name & " (" & Team_Name & ","
              & Players_Characteristics(Event.Player_Id.Team_Id,
                                        Event.Player_Id.Player_Number)
              .Shirt_Number'Img & ") passes the ball.";
            -- and update the statistics #of passing for player and his team
            Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                          Event.Player_Id.Player_Number)
              .Passings_Number := Statistics.Players_Statistics(Event.Player_Id.Team_Id,
                                                                Event.Player_Id.Player_Number)
              .Passings_Number + 1;
            Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Passings_Number :=
              Statistics.Teams_Statistics(Event.Player_Id.Team_Id).Passings_Number + 1;
         end if;
      end if;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to SpectatorGUIs)
      Send_Message(To_String(Message), Output_Types.Spectator);
   end Process;

end Events.Throw;
