with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Utils;
with Billboard_Output; use Billboard_Output;

package body Events.Time_Out is
   procedure Process(Event : in First_Half_Time_Out_Event_T) is
      Message : Unbounded_String;
   begin
      -- check if the event corresponds to the referee whistle
      if Event.Is_Whistle then
         -- create the log message
         Message := To_Unbounded_String("Half time.");
      else
         -- create the log message
         Message := To_Unbounded_String("First half has finished: we go into extra time.");
      end if;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to all GUIs)
      Send_Message(To_String(Message), Output_Types.All_Receivers);
   end Process;

   procedure Process(Event : in Break_Time_Out_Event_T) is
      Message : constant Unbounded_String := To_Unbounded_String("Half time break has finished.");
   begin
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to all GUIs)
      Send_Message(To_String(Message), Output_Types.All_Receivers);
   end Process;

   procedure Process(Event : in Second_Half_Time_Out_Event_T) is
      Message : Unbounded_String;
   begin
      -- check if the event corresponds to the referee whistle
      if Event.Is_Whistle then
         -- create the log message
         Message := To_Unbounded_String("Referee ends the second half; match is finished.");
      else
         -- create the log message
         Message := To_Unbounded_String("Regular time has finished: we go into extra time.");
      end if;
      -- print the message
      Utils.Logger.Add_Log(To_String(Message));
      -- send message (to all GUIs)
      Send_Message(To_String(Message), Output_Types.All_Receivers);
   end Process;
end Events.Time_Out;
