with Ada.Containers.Indefinite_Doubly_Linked_Lists;
with Output_Types; use Output_Types;

package Events is
   -- a boolean to say if the Events_Manager_Task has to terminate
   Has_To_Terminate : Boolean := False;

   -- an abstract tagged type used as root event for all the simulation events
   type Event_T is abstract tagged null record;
   -- a primitive operation to process an event (by Events_Manager_Task)
   procedure Process(Event : in Event_T) is abstract;

   -- we istantiate the Doubly_Linked_Lists generic to implement a list of
   -- simulation events
   package Events_List is new Ada.Containers.Indefinite_Doubly_Linked_Lists(Event_T'Class);

   -- an access type to an event object
   type Event_Ptr_T is access Event_T'Class;

   -- a protected buffer used to store incoming events generated by simulation
   protected Events_Buffer is
      -- an entry used by events producers to add new events in the buffer
      procedure Add(New_Event : in Event_T'Class);
      -- an entry used by Events_Manager_Task (the consumer) to wait for a new event
      entry Wait_For_Event;
      -- a function to get the first element of the buffer
      function Get_First return Event_T'Class;
      -- a procedure used by Events_Manager_Task (the consumer) to delete the first
      -- event from buffer
      procedure Delete_First;
   private
      -- a doubly linked list to implement the events buffer
      Events_Buffer : Events_List.List;
   end Events_Buffer;

   -- a task to manage events generated by simulation
   task Events_Manager_Task is
      -- an entry used by main procedure to start the manager task
      entry Start;
   end Events_Manager_Task;

   -- a object to store the simulation statistics
   Statistics : Statistics_Frame_T;
end Events;
