with Position; use Position;
with Output_Types; use Output_Types;

package Field is
   -- a record type to implement an area in the game field
   type Field_Area_T is record
      Left: Integer;
      Right: Integer;
      Top: Integer;
      Bottom: Integer;
   end record;
   -- a function to say if an area contains a point
   function Contains(Area 	: in Field_Area_T;
                     Position	: in Coordinates_T'Class) return Boolean;
   -- an enum type to represent the field side
   type Side_T is (Left, Right);
   function Other_Side(My_Side: Side_T) return Side_T;

   -- predefined areas
   Pitch : constant Field_Area_T :=
     Field_Area_T'(Left => 1, Right => 104, Top => 1, Bottom => 68);
   Left_Midfield : constant Field_Area_T :=
     Field_Area_T'(Pitch.Left, Pitch.Right / 2, Pitch.Top, Pitch.Bottom);
   Right_Midfield : constant Field_Area_T :=
     Field_Area_T'(Pitch.Right / 2 + 1, Pitch.Right, Pitch.Top, Pitch.Bottom);
   Left_Goal : constant Field_Area_T :=
     Field_Area_T'(Pitch.Left - 1, Pitch.Left - 1, Pitch.Bottom / 2 - 3,
                   Pitch.Bottom / 2 + 4);
   Right_Goal : constant Field_Area_T :=
     Field_Area_T'(Pitch.Right + 1, Pitch.Right + 1, Pitch.Bottom / 2 - 3,
                   Pitch.Bottom / 2 + 4);
   Left_Goal_Area : constant Field_Area_T :=
     Field_Area_T'(Pitch.Left, Right =>  Pitch.Left + 5,
                   Top => Pitch.Bottom / 2 - 9,
                   Bottom => Pitch.Bottom / 2 + 10);
   Right_Goal_Area : constant Field_Area_T :=
     Field_Area_T'(Left => Pitch.Right - 5, Right =>  Pitch.Right,
                   Top => Pitch.Bottom / 2 - 9,
                   Bottom => Pitch.Bottom / 2 + 10);
   Goal_Area : array (Side_T) of Field_Area_T :=
     (Left_Goal_Area, Right_Goal_Area);
   Left_Penalty_Area : constant Field_Area_T :=
     Field_Area_T'(Pitch.Left, Right =>  Pitch.Left + 17,
                   Top => Pitch.Bottom / 2 - 21,
                   Bottom => Pitch.Bottom / 2 + 22);
   Right_Penalty_Area : constant Field_Area_T :=
     Field_Area_T'(Left => Pitch.Right - 17, Right =>  Pitch.Right,
                   Top => Pitch.Bottom / 2 - 21,
                   Bottom => Pitch.Bottom / 2 + 22);
   Penalty_Area	: array (Side_T) of Field_Area_T :=
     (Left_Penalty_Area, Right_Penalty_Area);

   -- goals
   Left_Goal_X : constant Natural := Pitch.Left - 1;
   Right_Goal_X : constant Natural := Pitch.Right + 1;
   Goal_X: constant array (Side_T) of Natural := (Left => Left_Goal_X,
                                                  Right => Right_Goal_X);
   Goal_Y_Min : constant Natural := Pitch.Bottom/2 - 3;
   Goal_Y_Max : constant Natural := Pitch.Bottom/2 + 4;
   Crossbar_Height : constant Float := 2.44; -- 2.44 m

   -- lines
   Left_Goal_Line : constant X_Range_T	:= Pitch.Left;
   Right_Goal_Line : constant X_Range_T := Pitch.Right;
   Top_Touchline : constant Y_Range_T := Pitch.Top;
   Bottom_Touchline : constant Y_Range_T := Pitch.Bottom;

   -- corners
   Top_Left_Corner: constant Coordinates_T
     := Coordinates_T'(Pitch.Left, Pitch.Top);
   Bottom_Left_Corner: constant Coordinates_T
     := Coordinates_T'(Pitch.Left, Pitch.Bottom);
   Top_Right_Corner: constant Coordinates_T
     := Coordinates_T'(Pitch.Right, Pitch.Top);
   Bottom_Right_Corner: constant Coordinates_T
     := Coordinates_T'(Pitch.Right, Pitch.Bottom);

   -- penalty spots
   Left_Penalty_Spot : Coordinates_T := Coordinates_T'(11, Bottom_Touchline / 2);
   Right_Penalty_Spot : Coordinates_T :=
     Coordinates_T'(Right_Goal_Line-10, Bottom_Touchline / 2);

   -- midfield
   Midfield_Line: constant X_Range_T := Right_Goal_Line / 2;
   Midfield : constant array (Side_T) of Coordinates_T :=
     (Left => Coordinates_T'(Midfield_Line, Bottom_Touchline / 2 + 1),
      Right => Coordinates_T'(Midfield_Line+1, Bottom_Touchline / 2));

   -- a protected type to store the current position of a player
   protected type Position_T is
      -- getter and setter
      function Get return Coordinates_T;
      procedure Set(New_X : Integer; New_Y : Integer);
   private
      Pos : Coordinates_T;
   end Position_T;
end Field;
