package body Player is

   function Opposite_Team (Team : Team_Id_T) return Team_Id_T is
   begin
      if Team = Local then
         return Visitor;
      else
         return Local;
      end if;
   end;

   ---------------------
   -- Player_Status_T --
   ---------------------
   protected body Player_Status_T is

      -- a procedure to decrease the player fitness value
      procedure Decrease_Fitness_Value (Damage		: in Float;
                                        With_Foul	: in Boolean;
                                        Caused_Injury 	: out Boolean;
                                        New_Fitness	: out Float) is
      begin
         if not With_Foul then
            -- if tackle is without foul and the fitness decrease would lead to
            -- an injury, ignore it.
            if Fitness - Damage > 1.0 then
               Fitness := Fitness - Damage;
            end if;
         else
            -- Do not decrease the fitness below zero
            Fitness := Float'Max(0.0, Fitness - Damage);
         end if;

         -- change player status if he is now injured
         if Fitness = 0.0 then
            State := Injured;
            Caused_Injury := True;
         else
            Caused_Injury := False;
         end if;
         New_Fitness := Fitness;
      end Decrease_Fitness_Value;

      procedure Decrease_Fitness_Value(Damage 	 : in Float;
                                       New_Fitness : out Float) is
         Caused_Injury : Boolean;
      begin
         Decrease_Fitness_Value(Damage        => Damage,
                                With_Foul     => False,
                                Caused_Injury => Caused_Injury,
                                New_Fitness   => New_Fitness);
      end;

      procedure Substitute (Substituted_Id : in Player_Number_T) is
      begin
         State := Substitution;
         Substituted_By := Substituted_Id;
      end Substitute;

      procedure Set_State (New_State: in Player_State_T) is
      begin
         State := New_State;
      end Set_State;

      procedure Set_Role(New_Role: in Player_Index_T) is
      begin
         Role := New_Role;
      end Set_Role;

      function Get_Fitness_Value return Float is
      begin
         return Fitness;
      end Get_Fitness_Value;

      function Get_State return Player_State_T is
      begin
         return State;
      end Get_State;

      function Get_Role return Player_Index_T is
      begin
         return Role;
      end Get_Role;

      entry Wait_For_Substitution when State = Substitution is
      begin
         null;
      end;

      function Incoming_Substitute return Player_Number_T is
      begin
         return Substituted_By;
      end;

   end Player_Status_T;

   protected body Player_Mapper_T is
      procedure Set_Numbers (New_Player_Number : in Player_Number_T;
                             New_Shirt_Number  : in Positive) is
      begin
         Player_Number := New_Player_Number;
         Shirt_Number  := New_Shirt_Number;
      end Set_Numbers;

      procedure Set_Position (New_Position : in Coordinates_T) is
      begin
         Position := New_Position;
      end Set_Position;

      function Get_Player_Number return Player_Number_T is
      begin
         return Player_Number;
      end Get_Player_Number;

      function Get_Shirt_Number return Positive is
      begin
         return Shirt_Number;
      end Get_Shirt_Number;

      function Get_Position return Coordinates_T is
      begin
         return Position;
      end Get_Position;
   end Player_Mapper_T;


end Player;
