with Player; use Player;
with Ball;
with Ada.Real_Time; use Ada.Real_Time;
with Output_Types; use Output_Types;

package Referee is
   -- a record type to represent a substitution request
   type Substitution_Request_T is record
      Team_Id 			: Team_Id_T;
      Outgoing_Shirt_Number 	: Positive;
      Incoming_Shirt_Number 	: Positive;
   end record;

   -- a task to implement the referee
   task Referee_Task is
      -- an entry to init the match
      entry Game_Configuration (New_Period_Duration          : in Time_Span;
                                New_Half_Time_Break_Duration : in Time_Span;
                                New_Substitutions_Limit      : in Positive);
      -- an entry to handle foul interruption
      entry Foul (Player_Id 		: in Player.Identifier_T;
                  Player_Position	: in Coordinates_T;
                  Tackle_Damage 	: in Float;
                  Caused_Injury 	: in Boolean);
      -- an entry to handle throw in interruption
      entry Throw_In (Ball_Status : in Ball.Ball_Status_T);
      -- an entry to handle goal kick interruption
      -- (Team_Id is the id of the team which has sent off the ball)
      entry Goal_Kick (Team_Id : in Team_Id_T);
      -- an entry to handle corner kick interruption
      entry Corner_Kick (Ball_Status : in Ball.Ball_Status_T);
      -- an entry to handle goal interruption; Player_Id is the id of the player
      -- which has made the goal/autogoal, Team_Id is the id of the player which
      -- has made/gained the goal
      entry Goal (Player_Id : in Identifier_T;
                  Team_Id : in Team_Id_T;
                  Is_Autogoal : in Boolean);
      -- an entry to handle the second-half start request
      entry End_Half_Time_Break;
      -- an entry to handle a substitution request
      entry Substitution (Substitution_Request 	: in Substitution_Request_T);
      -- an entry to handle the abort match request
      entry Abort_Match;
   end Referee_Task;
end Referee;
