with Player; use Player;
with Player.Logic; use Player.Logic;
with Status; use Status;
with Ada.Real_Time; use Ada.Real_Time;
with Referee; use Referee;
with Television;
with Distribution_Control; use Distribution_Control;
with Configuration; use Configuration;
with Output_Types; use Output_Types;
with Ada.Text_IO; use Ada.Text_IO;
with Core_Listener; use Core_Listener;
with Events; use Events;

procedure soccer_simulator is
   A1 : Player_Task_T(Local, 1);   	A2 : Player_Task_T(Local, 2);
   A3 : Player_Task_T(Local, 3);	A4 : Player_Task_T(Local, 4);
   A5 : Player_Task_T(Local, 5);	A6 : Player_Task_T(Local, 6);
   A7 : Player_Task_T(Local, 7);	A8 : Player_Task_T(Local, 8);
   A9 : Player_Task_T(Local, 9);	A10 : Player_Task_T(Local, 10);
   A11 : Player_Task_T(Local, 11);

   B1 : Player_Task_T(Visitor, 1);	B2 : Player_Task_T(Visitor, 2);
   B3 : Player_Task_T(Visitor, 3);	B4 : Player_Task_T(Visitor, 4);
   B5 : Player_Task_T(Visitor, 5);    	B6 : Player_Task_T(Visitor, 6);
   B7 : Player_Task_T(Visitor, 7);    	B8 : Player_Task_T(Visitor, 8);
   B9 : Player_Task_T(Visitor, 9);	B10 : Player_Task_T(Visitor, 10);
   B11 : Player_Task_T(Visitor, 11);

   SC: Seconds_Count; TS: Time_Span;
   Simulator_Connection_Timeout : Time;
begin
   -- set the initial time
   Split(Clock, SC, TS);
   Start_Time := Time_Of(SC+1, Time_Span_Zero);
   -- read simulation and teams configuration file
   Read_Simulation_Configuration(Simulation_Xml_String);
   Read_Teams_Configuration(Teams_Xml_String);
   -- load simulation configuration
   Load_Default_Simulation(Local_Simulation_Configuration);
   -- init the simulation configuration with the default value
   Simulation_Configuration.Init(Local_Simulation_Configuration);
   -- wait for timeout or SimulatorGUI connection
   Simulator_Connection_Timeout := Get_Corresponding_Quantum(Clock + Minutes(2));
   select
      -- wait the start signal from SimulatorGUI
      Simulation_Configuration.Wait_For_Simulation_Start;
   or
      delay until Simulator_Connection_Timeout;
      -- if delay expires go on with default values
   end select;
   -- stop receiving configuration from "distribution"
   Simulation_Configuration.Stop_Receiving;
   Teams_Configuration.Stop_Receiving;
   -- initialize teams and players configuration
   Load_Teams_Configuration;
   -- initialize player statistics
   Init_Players_Statistics(Local);
   Init_Players_Statistics(Visitor);
   -- start the event and frame manager
   Events_Manager_Task.Start;
   Television.Frame_Manager_Task.Start;
   -- get the simulation configuration and initialize the referee with it
   Local_Simulation_Configuration := Simulation_Configuration.Get;
   Referee_Task.Game_Configuration
     (Minutes(Local_Simulation_Configuration.Period_Duration),
      Minutes(Local_Simulation_Configuration.Break_Duration),
      Local_Simulation_Configuration.Substitutions_Allowed);
end soccer_simulator;
