with Field_Control;
with Assistant_Referee; use Assistant_Referee;
with Referee; use Referee;
with Television;
with Field; use Field;
with Events; use Events;
with Events.Catch; use Events.Catch;
with Events.Fitness; use Events.Fitness;
with Events.Injury; use Events.Injury;
with Events.Tackle; use Events.Tackle;
with Events.Throw; use Events.Throw;

package body Status.Manager is
   task body Manager is
      -- variables used to store information about the current calling player
      Local_Team_Id	: Team_Id_T;
      Player_Number 	: Player_Number_T;
      Local_Task_Id 	: Player_Index_T;
      Player_Position 	: Coordinates_T;
      Local_New_Position: Coordinates_T;
      -- the shirt number of the player offended in case of tackle
      Local_Offended_Player_Number : Player_Number_T;
      -- fitness values
      Fitness_Decrease	: Float := 0.0;
      Player_Fitness	: Float := 100.0;
      -- information related to the ball status
      Ball_Status	: Ball.Ball_Status_T;
      Is_Ball_Possessor	: Boolean;
      Last_Ball_Position: Coordinates_3D_T;
      Local_Motion	: Motion_3d_T;
      -- local category and mode of throw
      Local_Category 	: Events.Throw.Category_T;
      Local_Mode 	: Events.Throw.Mode_T;
      -- local natural number to store the foul severity for tackle handling
      Local_Tackle_Damage : Float;
      Local_Success	: Boolean;
      -- a boolean to store the assistant referee status
      Is_Assistant_Referee_Active : Boolean := False;
   begin
      loop
         -- wait for a player action
         select
            -- accept an update to move a player in the game field
            accept Move (Team_Id		: in Team_Id_T;
                         Task_Id	 	: in Player_Index_T;
                         New_Position 		: in Coordinates_T) do
               -- the randevouz is used only for parameter passing
               Local_Team_Id := Team_Id;
               Local_Task_Id := Task_Id;
               Local_New_Position := New_Position;
            end Move;
            Player_Number :=
              Player_Mapper(Local_Team_Id, Local_Task_Id).Get_Player_Number;
            Player_Position :=
              Player_Mapper(Local_Team_Id,Local_Task_Id).Get_Position;
            Ball_Status := Status.Ball_Status.Get;
            Is_Ball_Possessor :=
              Ball_Status.Possessor.Team_Id = Local_Team_Id
              and Ball_Status.Possessor.Player_Number = Player_Number
              and Ball_Status.Free = False;
            -- update status with the new player position
            Player_Mapper(Local_Team_Id,Local_Task_Id)
              .Set_Position(Local_New_Position);
            -- if player is the ball possessor, update the ball status
            if (Is_Ball_Possessor) then
               Status.Ball_Status.Set_Position((Local_New_Position with 0.0));
            end if;
            -- deregister the player from the field access protocol
            Field_Control.Barrier.Leave(Player_Position);
            -- Notify the new position to the frame manager
            Television.Frame_Manager_Task.Move_Player
              (Player_Team_Id => Local_Team_Id,
               Player_Task_Id => Local_Task_Id,
               New_Position   => Local_New_Position);
         or
            accept Tackle(Team_Id 		: in Team_Id_T;
                          Player_Task_Id	: in Player_Index_T;
                          Offended_Shirt	: in Player_Number_T;
                          Tackle_Damage 	: in Float;
                          Success		: in Boolean)
            do
               -- the rendez-vous is used only for parameter passing
               Local_Team_Id	:= Team_Id;
               Local_Task_Id	:= Player_Task_Id;
               Local_Offended_Player_Number := Offended_Shirt;
               Local_Tackle_Damage := Tackle_Damage;
               Local_Success	:= Success;
            end Tackle;
            Player_Position :=
              Player_Mapper(Local_Team_Id,Local_Task_Id).Get_Position;
            if Local_Success then
               -- Read all the necessary data
               Player_Number :=
                 Player_Mapper(Local_Team_Id,Local_Task_Id).Get_Player_Number;
               -- Update ball status
               Status.Ball_Status.Set(Position  => (Player_Position with 0.0),
                                      Possessor => (Local_Team_Id, Player_Number),
                                      Free      => False);
               declare
                  With_Foul 	: constant Boolean
                    := Local_Tackle_Damage > Foul_Limit;
                  Caused_Injury	: Boolean;
                  New_Fitness_Value 	: Float;
                  Fitness_Decrease	: Float;
               begin
                  -- *** Decrease fitness of the offended player ***
                  -- NB: if there is an injury, this procedure set the injury status
                  -- in player status!
                  Fitness_Decrease := Local_Tackle_Damage *
                    (1.0 - Players_Characteristics
                       (Player.Opposite_Team(Local_Team_Id),
                        Local_Offended_Player_Number)
                     .Stamina);
                  Players_Status(Player.Opposite_Team(Local_Team_Id),
                                 Local_Offended_Player_Number)
                    .Decrease_Fitness_Value(Fitness_Decrease,
                                            With_Foul,
                                            Caused_Injury,
                                            New_Fitness_Value);
                  Events_Buffer.Add(Fitness_Event_T'
                      (Player_Id => (Player.Opposite_Team(Local_Team_Id),
                                     Local_Offended_Player_Number),
                       Value     => New_Fitness_Value));
                  -- Disable the role of the injured player so that it will not
                  -- be selected as the one who should restart the game
                  if Caused_Injury then
                     Teams_Status(Player.Opposite_Team(Local_Team_Id))
                       .Set_Role_Status
                         (Players_Status(Player.Opposite_Team(Local_Team_Id),
                          Local_Offended_Player_Number).Get_Role,
                          False);
                     -- create the injury event and add it to the buffer
                     Events_Buffer.Add(Injury_Event_T'
                         (Player_Id => (Player.Opposite_Team(Local_Team_Id),
                                        Local_Offended_Player_Number)));
                  end if;
                  -- if tackle is without foul, unregister the player from access
                  -- protocol; otherwise notify the referee
                  if not With_Foul then
                     Field_Control.Barrier.Leave(Player_Position);
                     -- create the tackle (successful) event and push it into the buffer
                     Events_Buffer.Add(Tackle_Event_T'(
                       Offending_Player_Id => (Local_Team_Id,Player_Number),
                       Offended_Player_Id  =>
                         (Player.Opposite_Team(Local_Team_Id),
                          Local_Offended_Player_Number),
                       Category            => Successful));
                  else
                     -- create the tackle (foul) event and push it into the buffer
                     Events_Buffer.Add(Tackle_Event_T'(
                       Offending_Player_Id => (Local_Team_Id,Player_Number),
                       Offended_Player_Id  =>
                         (Player.Opposite_Team(Local_Team_Id),
                          Local_Offended_Player_Number),
                       Category            => Foul));
                     -- notify the referee
                     Referee.Referee_Task.Foul((Local_Team_Id,Player_Number),
                                               Player_Position,
                                               Local_Tackle_Damage,
                                               Caused_Injury);
                  end if;
               end;
            else
               Field_Control.Barrier.Leave(Player_Position);
               -- create the tackle (unsuccessful) event and push it into the buffer
               Events_Buffer.Add(Tackle_Event_T'(
                 Offending_Player_Id => (Local_Team_Id,Player_Number),
                 Offended_Player_Id  =>
                   (Player.Opposite_Team(Local_Team_Id),
                    Local_Offended_Player_Number),
                 Category            => Unsuccessful));
            end if;
         or
            accept Catch (Team_Id 	: in Team_Id_T;
                          Player_Task_Id: in Player_Index_T;
                          Success	: in Boolean) do
               -- the randevouz is used only for parameter passing
               Local_Team_Id := Team_Id;
               Local_Task_Id := Player_Task_Id;
               Local_Success := Success;
            end Catch;
            -- Read all the necessary data
            Player_Position :=
              Player_Mapper(Local_Team_Id, Local_Task_Id).Get_Position;
            Player_Number :=
              Player_Mapper(Local_Team_Id, Local_Task_Id).Get_Player_Number;
            Ball_Status := Status.Ball_Status.Get;
            if Local_Success then
               -- set the new ball status
               Status.Ball_Status.Set(Position  => Ball_Status.Motion.Get_Current_Position,
                                      Possessor => (Local_Team_Id, Player_Number),
                                      Free      => False);
               -- deregister the player from the field access protocol
               Field_Control.Barrier.Leave(Player_Position);
               -- if the assistant referee is active, stop it
               if Is_Assistant_Referee_Active then
                  Assistant_Referee_Task.Reset_Ball_Out_Time;
               end if;
               -- create the catch (successful) event and add it to the buffer
               Events_Buffer.Add(
                 Catch_Event_T'(
                   Player_Id  => Identifier_T'(Local_Team_Id, Player_Number),
                   Successful => True));
            else
               -- deregister the player from the field access protocol
               Field_Control.Barrier.Leave(Player_Position);
               -- create the catch (failed) event and add it to the buffer
               Events_Buffer.Add(
                 Catch_Event_T'(
                   Player_Id  => Identifier_T'(Local_Team_Id, Player_Number),
                   Successful => False));
            end if;
            -- Slightly decrease player fitness
            Fitness_Decrease := 0.1 *
              (1.0 - Players_Characteristics(Local_Team_Id, Player_Number).Stamina);
            Players_Status(Local_Team_Id, Player_Number)
              .Decrease_Fitness_Value(Fitness_Decrease, Player_Fitness);
            Events_Buffer.Add
              (Fitness_Event_T'((Local_Team_Id, Player_Number),Player_Fitness));
         or
            accept Throw (Team_Id 	: in Team_Id_T;
                          Player_Task_Id: in Player_Index_T;
                          Motion	: in Motion_3d_T;
                          Category	: in Events.Throw.Category_T;
                          Mode		: in Events.Throw.Mode_T) do
               -- the randevouz is used only for parameter passing
               Local_Team_Id := Team_Id;
               Local_Task_Id := Player_Task_Id;
               Local_Motion  := Motion;
               Local_Category := Category;
               Local_Mode := Mode;
               -- NOTE: the ball status is currently built inside the player logic,
               -- but given the new motion, that work could be done here
            end Throw;
            Player_Position :=
              Player_Mapper(Local_Team_Id, Local_Task_Id).Get_Position;
            Player_Number :=
              Player_Mapper(Local_Team_Id, Local_Task_Id).Get_Player_Number;
            -- set the new ball status
            Status.Ball_Status.Set(Motion    => Local_Motion,
                                   Possessor => (Local_Team_Id, Player_Number),
                                   Free      => True);
            -- check if the new motion can put out the ball
            Last_Ball_Position := Local_Motion.Element_At(Local_Motion.Last);
            -- check if the ball last position is out of field
            if not Field.Contains(Field.Pitch, Coordinates_T(Last_Ball_Position)) then
               -- wake up the assistant referee
               Assistant_Referee_Task.Set_Ball_Out_Time
                 (Local_Motion.Time_At(Local_Motion.Last));
               Is_Assistant_Referee_Active := True;
            end if;
            -- deregister the player from the field access protocol
            Field_Control.Barrier.Leave(Player_Position);
            -- create the throw event and push it into the buffer
            Events_Buffer.Add(Throw_Event_T'(
              Player_Id => (Local_Team_Id,Player_Number),
              Category  => Local_Category,
              Mode      => Local_Mode));
            -- Slightly decrease player fitness
            Fitness_Decrease := 0.1 *
              (1.0 - Players_Characteristics(Local_Team_Id, Player_Number).Stamina);
            Players_Status(Local_Team_Id, Player_Number)
              .Decrease_Fitness_Value(Fitness_Decrease, Player_Fitness);
            Events_Buffer.Add
              (Fitness_Event_T'((Local_Team_Id, Player_Number),Player_Fitness));
         or terminate;
         end select;
      end loop;
   end Manager;
end Status.Manager;
