with Output_Types; use Output_Types;
with Ball;
with Team;
with Player;
use Player;
with Ada.Real_Time; use Ada.Real_Time;
with Motion; use Motion;
with Ada.Containers.Ordered_Maps;

package Status is
   Start_Time: Ada.Real_Time.Time := Clock; -- := Utils.Get_Corresponding_Quantum(Clock);
   Quantum: constant Ada.Real_Time.Time_Span := Milliseconds(30);
   Foul_Limit : constant Float := 20.0;

   function Get_Corresponding_Quantum(Time_Value: in Ada.Real_Time.Time)
                                      return Ada.Real_Time.Time;

   package Shirt_To_Player_Number_Map is
     new Ada.Containers.Ordered_Maps(Key_Type     => Positive,
                                     Element_Type => Player_Number_T);

   -- a protected object (singleton) to implement the ball status
   protected Ball_Status is
      -- a procedure used to update the ball position
      procedure Set_Position(New_Position: Coordinates_3D_T);

      -- getter and setter
      procedure Set(New_Status: in Ball.Ball_Status_T);  -- "tackle", "catch"
      procedure Set(Motion: Motion_3d_T; Possessor: Identifier_T; Free: Boolean);
      procedure Set(Position: Coordinates_3D_T; Possessor: Identifier_T; Free: Boolean);
      function Get return Ball.Ball_Status_T;
   private
      Status : Ball.Ball_Status_T;
   end Ball_Status;

   -- an array of protected objects to store players dynamic infos;
   -- the array index is the player t-shirt number
   Players_Status : array (Team_Id_T, Player_Number_T)
     of Player.Player_Status_T;
   -- an array to store the players characteristics
   Players_Characteristics : array (Team_Id_T, Player_Number_T)
     of Player.Player_Characteristics_T;
   -- an array to store teams status
   Teams_Status : array (Team_Id_T) of Team.Team_Status_T;
   -- a bidimensional array to map a player task id to his t-shirt number
   -- and his current position on the game field
   Player_Mapper : array (Team_Id_T, Player_Index_T) of Player.Player_Mapper_T;
   Shirt_To_Player_Number : array (Team_Id_T) of Shirt_To_Player_Number_Map.Map;
end Status;
