with Position; use Position;

Package body Team is
   protected body Team_Status_T is
      procedure Set_Name(New_Name : in Unbounded_String) is
      begin
         Name := New_Name;
      end Set_Name;

      procedure Set_Id(New_Id : in Unbounded_String) is
      begin
         Id := New_Id;
      end Set_Id;

      procedure Set_Tactic(New_Formation : in Formation_T;
                           New_Pressing_Value : in Integer) is
      begin
         Formation := New_Formation;
         Pressing := New_Pressing_Value;
      end Set_Tactic;

      procedure Set_Side (New_Side : in Side_T) is
      begin
         Side := New_Side;
      end Set_Side;

      procedure Swap_Side is
      begin
         if Side = Left then
            Side := Right;
         else
            Side := Left;
         end if;
      end Swap_Side;

      procedure Set_Role_Status(Role: Player_Index_T; Status: Boolean) is
      begin
         Is_Active(Role) := Status;
      end Set_Role_Status;

      procedure Set_Starting_Player (State_Record: in Game_Control.State_Record_T)
      is
         First_Midfielder: constant Player_Index_T
           := 1 + Formation.Players_By_Role(Defender) + 1;
         Last_MidFielder: constant Player_Index_T
           := 1 + Formation.Players_By_Role(Defender)
           + Formation.Players_By_Role(Midfielder);
         Last : Player_Index_T	:= 11;
         Shortest_Player_Distance : Float := Float'Last;
         New_Distance : Float := Float'Last;
         Ball_Side : Side_T;
      begin
         if (State_Record.Position.X <= Field.Left_Midfield.Right) then
            Ball_Side := Left;
         else
            Ball_Side := Right;
         end if;

         if State_Record.State = Corner_Kick
           or State_Record.State = Throw_In then
            Last := Last_MidFielder;
         end if;
         -- Starting_Player initially set to 1, then changed if state /= Goal_Kick
         Starting_Player := 1;
         if not (State_Record.State = Goal_Kick) then
            case State_Record.State is
               when Kick_Off..Penalty_Kick =>
                  for Role in 2..Last loop
                     -- In case of Throw-In thrown from the the team field side,
                     -- if there's an available defender, skip any closer
                     -- midfielder or forward
                     exit when Role = First_Midfielder and Starting_Player /= 1
                       and State_Record.State = Throw_In and Side = Ball_Side;
                     -- Calculate the distance between the reference position
                     -- for the current role and the ball
                     New_Distance := Position.Distance
                       (From 	=> Reference_Position
                          (Formation     => Formation,
                           Side          => Side,
                           Role_Index    => Role,
                           State         => State_Record.State,
                           Ball_Position => State_Record.Position,
                           Pressing 	 => Pressing),
                        To 	=> Coordinates_T(State_Record.Position));
                     -- if the distance from the reference position (of the player
                     -- associated with Role) to the position in which the game
                     -- will restart is less than the shortest already found
                     if New_Distance < Shortest_Player_Distance
                       and Is_Active(Role)
                     then
                        Shortest_Player_Distance := New_Distance;
                        Starting_Player := Role;
                     end if;
                  end loop;
               when others => -- Play, Half_Match, End_Match, Stopped
                  Starting_Player := 1;
            end case;
         end if;
      end Set_Starting_Player;

      function Get_Name return Unbounded_String is
      begin
         return Name;
      end Get_Name;

      function Get_Id return Unbounded_String is
      begin
         return Id;
      end Get_Id;

      function Get_Formation return Formation_T is
      begin
         return Formation;
      end Get_Formation;

      function Get_Pressing return Integer is
      begin
         return Pressing;
      end Get_Pressing;

      function Get_Side return Side_T is
      begin
         return Side;
      end Get_Side;

      function Get_Starting_Player return Player_Index_T is
      begin
         return Starting_Player;
      end Get_Starting_Player;

      function Get_Role_Status(Role: Player_Index_T) return Boolean is
      begin
         return Is_Active(Role);
      end Get_Role_Status;
   end Team_Status_T;
end Team;
