with Tactic; use Tactic;
with Field; use Field;
with Game_Control; use Game_Control;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Output_Types; use Output_Types;

package Team is
   subtype Squad_Size_T is Positive range 11..25;
   type Role_Status_T is array(Player_Index_T) of Boolean;
   -- a protected object type to store team dynamic infos
   protected type Team_Status_T is
      procedure Set_Name(New_Name : in Unbounded_String);
      procedure Set_Id(New_Id : in Unbounded_String);
      --procedure Set_Formation(New_Formation : in Formation_T);
      --procedure Set_Pressing(New_Pressing : in Natural);
      procedure Set_Tactic(New_Formation : in Formation_T;
                           New_Pressing_Value : in Integer);
      procedure Set_Side(New_Side : in Side_T);
      procedure Swap_Side;
      procedure Set_Starting_Player(State_Record: in Game_Control.State_Record_T);
      procedure Set_Role_Status(Role: Player_Index_T; Status: Boolean);
      function Get_Name return Unbounded_String;
      function Get_Id return Unbounded_String;
      function Get_Formation return Formation_T;
      function Get_Pressing return Integer;
      function Get_Side return Side_T;
      function Get_Starting_Player return Player_Index_T;
      function Get_Role_Status(Role: Player_Index_T) return Boolean;
   private
      -- a reference to the current team formation
      Name	: Unbounded_String;
      Id	: Unbounded_String := Ada.Strings.Unbounded.Null_Unbounded_String;
      Formation	: Formation_T	:= F_442;	-- 4-4-2 is the default scheme
      Pressing	: Integer := 0;
      Is_Active	: Role_Status_T := (others => True);
      Side 	: Side_T;
      Starting_Player: Player_Index_T;
   end Team_Status_T;
end Team;
