with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with DOM.Core; use DOM.Core;
with Output_Types; use Output_Types;
with Distribution_Control; use Distribution_Control;

package Configuration is
   -- a procedure to read simulation configuration XML file
   procedure Read_Simulation_Configuration(Simulation_Xml_String : out Unbounded_String);

   -- a procedure to read teams configuration XML file
   procedure Read_Teams_Configuration(Teams_Xml_String : out Unbounded_String);

   -- a procedure to load the simulation configuration from XML file;
   -- this procedure load the XML file in memory and get the default configuration
   -- for period duration, break duration and substitutions allowed
   procedure Load_Default_Simulation(Configuration : out Simulation_Configuration_T);

   -- a procedure to set the player characteristics
   procedure Set_Player_Characteristics(Team_Id : in Team_Id_T;
                                        Player_Number : in Natural;
                                        Current_Player_Node : in Node);

   -- a procedure to load and init the players configuration
   procedure Load_Players(Team_Id		: in Team_Id_T;
                          Players_Configuration : in Distribution_Control.Players_Configuration_T);

   -- a procedure to load and init the players default configuration
   -- of a team (from XML file)
   -- NB: this procedure requires Teams_Status has been already initialized
   -- with the team side
   procedure Load_Default_Players(Team_Id : in Team_Id_T;
                                  Xml_Team_Id : in Unbounded_String);

   -- a procedure to load the configuration of a team from the teams configuration
   -- XML file
   procedure Load_Team(Team_Id			: in Team_Id_T;
                       Xml_Team_Id		: in Unbounded_String;
                       Teams_Configuration 	: in Distribution_Control.Teams_Configuration_T;
                       Use_Default 		: in Boolean);

   -- a procedure to load the teams configuration; it loads configurations of the
   -- teams chosen by ManagerGUI (if any), otherwise it loads the first two teams
   -- of the teams sequence in the XML file
   procedure Load_Teams_Configuration;

   -- a procedure to init players statistics
   procedure Init_Players_Statistics(Team_Id : in Team_Id_T);
end Configuration;
