with Ada.Text_IO; use Ada.Text_IO;
with Ada.Real_Time; use Ada.Real_Time;
with Position; use Position;
with Motion; use Motion;
with Status;
with Field_Control;
with Output_Types; use Output_Types;
with Game_Control; use Game_Control;

Package body Utils is
   -- Change the value of this variable to change the display mode
   Show_Field	  : constant Boolean := False;
   Last_Log_Index : Natural := 0;

   protected body Logger is
      entry Add_Log (Text : in String) when True is
      begin
         Has_Changed := True;
         Ada.Strings.Unbounded
           .Append(Log, Ada.Strings.Unbounded.To_Unbounded_String(" | "));
         Ada.Strings.Unbounded
           .Append(Log, Ada.Strings.Unbounded.To_Unbounded_String(Text));
      end Add_Log;

      entry Read_Log (Text :    out Unbounded_String) when Add_Log'Count = 0 is
      begin
         if (Log = Ada.Strings.Unbounded.Null_Unbounded_String)
         then return;
         end if;
         -- Field_Control.Barrier.Read_ETs(ACET, WCET);
         -- Text := Ada.Strings.Unbounded.To_Unbounded_String("ACET = ");
         -- Ada.Strings.Unbounded.Append(Text, To_Duration(ACET)'Img);
         -- Ada.Strings.Unbounded.Append(Text, " | WCET = ");
         -- Ada.Strings.Unbounded.Append(Text, To_Duration(WCET)'Img);
         Ada.Strings.Unbounded.Append(Text, Log);
         Log:= Ada.Strings.Unbounded.Null_Unbounded_String;
         Has_Changed := False;
      end Read_Log;

      entry Print_Log when Has_Changed is
         -- ACET: Time_Span;
         -- WCET: Time_Span;
         Text: Unbounded_String := Null_Unbounded_String;
      begin
         -- Field_Control.Barrier.Read_ETs(ACET, WCET);
         -- Text := Ada.Strings.Unbounded.To_Unbounded_String("ACET = ");
         -- Ada.Strings.Unbounded.Append(Text, To_Duration(ACET)'Img);
         -- Ada.Strings.Unbounded.Append(Text, " | WCET = ");
         -- Ada.Strings.Unbounded.Append(Text, To_Duration(WCET)'Img);
         Ada.Strings.Unbounded.Append(Text, Log);
         Put_Line(To_String(Text));
         Log:= Ada.Strings.Unbounded.Null_Unbounded_String;
         Has_Changed := False;
      end Print_Log;
   end Logger;

   task body Printer is
      --type Field_T is array (Y_Range_T) of String(0..X_Range_T'Last*2);
      type Field_T is array (Y_Range_T, 0..X_Range_T'Last*2) of Character;
      Field: Field_T;
      Position: Coordinates_T;
      Ball_Position: Coordinates_3D_T;
      Log : array(1..4) of Unbounded_String;
      Last_Print_Time: Ada.Real_Time.Time := Clock;

      procedure Clear_Field is
      begin
         -- Draw the field lines...
         Field:= (0	 => (2..208 => '_', others =>' '),
                  1..13	 => (1  => '|', 105  => '|', 209  => '|', others =>' '),
                  14	 => (1  => '|', 2..34  => '_', 105  => '|',
                          176..208=>'_', 209=> '|', others => ' '),
                  15..53 => (1  => '|', 35 => '|', 105  => '|',
                             175=> '|', 209=> '|', others =>' '),
                  54	 => (1  => '|', 2..34  => '_', 35 => '|', 105  => '|',
                          175=> '|', 176..208=>'_', 209=> '|', others => ' '),
                  55..67 => (1  => '|', 105  => '|', 209  => '|', others =>' '),
                  68 => (1  => '|', 2..104=>'_', 105  => '|',
                         106..208=>'_', 209  => '|', others =>' '),
                  others =>(others =>' '));
      end Clear_Field;

      procedure Read_Positions is
         Shirt_Num : Player_Number_T;
      begin

         -- Read player positions
         Position := Status.Player_Mapper(Local,1).Get_Position;
         Field(Position.Y,Position.X * 2) := 'A';
         Position := Status.Player_Mapper(Visitor,1).Get_Position;
         Field(Position.Y,Position.X * 2) := 'B';
         Ball_Position:=
           Motion.Get_Current_Position(Status.Ball_Status.Get.Motion);
         Field(Ball_Position.Y,Ball_Position.X * 2) := 'o';
         for t in Team_Id_T loop
            for i in 2..11 loop
               Shirt_Num := Status.Player_Mapper(t,i).Get_Player_Number;
               Position := Status.Player_Mapper(t,i).Get_Position;

               Field(Position.Y,Position.X*2)
                 := Character'Val((Shirt_Num mod 10) + 48);
            end loop;
         end loop;
      end Read_Positions;

      procedure Print_Field is
      begin
         Clear_Field;
         Read_Positions;
         -- Print header (X coordinates)
         Put("   ");
         for j in X_Range_T loop
            Put(Integer'Image((j mod 100) / 10));
         end loop;
         New_Line; Put("   ");
         for j in X_Range_T loop
            Put(Integer'Image(j mod 10));
         end loop;
         New_Line;

         for i in Y_Range_T loop
            -- Print Y Coordinates
            if i < 10 then
               Put(" ");
            end if;
            Put(Integer'Image(i));  Put(" ");
            -- Print the field
            for j in 0..X_Range_T'Last*2 loop
               Put(Field(i,j));
            end loop;
            New_Line;
         end loop;
      end Print_Field;

      procedure Show_Log(With_History : Boolean := False) is
         i: Integer := Log'Last;
         New_Log : Ada.Strings.Unbounded.Unbounded_String;
      begin
         -- TODO: finalize...
         Logger.Read_Log(New_Log);
         if New_Log /= Ada.Strings.Unbounded.Null_Unbounded_String then
            Last_Log_Index := Last_Log_Index + 1;
            if With_History then
               i := Log'Last;
               while i > Log'First loop
                  Log(i) := Log(i-1);
                  i := i-1;
               end loop;
            end if;
            Log(1) := New_Log;
         end if;
         if With_History then
            for i in Log'Range loop
               if Log(i) /= Null_Unbounded_String then
                  Put("    " & Integer'Image(Last_Log_Index + 1 - i) & ") ");
                  Put_Line(Ada.Strings.Unbounded.To_String(Log(i)));
               else
                  New_Line;
               end if;
            end loop;
         else
            Put("    " & Integer'Image(Last_Log_Index) & ") ");
            Put_Line(Ada.Strings.Unbounded.To_String(Log(1)));
         end if;
      end;


   begin
      if Show_Field then
         Last_Print_Time := Clock;
         loop
            delay until Last_Print_Time + Milliseconds(250);
            Last_Print_Time := Clock;
            Read_Positions;
            Print_Field;
            Show_Log(True);
            select
               Game_Control.Game_State.Wait_The_End;
               exit;
            else
               null;
            end select;

         end loop;
      else
         select  -- until the last player leaves the field
            Game_Control.Game_State.Wait_The_End;
         then abort	-- keep printing logs
            loop
               Logger.Print_Log;
            end loop;
         end select;
         -- then keep printing logs
         -- so that other tasks can print logs and terminate
         loop
            select
               Logger.Print_Log;
            or -- until no log is received for 3 seconds
               delay 3.0;
               exit;
            end select;
         end loop;
      end if;
   end Printer;
end Utils;
