
package manager_gui;

import javax.swing.SwingWorker;
import com.inspirel.yami.Agent;
import com.inspirel.yami.Parameters;
import com.inspirel.yami.OutgoingMessage;
import com.inspirel.yami.YAMIIOException;

class Client extends SwingWorker<Void, Void> {
    
    //This Class follow Client pattern
	//asynchronous properties are extended by yami4
    private String[] args;
    private ManagerGUI manager_gui;
    private boolean is_local_team;
    private boolean exit;
	private boolean affirmative_validation;
    private Agent client_agent;
	private Parameters params;
	private OutgoingMessage message;
    private String server_address;
    private final static String[] actions = { "localManagerGUI.connect",
                        			 		"visitorManagerGUI.connect",
                        			 		"localManagerGUI.disconnect",
                        			 		"visitorManagerGUI.disconnect",
                        			 		"localManagerGUI.saveTeamConfig",
                        			 		"visitorManagerGUI.saveTeamConfig",
                        			 		"localManagerGUI.tacticChange",
                        			 		"visitorManagerGUI.tacticChange",
                        			 		"localManagerGUI.playerSubstitution",
                        			 		"visitorManagerGUI.playerSubstitution"};
    
    public Client(String billboard_input_address, ManagerGUI manager_gui) {
		this.manager_gui = manager_gui;
		this.server_address = billboard_input_address;
		exit = false;
		affirmative_validation = false;
		try {
			//prepare client agent
			client_agent = new Agent();
			params = new Parameters();
		}
		catch (YAMIIOException ex) {
			System.out.println("[client]* error: " + ex.getMessage());
		}
		System.out.println("[client]* Client is ready");
    }
    
    protected Void doInBackground() {
		System.out.println("[client]* Client started");
		manager_gui.runningSignal();
		try {
			while (!exit) {
				try {
					//try to connect to Billboard after gui opening
					if (is_local_team) {
						manager_gui.printToTextBox("Trying to connect to " +
										  "Billboard as Local Team Manager...");
						System.out.println("[client]> " +actions[0]+ " was sent");
						message = client_agent.send(server_address, "controller",
															actions[0], params);
					}
					else {
						manager_gui.printToTextBox("Trying to connect to " +
										"Billboard as Visitor Team Manager...");
						System.out.println("[client]> " +actions[1]+ " was sent");
						message = client_agent.send(server_address, "controller",
															actions[1], params);
					}
					System.out.println("[client]* Waiting vaidation response...");
					//wait at most 3 seconds for the reply
					message.waitForCompletion(3000);
					//close wait dialog
					manager_gui.closeConnectDialog();
					OutgoingMessage.MessageState state = message.getState();
					if (state == OutgoingMessage.MessageState.REPLIED) {
						System.out.println("[client]< Connected to Billboard, " +
													"ManagerGUI is allowed.");
						manager_gui.printToTextBox("Connected to Billboard, " +
													"ManagerGUI is allowed.");
						affirmative_validation = true;
						//start subscribers
						manager_gui.startSubscribers();
					}
					else
					if (state == OutgoingMessage.MessageState.REJECTED) {
						System.out.println("[client]< Rejected from Billboard");
						manager_gui.printToTextBox("Rejected from " +
													"Billboard, only one " +
													"connected instance of " +
													"Local and Visitor Manager "+
													"is allowed.");
					}
					else {
						System.out.println("[client]< Error while receiving " +
														"validation response.");
						manager_gui.printToTextBox("Error while receiving " +
														"validation response.");
						exit = true;
					}
					message.close();
					//close wait dialog
					manager_gui.closeConnectDialog();
					if (!exit) {
						synchronized (this) {
							wait();
						}
					}
				}
				catch (YAMIIOException ex) {
					System.out.println("[client]* error: " + ex.getMessage());
				}
				
				//jumped on yami exception
				//retries every 5 seconds if billboard is not available
				if (!exit)
					Thread.sleep(5000);
			}
		}
		catch (InterruptedException ex) {
			exit = true;
		}
		//deregister from Billboard on gui closure
		if (affirmative_validation) {
			try {
				if (is_local_team) {
					System.out.println("[client]> " + actions[2] + " was sent" );
					client_agent.sendOneWay(server_address, "controller",
														actions[2], params);
				}
				else {
					System.out.println("[client]> " + actions[3] + " was sent" );
					client_agent.sendOneWay(server_address, "controller",
														actions[3], params);
				}
			}
			catch (YAMIIOException ex) {
				System.out.println("[client]* error: " + ex.getMessage());
			}
		}
		
		try {
			Thread.sleep(1000);
		}
		catch (InterruptedException ex) { }
		
		System.out.println("[client]* Client terminates");
		client_agent.close();
		manager_gui.completitionSignal();
        return null;
	}
	
	void setMode(boolean is_local_team) {
		this.is_local_team = is_local_team;
	}
	
	void exitTask() {
		exit = true;
		cancel(true);
	}
	
	void saveTeamConfig(String xml_to_send) {
		manager_gui.printToTextBox("Save config signal was sent");
		params.setString("configuration", xml_to_send);
		try {
			if (is_local_team) {
				System.out.println("[client]> " + actions[4] + " was sent" );
				client_agent.sendOneWay(server_address, "controller", actions[4],
																		params);
			}
			else {
				System.out.println("[client]> " + actions[5] + " was sent" );
				client_agent.sendOneWay(server_address, "controller", actions[5],
																		params);
			}
		}
		catch (YAMIIOException ex) {
			System.out.println("[client]* error: " + ex.getMessage());
		}
	}
	
	void changeTactic(int formation_index, int pressing_value) {
		manager_gui.printToTextBox("Change tactic signal was sent");
		params.setInteger("Line_Up", formation_index);
		params.setInteger("Pressing_Level", pressing_value);
		try {
			if (is_local_team) {
				System.out.println("[client]> " +actions[6]+ " was sent" );
				client_agent.sendOneWay(server_address, "controller", actions[6],
																		params);
			}
			else {
				System.out.println("[client]> " +actions[7]+ " was sent" );
				client_agent.sendOneWay(server_address, "controller", actions[7],
																		params);
			}
		}
		catch (YAMIIOException ex) {
			System.out.println("[client]* error: " + ex.getMessage());
		}
	}
	
	void substitutePlayer(int player_in, int player_out) {
		manager_gui.printToTextBox("Substitute player signal was sent");
		params.setInteger("In_Player_Id", player_in);
		params.setInteger("Out_Player_Id", player_out);
		try {
			if (is_local_team) {
				System.out.println("[client]> " +actions[8]+ " was sent" );
				client_agent.sendOneWay(server_address, "controller", actions[8],
																		params);
			}
			else {
				System.out.println("[client]> " +actions[9]+ " was sent" );
				client_agent.sendOneWay(server_address, "controller", actions[9],
																		params);
			}
		}
		catch (YAMIIOException ex) {
			System.out.println("[client]* error: " + ex.getMessage());
		}
	}
	
}

