
package manager_gui;

import javax.swing.SwingWorker;
import com.inspirel.yami.Agent;
import com.inspirel.yami.IncomingMessage;
import com.inspirel.yami.IncomingMessageCallback;
import com.inspirel.yami.Parameters;
import com.inspirel.yami.YAMIIOException;

class EventSubscriber extends SwingWorker<Void, Void> {

 	private static ManagerGUI manager_gui;
	private String[] args;
	private boolean is_local_team;
	private String publisher_address;
	private Agent stats_subscriber_agent;
	private Parameters stats_params;
	private Agent msg_subscriber_agent;
	private Parameters msg_params;
    private final String update_object_name = "update_handler";
	
	public EventSubscriber(String billboard_output_address,
													ManagerGUI manager_gui) {
		this.manager_gui = manager_gui;
		this.publisher_address = billboard_output_address;
		try {
			//prepare callback agents
			stats_subscriber_agent = new Agent();
		    stats_params = new Parameters();
		    stats_params.setString("destination_object", update_object_name);
			msg_subscriber_agent = new Agent();
		    msg_params = new Parameters();
		    msg_params.setString("destination_object", update_object_name);
			
		}
		catch (YAMIIOException ex) {
			System.out.println("[event subscriber]* error: " + ex.getMessage());
		}
		System.out.println("[event subscriber]* EventSubscriber is ready");
	}
	
	protected Void doInBackground() {
        System.out.println("[event subscriber]* EventSubscriber started");
        manager_gui.runningSignal();
		try {
			Thread.sleep(200);
			try {
			    //try to subscribe to the players statistics channel
			    stats_subscriber_agent.registerObject(update_object_name,
		    									new StatisticHandler());
			    if (is_local_team) {
					stats_subscriber_agent.sendOneWay(publisher_address,
												"Local_Team_Players_Statistics",
													"subscribe", stats_params);
					System.out.println("[event subscriber]< Subscribed to " +
			    					"Local_Team_Players_Statistics channel");
			    }
				else {
					stats_subscriber_agent.sendOneWay(publisher_address,
											"Visitor_Team_Players_Statistics",
													"subscribe", stats_params);
					System.out.println("[event subscriber]< Subscribed to " +
			    								"Teams_Statistics channel");
				}
				manager_gui.printToTextBox("Subscribed to Players " +
			    									"Statistics Channel.");
			    //try to subscribe to players messages channel
			    msg_subscriber_agent.registerObject(update_object_name,
													  new TextMessageHandler());
			    if (is_local_team) {
					msg_subscriber_agent.sendOneWay(publisher_address,
													"Local_Messages",
													"subscribe", msg_params);
					System.out.println("[event subscriber]< Subscribed to " +
												  "Local_Messages channel");
			    }
				else {
					msg_subscriber_agent.sendOneWay(publisher_address,
													"Visitor_Messages",
													"subscribe", msg_params);
					System.out.println("[event subscriber]< Subscribed to " +
												  "Visitor_Messages channel");
				}
			    manager_gui.printToTextBox("Subscribed to Players " +
			    										"Messages Channel.");
			    //wait for gui closure and receive updates in background
	        	synchronized (this) {
					wait();
				}
			}
			catch (YAMIIOException ex) {
			    //billboard output is not available
			    System.out.println("[event subscriber]* error: " +
			    										ex.getMessage());
			}
		}
		catch (InterruptedException ex) { }
        System.out.println("[event subscriber]* EventSubscriber terminates");
        stats_subscriber_agent.unregisterObject(update_object_name);
        msg_subscriber_agent.unregisterObject(update_object_name);
        stats_subscriber_agent.close();
		msg_subscriber_agent.close();
        manager_gui.completitionSignal();
        return null;
	}
	
	void setMode(boolean is_local_team) {
		this.is_local_team = is_local_team;
	}
	
	private static class StatisticHandler implements IncomingMessageCallback {
		//Method that receive callbacks
		@Override
		public void call(IncomingMessage im) throws Exception {
			//receive message from channel
			Parameters content = im.getParameters();
			//get players stats from message
			if (manager_gui.getInGame()) {
				String status;
				int shirt_number, catches, goals, autogoals, shootings, passings,
					headings, tackles_done, tackles_suff, fouls_done, fouls_suff,
					yellow_cards, red_cards;
				double fitness;
				int[] shirt_numbers = new int[17];
				for (int i = 1; i <= 17; i++) {
					shirt_number = content.getInteger("Player" + i +
															 "_Shirt_Number");
					catches = content.getInteger("Player" + i +
															 "_Catches_Number");
					goals = content.getInteger("Player" + i + "_Goals_Number");
					autogoals = content.getInteger("Player" + i +
														   "_Autogoals_Number");
					shootings = content.getInteger("Player" + i +
														   "_Shootings_Number");
					passings = content.getInteger("Player" + i +
															"_Passings_Number");
					headings = content.getInteger("Player" + i +
															"_Headings_Number");
					tackles_done = content.getInteger("Player" + i +
														"_Tackles_Done_Number");
					tackles_suff = content.getInteger("Player" + i +
													"_Tackles_Suffered_Number");
					fouls_done = content.getInteger("Player" + i +
														  "_Fouls_Done_Number");
					fouls_suff = content.getInteger("Player" + i +
													  "_Fouls_Suffered_Number");
					yellow_cards = content.getInteger("Player" + i +
														"_Yellow_Cards_Number");
					red_cards = content.getInteger("Player" + i +
														   "_Red_Cards_Number");
					fitness = content.getDouble("Player" + i +
													  		"_Current_Fitness");
					status = content.getString("Player" + i + "_Current_Status");
				
					//updates ingame table row
					manager_gui.updateInGameTableRow(i - 1, shirt_number,
												status, catches, goals,
												autogoals, shootings,
												passings, headings,
												tackles_done, tackles_suff,
												fouls_done, fouls_suff,
												fitness, yellow_cards,
												red_cards);
					shirt_numbers[i - 1] = shirt_number;
				}
				if (manager_gui.getAdaptPlayersModel()) {
					manager_gui.updatePlayersModels(shirt_numbers);
					manager_gui.setAdaptPlayersModel(false);
				}
			}
		}
	}
	
	private static class TextMessageHandler implements IncomingMessageCallback {
		//Method that receive callbacks
		@Override
		public void call(IncomingMessage im) throws Exception {
			//receive message from channel
			Parameters content = im.getParameters();
			if (manager_gui.getInGame()) {
				//update text box with the new match fact message
				manager_gui.printToTextBox(content.getString("Message"));
			}
		}
	}

}

