
package manager_gui;

import javax.swing.SwingWorker;
import com.inspirel.yami.Agent;
import com.inspirel.yami.IncomingMessage;
import com.inspirel.yami.IncomingMessageCallback;
import com.inspirel.yami.Parameters;
import com.inspirel.yami.YAMIIOException;

class GameStatusSubscriber extends SwingWorker<Void, Void> {

 	private static ManagerGUI manager_gui;
	private String[] args;
	private String publisher_address;
	private Agent subscriber_agent;
	private Parameters params;
    private final String update_object_name = "update_handler";
	
	public GameStatusSubscriber(String billboard_output_address,
													ManagerGUI manager_gui) {
		this.manager_gui = manager_gui;
		this.publisher_address = billboard_output_address;
		try {
			//prepare callback agent
			subscriber_agent = new Agent();
			params = new Parameters();
		}
		catch (YAMIIOException ex) {
			System.out.println("[gamestatus subscriber]* error: " + ex.getMessage());
		}
		System.out.println("[gamestatus subscriber]* GameStatusSubscriber is ready");
	}
	
	protected Void doInBackground() {
        System.out.println("[gamestatus subscriber]* GameStatusSubscriber started");
        manager_gui.runningSignal();
		try {
			Thread.sleep(250);
			try {
			    // prepare subscription update callback
			    subscriber_agent.registerObject(update_object_name,
			    								new GameStatusHandler());
				// subscribe to the Frame_Producer
				Parameters params = new Parameters();
				params.setString("destination_object", update_object_name);
				subscriber_agent.sendOneWay(publisher_address,
								"Match_Status", "subscribe", params);
				System.out.println("[gamestatus subscriber]< Subscribed to " +
														"Match_Status channel");
		    	manager_gui.printToTextBox("Subscribed to Game Status Channel.");
			    //block and receive updates in background
	    		synchronized (this) {
					wait();
				}
			    subscriber_agent.close();
			}
			catch (YAMIIOException ex) {
				//billboard output is not available
			    System.out.println("[gamestatus subscriber]* error: " + ex.getMessage());
			}
        }
		catch (InterruptedException ex) { }
        System.out.println("[gamestatus subscriber]* GameStatusSubscriber terminates");
        subscriber_agent.unregisterObject(update_object_name);
        subscriber_agent.close();
        manager_gui.completitionSignal();
        return null;
	}
	
	private static class GameStatusHandler implements IncomingMessageCallback {
		//Method that receive callbacks
		@Override
		public void call(IncomingMessage im) throws Exception {
			System.out.println("[gamestatus subscriber]< Notification " +
            					"received.");
			//receive message from channel
			Parameters content = im.getParameters();
			if (manager_gui.getInGame()) {
				String match_status = content.getString("Match_Status");
			    System.out.println("\nMatch_Status: " + match_status);
				if (match_status.equals("MATCH_STARTED"))
					manager_gui.inGameButtons(true);
				else
				if (match_status.equals("MATCH_ABORTED") ||
					match_status.equals("SECOND_HALF_FINISHED")) {
						manager_gui.inGameButtons(false);
						manager_gui.getTacticDialog().setVisible(false);
						manager_gui.getSubstitutionDialog().setVisible(false);
				}
			}
		}
	}
	
}

