
package manager_gui;

import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.JSlider;
import javax.swing.JTable;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.CardLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Vector;

class Listener implements ActionListener, ChangeListener {
	
	private ManagerGUI manager_gui;
	//gui internal state
	private boolean pc_sel_in_progress;
	private int player_index_out;
	private int player_index_in;
	private int substitutions_number;
	private int team_index;
	private int formation_index;
	private int saved_formation_index;
	private int pressing_level;
	private int new_pressing_level;
	
	public Listener(ManagerGUI manager_gui) {
		this.manager_gui = manager_gui;
		pc_sel_in_progress = false;
		substitutions_number = 0;
	}
	
	public void actionPerformed(ActionEvent e) {
		//connect button action
		if ("connect".equals(e.getActionCommand())) {
			manager_gui.startClient();
		}
		else
		//next card button action
		if ("next".equals(e.getActionCommand())) {
			if (!pc_sel_in_progress) {
				JPanel cards = manager_gui.getCardsPanel();
				CardLayout cl = (CardLayout)(cards.getLayout());
				cl.next(cards);
			}
			else {
				JOptionPane.showMessageDialog(manager_gui,
						"Please complete the player's swap.",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
			}
		}
		else
		//previous card button action
		if ("back".equals(e.getActionCommand())) {
			if (!pc_sel_in_progress) {
				JPanel cards = manager_gui.getCardsPanel();
				CardLayout cl = (CardLayout)(cards.getLayout());
				cl.previous(cards);
			}
			else {
				JOptionPane.showMessageDialog(manager_gui,
							"Please complete the player's swap.",
							"Warning",
							JOptionPane.WARNING_MESSAGE);
			}
		}
		else
		//team selection action
		if ("change-team".equals(e.getActionCommand())) {
			team_index = manager_gui.getTeamsCombo().getSelectedIndex();
			manager_gui.populateTeam(team_index);
		}
		else
		//formation selection action
		if ("change-formation".equals(e.getActionCommand())) {
			formation_index = manager_gui.getFormationsCombo().
														getSelectedIndex();
			manager_gui.getSoccerField().getSoccerFieldStatus().
												  setFormation(formation_index);
			manager_gui.getPressingSlider().setValue(0);
			manager_gui.getSoccerField().repaint();
		}
		else
		//pick/release player
		if ("pc-pick-release".equals(e.getActionCommand())) {
			JTable players_table = manager_gui.getPlayersChooserTable();
			int[] selection = players_table.getSelectedRows();
			//pick player
			if (!pc_sel_in_progress) {
				if (selection.length != 0) {
					if (!manager_gui.getInGame()) {
						pc_sel_in_progress = true;
						selection[0] = players_table.
										convertRowIndexToModel(selection[0]);
						player_index_out = selection[0];
						manager_gui.getPickButton().setText("Release Player");
					}
					else {
						if (substitutions_number == 0) {
							pc_sel_in_progress = true;
							selection[0] = players_table.
										   convertRowIndexToModel(selection[0]);
							player_index_out = selection[0];
							manager_gui.getPickButton().setText("Release Player");
						}
						else {
							JOptionPane.showMessageDialog(manager_gui,
								"Multiple substitutions at the same time, " +
															" not allowed.",
								"Warning",
								JOptionPane.WARNING_MESSAGE);
						}
					}
				}
				else {
					JOptionPane.showMessageDialog(manager_gui,
						"Please select a player.",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
				}
			}
			//release player
			else {
				if (selection.length != 0) {
					pc_sel_in_progress = false;
					selection[0] = players_table.
									convertRowIndexToModel(selection[0]);
					player_index_in = selection[0];
					if (player_index_in != player_index_out) {
						if (!manager_gui.getInGame()) {
							manager_gui.swapPlayers(player_index_in,
													player_index_out);
							manager_gui.setPlayersUniform();
						}
						else {
							if (substitutions_number == 0) {
								String p_state_out = (String)manager_gui.
												getPlayersChooserTableModel().
												getValueAt(player_index_out, 2);
								String p_state_in = (String)manager_gui.
												getPlayersChooserTableModel().
												getValueAt(player_index_in, 2);
								if (p_state_out.equals("first-string") && 
									p_state_in.equals("first-string") ||
									p_state_out.equals("substitute") && 
									p_state_in.equals("substitute")) {
									JOptionPane.showMessageDialog(manager_gui,
										"This kind of substitution is " +
																"not allowed.",
										"Warning",
										JOptionPane.WARNING_MESSAGE);
										player_index_in = player_index_out;
								}
								else {
									if (p_state_out.equals("substitute")) {
										int temp = player_index_in;
										player_index_in = player_index_out;
										player_index_out = temp;
									}
									substitutions_number++;
									manager_gui.swapPlayers(player_index_in,
															player_index_out);
									manager_gui.setPlayersUniform();
								}
							}
						}
					}
					manager_gui.getPickButton().setText("Pick Player");
				}
				else {
					JOptionPane.showMessageDialog(manager_gui,
						"Please select a player to be replaced.",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
				}
			}
		}
		else
		//save configuration action
		if ("save".equals(e.getActionCommand())) {
			String xml_to_send = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
			//get selected team id
			team_index = manager_gui.getTeamsCombo().getSelectedIndex();
			Team selected_team = manager_gui.getTeams().elementAt(team_index);
			String team_id = selected_team.getId();
			//get selected formation id
			formation_index = manager_gui.getFormationsCombo().getSelectedIndex();
			saved_formation_index = formation_index;
			//get selected pressing level
			pressing_level = manager_gui.getPressingSlider().getValue();
			xml_to_send += "<team id=\""+ team_id +"\" formation_id=\""+
							(saved_formation_index + 1) +"\" pressing=\""+
							pressing_level +"\">\n";
			//get selected team players id 11+6
			Vector<Player> team_players = selected_team.getAllPlayers();
			int[] selected_players_numbers = manager_gui.getSelectedPlayers();
			for (int i = 0; i < 17; i++) {
				for (int j = 0; j < team_players.size(); j++) {
					if (selected_players_numbers[i] ==
							team_players.elementAt(j).getShirtNumber()) {
						if (i >= 0 && i <= 10)
							xml_to_send += "<player id=\""+
											team_players.elementAt(j).getId() +
											"\" role=\""+ (i + 1) +
											"\" is-first-string=\"true\" />\n";
						else
							xml_to_send += "<player id=\""+
											team_players.elementAt(j).getId() +
											"\" role=\""+
											team_players.elementAt(j).getRole() +
											"\" is-first-string=\"false\" />\n";
					}
				}
			}
			xml_to_send += "</team>";
			//send config to the core
			manager_gui.getClient().saveTeamConfig(xml_to_send);
		}
		else
		//open dialog change tactic action
		if ("change-tactic".equals(e.getActionCommand())) {
			manager_gui.getTacticDialog().setVisible(true);
		}
		else
		//close tactic dialog action
		if ("exit-tactic-dialog".equals(e.getActionCommand())) {
			closeTacticDialog();
		}
		else
		//apply in-game formation selection action
		if ("apply-tactic".equals(e.getActionCommand())) {
			//get selected formation id
			formation_index = manager_gui.getFormationsCombo().getSelectedIndex();
			//get chosen pressing level
			new_pressing_level = manager_gui.getPressingSlider().getValue();
			if (saved_formation_index == formation_index &&
				new_pressing_level == pressing_level) {
					JOptionPane.showMessageDialog(manager_gui,
						"Nothing has changed.",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
			}
			else {
				manager_gui.getTacticDialog().setEnabledButtons(false);
				//send config to the core
				manager_gui.getClient().changeTactic(formation_index + 1,
													 new_pressing_level);
			}
		}
		else
		//open dialog substitution action
		if ("substitute".equals(e.getActionCommand())) {
			manager_gui.getSubstitutionDialog().setVisible(true);
		}
		else
		//close substitution dialog action
		if ("exit-substitution-dialog".equals(e.getActionCommand())) {
			closeSubstitutionDialog();
		}
		else
		//apply in-game substitution selection action
		if ("apply-substitution".equals(e.getActionCommand())) {
			switch (substitutions_number) {
				case 0:
					JOptionPane.showMessageDialog(manager_gui,
						"Nothing has changed.",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
					break;
				case 1:
					if (!pc_sel_in_progress) {
						manager_gui.getSubstitutionDialog().
												setEnabledButtons(false);
						int p_in = ((Integer)manager_gui.getPlayersChooserTable().
									getValueAt(player_index_in, 1)).intValue();
						int p_out = ((Integer)manager_gui.getPlayersChooserTable().
									getValueAt(player_index_out, 1)).intValue();
						manager_gui.getClient().substitutePlayer(p_out, p_in);
						//restore swapped players
						manager_gui.swapPlayers(player_index_in, player_index_out);
						manager_gui.setPlayersUniform();
						manager_gui.getPlayersChooserTable().revalidate();
						manager_gui.getPlayersChooserTable().repaint();
					}
					else {
						JOptionPane.showMessageDialog(manager_gui,
							"Please complete the player's swap.",
							"Warning",
							JOptionPane.WARNING_MESSAGE);
					}
					break;
			}
		}
	}
	
	//pressing level slider actions
	public void stateChanged(ChangeEvent e) {
	 	JSlider source = (JSlider)e.getSource();
        int value = source.getValue();
        manager_gui.getSoccerField().getSoccerFieldStatus().setTeamBalance(value);
	 	manager_gui.getSoccerField().repaint();
	 }
	 
	 void closeTacticDialog() {
	 	manager_gui.getTacticDialog().setVisible(false);
		manager_gui.getFormationsCombo().setSelectedIndex(saved_formation_index);
		manager_gui.getPressingSlider().setValue(pressing_level);
	 }
	 
	 void closeSubstitutionDialog() {
	 	if (!pc_sel_in_progress) {
	 		manager_gui.getSubstitutionDialog().setVisible(false);
	 		if (substitutions_number == 1) {
				manager_gui.swapPlayers(player_index_in, player_index_out);
				manager_gui.setPlayersUniform();
				substitutions_number = 0;
				manager_gui.getPlayersChooserTable().revalidate();
				manager_gui.getPlayersChooserTable().repaint();
			}
		}
		else {
			JOptionPane.showMessageDialog(manager_gui,
				"Please complete the player's swap.",
				"Warning",
				JOptionPane.WARNING_MESSAGE);
		}
	 }
	 
	 void changeTacticAccepted() {
	 	saved_formation_index = formation_index;
		pressing_level = new_pressing_level;
	 }
	 
	 void substitutionAccepted() {
		manager_gui.substitutePlayer(player_index_in, player_index_out);
		substitutions_number = 0;
	 }
	 
	 void setInGameTactic(int saved_formation_index, int pressing_level) {
	 	this.saved_formation_index = saved_formation_index;
	 	this.pressing_level = pressing_level;
	 }
	
}

