
package manager_gui;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.swing.JTable;
import javax.swing.JSlider;
import javax.swing.ListSelectionModel;
import javax.swing.border.EmptyBorder;
import javax.swing.UIManager;
import javax.swing.SwingWorker;
import javax.swing.UnsupportedLookAndFeelException;
import java.awt.GridLayout;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.CardLayout;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.Vector;
import java.net.URI;
import java.net.URISyntaxException;

class ManagerGUI extends JPanel {
	
	private Listener listener;
	//tasks
	private Client client;
	private ResponseEventSubscriber response_event_subscriber;
	private EventSubscriber event_subscriber;
	private GameStatusSubscriber game_status_subscriber;
	
	//ui elements
    private JPanel cards_panel;
    private ConnectDialog connect_dialog;
		//team chooser
	private final static String TEAM_CHOOSER = "Team_Chooser";
	private JLabel tc_label;
	private JComboBox tc_combo;
	private JButton tc_next_button;
	private JTextArea tc_output_textarea;
		//players chooser
	private final static String PLAYERS_CHOOSER = "Players_Chooser";
	private JLabel pc_label;
	private JTable pc_table;
	private PlayersChooserTableModel pc_table_model;
	private JButton pc_pick_button;
	private JTextArea pc_output_textarea;
		//formation chooser
	private final static String FORMATION_CHOOSER = "Formation_Chooser";
	private JLabel fc_label;
	private SoccerFieldPane fc_soccer_field_pane;
	private JComboBox fc_combo;
	private JSlider fc_slider;
	private JTextArea fc_output_textarea;
	private static final int RATIO_MIN = -5;
    private static final int RATIO_MAX = 5;
    private static final int RATIO_INIT = 0;
		//in-game ct panel
	private final static String CT_PANEL = "CT_Panel";
	private JLabel ct_label;
	private JTable ct_table;
	private InGameTableModel ct_table_model;
	private JButton ct_tactic_button;
	private JButton ct_subst_button;
			//in-game dialogs
	private TacticDialog tactic_dialog;
	private SubstitutionDialog substitution_dialog;
	private JTextArea ct_output_textarea;
	
	//gui internal state
	private Vector<Team> teams;
	private boolean is_local_team;
	private boolean is_ingame = false;
	private boolean adapt_players_model = false;
	private int running_tasks = 0;
	private int completed_tasks = 0;

	public ManagerGUI(String[] args) {
		System.out.println("[manager gui]* Starting application...");
		
		//define the GUI listener
		listener = new Listener(this);
		
		//team chooser
		tc_label = new JLabel("Select Team: ");
		String[] available_teams = { };
		tc_combo = new JComboBox(available_teams);
		tc_combo.setActionCommand("change-team");
		tc_combo.addActionListener(listener);
		tc_combo.setEnabled(false);
		
		tc_next_button = new JButton("Next");
        tc_next_button.setActionCommand("next");
        tc_next_button.addActionListener(listener);
        tc_next_button.setEnabled(false);
		JPanel tc_grid_pane = new JPanel(new GridLayout(1, 0, 5, 5));
  		tc_grid_pane.add(tc_next_button);
  		JPanel tc_flow_pane = new JPanel(new FlowLayout(FlowLayout.RIGHT,
  																	40, 20));
  		tc_flow_pane.add(tc_grid_pane);
  		tc_flow_pane.setBorder(new EmptyBorder(100, 0, 0, 0));
  		
  		tc_output_textarea = new JTextArea("> Manager output:");
		tc_output_textarea.setLineWrap(true);
		tc_output_textarea.setWrapStyleWord(true);
		tc_output_textarea.setEditable(false);
		JScrollPane tc_scroll_pane_textarea = new JScrollPane(tc_output_textarea);
		
		JPanel tc_gridbag_pane = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.weightx = 1;
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0; c.gridy = 0;
		JPanel tc_grid_pane2 = new JPanel();
  		tc_grid_pane2.add(tc_label);
  		tc_grid_pane2.add(tc_combo);
		tc_gridbag_pane.add(tc_grid_pane2, c);
		c.gridx = 0; c.gridy = 1;
		tc_gridbag_pane.add(tc_flow_pane, c);
		c.weighty = 1;
		c.gridx = 0; c.gridy = 2;
		tc_gridbag_pane.add(tc_scroll_pane_textarea, c);
        
        JPanel tc_pane = new JPanel(new BorderLayout());
        tc_pane.add(tc_gridbag_pane, BorderLayout.CENTER);
        tc_pane.setBorder(new EmptyBorder(150, 20, 20, 20));
        
		//players chooser
		pc_label = new JLabel("Select Players");
		JPanel pc_flow_pane2 = new JPanel(new FlowLayout(FlowLayout.CENTER,
																	20, 20));
        pc_flow_pane2.add(pc_label);
        
        pc_table_model = new PlayersChooserTableModel();
		pc_table = new JTable(pc_table_model) {
			public boolean getScrollableTracksViewportWidth() {
				return getPreferredSize().width < getParent().getWidth();
			}
		};
        pc_table.setFillsViewportHeight(true);
        pc_table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        pc_table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JScrollPane pc_scrollpane = new JScrollPane(pc_table);
        
        pc_pick_button = new JButton("Pick Player");
        pc_pick_button.setAlignmentX(Component.CENTER_ALIGNMENT);
        pc_pick_button.setActionCommand("pc-pick-release");
        pc_pick_button.addActionListener(listener);
        JPanel pc_flow_pane3 = new JPanel(new FlowLayout(FlowLayout.LEFT,
        															40, 20));
        JPanel pc_grid_pane2 = new JPanel(new GridLayout(1, 0, 5, 5));
        pc_grid_pane2.add(pc_pick_button);
        pc_flow_pane3.add(pc_grid_pane2);
        
        JButton pc_back_button = new JButton("Back");
		pc_back_button.setActionCommand("back");
        pc_back_button.addActionListener(listener);
        JButton pc_next_button = new JButton("Next");
        pc_next_button.setActionCommand("next");
        pc_next_button.addActionListener(listener);
		JPanel pc_flow_pane = new JPanel(new FlowLayout(FlowLayout.RIGHT,
																	40, 20));
		JPanel pc_grid_pane = new JPanel(new GridLayout(1, 0, 5, 5));
  		pc_grid_pane.add(pc_back_button);
  		pc_grid_pane.add(pc_next_button);
  		pc_flow_pane.add(pc_grid_pane);
  		JPanel pc_grid_pane3 = new JPanel(new GridLayout(1, 0, 5, 5));
		pc_grid_pane3.add(pc_flow_pane3);
		pc_grid_pane3.add(pc_flow_pane);
		
		pc_output_textarea = new JTextArea("> Manager output:");
		pc_output_textarea.setLineWrap(true);
		pc_output_textarea.setWrapStyleWord(true);
		pc_output_textarea.setEditable(false);
		JScrollPane pc_scroll_pane_textarea = new JScrollPane(pc_output_textarea);
		
		JPanel pc_gridbag_pane = new JPanel(new GridBagLayout());
		c.weightx = 1; c.weighty = 2;
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0; c.gridy = 0;
		pc_gridbag_pane.add(pc_scrollpane, c);
		c.weighty = 0;
		c.gridx = 0; c.gridy = 2;
		pc_gridbag_pane.add(pc_grid_pane3, c);
		c.weighty = 1;
		c.gridx = 0; c.gridy = 3;
		pc_gridbag_pane.add(pc_scroll_pane_textarea, c);
		
		JPanel pc_pane = new JPanel(new BorderLayout());
		pc_pane.setBorder(new EmptyBorder(0, 20, 20, 20));
		pc_pane.add(pc_flow_pane2, BorderLayout.NORTH);
		pc_pane.add(pc_gridbag_pane, BorderLayout.CENTER);
        
		//formation chooser
		fc_label = new JLabel("Select Formation");
		JPanel fc_flow_pane2 = new JPanel(new FlowLayout(FlowLayout.CENTER,
																	20, 20));
        fc_flow_pane2.add(fc_label);
		
		fc_soccer_field_pane = new SoccerFieldPane();
		
		JLabel fc_formation_label = new JLabel("Formation");
		String[] available_formations = { "4-4-2", "4-3-3" };
		fc_combo = new JComboBox(available_formations);
		fc_combo.setActionCommand("change-formation");
		fc_combo.addActionListener(listener);
		JPanel fc_grid_pane2 = new JPanel(new GridLayout(0, 1, 0, 5));
		fc_grid_pane2.add(fc_formation_label);
		fc_grid_pane2.add(fc_combo);
		
		JLabel fc_pressing_level = new JLabel("Pressing Level");
		fc_slider = new JSlider(JSlider.HORIZONTAL, RATIO_MIN, RATIO_MAX,
																	RATIO_INIT);
		fc_slider.setPreferredSize(new Dimension(150, 20));
		fc_slider.addChangeListener(listener);
		JPanel fc_grid_pane3 = new JPanel(new GridLayout(0, 1, 0, 5));
		fc_grid_pane3.add(fc_pressing_level);
		fc_grid_pane3.add(fc_slider);
		
		JPanel fc_grid_pane4 = new JPanel(new GridLayout(0, 1, 0, 15));
		fc_grid_pane4.add(fc_grid_pane2);
		fc_grid_pane4.add(fc_grid_pane3);
		JPanel fc_flow_pane3 = new JPanel(new FlowLayout(FlowLayout.CENTER));
		fc_flow_pane3.add(fc_grid_pane4);
		
		JButton fc_back_button = new JButton("Back");
		fc_back_button.setActionCommand("back");
        fc_back_button.addActionListener(listener);
        JButton fc_next_button = new JButton("Save");
        fc_next_button.setActionCommand("save");
        fc_next_button.addActionListener(listener);
		JPanel fc_flow_pane = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JPanel fc_grid_pane = new JPanel(new GridLayout(1, 0, 5, 0));
  		fc_grid_pane.add(fc_back_button);
  		fc_grid_pane.add(fc_next_button);
  		fc_flow_pane.add(fc_grid_pane);
  		
  		fc_output_textarea = new JTextArea("> Manager output:");
		fc_output_textarea.setLineWrap(true);
		fc_output_textarea.setWrapStyleWord(true);
		fc_output_textarea.setEditable(false);
		JScrollPane fc_scroll_pane_textarea = new JScrollPane(fc_output_textarea);
		
		JPanel fc_gridbag_pane = new JPanel(new GridBagLayout());
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 0.9; c.weighty = 1.0;
		c.gridx = 0; c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 2;
		fc_gridbag_pane.add(fc_soccer_field_pane, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 0.1; c.weighty = 0.9;
		c.gridx = 1; c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		fc_gridbag_pane.add(fc_flow_pane3, c);
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 0.1; c.weighty = 0.1;
		c.gridx = 1; c.gridy = 1;
		c.gridwidth = 1;
		c.gridheight = 1;
		fc_gridbag_pane.add(fc_flow_pane, c);
		c.weightx = 1.0; c.weighty = 0.5;
		c.gridx = 0; c.gridy = 2;
		c.gridwidth = 2;
		c.insets = new Insets(20, 0, 0, 0);
		fc_gridbag_pane.add(fc_scroll_pane_textarea, c);
		
		JPanel fc_pane = new JPanel(new BorderLayout());
		fc_pane.setBorder(new EmptyBorder(0, 20, 20, 20));
		fc_pane.add(fc_flow_pane2, BorderLayout.NORTH);
		fc_pane.add(fc_gridbag_pane, BorderLayout.CENTER);
		
		//ct panel
		ct_label = new JLabel("CT Panel");
		JPanel ct_flow_pane2 = new JPanel(new FlowLayout(FlowLayout.CENTER,
																	20, 20));
        ct_flow_pane2.add(ct_label);
		
		ct_table_model = new InGameTableModel();
		ct_table = new JTable(ct_table_model) {
			public boolean getScrollableTracksViewportWidth() {
				return getPreferredSize().width < getParent().getWidth();
			}
		};

		ct_table.setPreferredScrollableViewportSize(new Dimension(500,
							 ct_table.getRowCount() * ct_table.getRowHeight()));
		ct_table.setFillsViewportHeight(false);
		ct_table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		ct_table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JScrollPane ct_scrollpane = new JScrollPane(ct_table);
		
		JPanel ct_gridbag_pane2 = new JPanel(new GridBagLayout());
		c.insets = new Insets(0, 0, 0, 0);
		c.gridwidth = 1;
		c.weightx = 1; c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0; c.gridy = 0;
		ct_gridbag_pane2.add(ct_flow_pane2, c);
		c.gridy = 1;
		ct_gridbag_pane2.add(ct_scrollpane, c);
		
		ct_tactic_button = new JButton("Change Tactic");
		ct_tactic_button.setActionCommand("change-tactic");
        ct_tactic_button.addActionListener(listener);
        ct_tactic_button.setEnabled(false);
        ct_subst_button = new JButton("Substitute Player");
        ct_subst_button.setActionCommand("substitute");
        ct_subst_button.addActionListener(listener);
        ct_subst_button.setEnabled(false);
        JPanel ct_flow_pane = new JPanel(new FlowLayout(FlowLayout.CENTER,
        															20, 20));
		JPanel ct_grid_pane = new JPanel(new GridLayout(1, 0, 5, 5));
  		ct_grid_pane.add(ct_tactic_button);
  		ct_grid_pane.add(ct_subst_button);
  		ct_flow_pane.add(ct_grid_pane);
  		
  		ct_output_textarea = new JTextArea("> Manager output:");
		ct_output_textarea.setLineWrap(true);
		ct_output_textarea.setWrapStyleWord(true);
		ct_output_textarea.setEditable(false);
		JScrollPane ct_scroll_pane_textarea = new JScrollPane(ct_output_textarea);
  		
  		System.out.println("[manager gui]* Listener binding complete");
  		
		JPanel ct_gridbag_pane = new JPanel(new GridBagLayout());
		c.weightx = 1; c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0; c.gridy = 0;
		ct_gridbag_pane.add(ct_flow_pane, c);
		c.fill = GridBagConstraints.BOTH;
		c.gridy = 1; c.weighty = 1;
		ct_gridbag_pane.add(ct_scroll_pane_textarea, c);
  		
		JPanel ct_pane = new JPanel(new BorderLayout());
		ct_pane.setBorder(new EmptyBorder(0, 20, 20, 20));
		ct_pane.add(ct_gridbag_pane2, BorderLayout.NORTH);
		ct_pane.add(ct_gridbag_pane, BorderLayout.CENTER);
		
		tc_pane.setName(TEAM_CHOOSER);
		pc_pane.setName(PLAYERS_CHOOSER);
		fc_pane.setName(FORMATION_CHOOSER);
		ct_pane.setName(CT_PANEL);
		
		//add cards to the card layout
		cards_panel = new JPanel(new CardLayout());
		cards_panel.add(tc_pane, TEAM_CHOOSER);
		cards_panel.add(pc_pane, PLAYERS_CHOOSER);
		cards_panel.add(fc_pane, FORMATION_CHOOSER);
		cards_panel.add(ct_pane, CT_PANEL);
		
		setLayout(new BorderLayout());
		add(cards_panel);
		
		//create ConnectDialog
		connect_dialog = new ConnectDialog(this);
		
		//create ResponseEventSubscriber
		response_event_subscriber = new ResponseEventSubscriber(args[1], this);
		
		//create EventSubscriber
		event_subscriber = new EventSubscriber(args[1], this);
		
		//create GameStatusSubscriber
		game_status_subscriber = new GameStatusSubscriber(args[1], this);
		
		//create Client
		client = new Client(args[0], this);
	}
	
	Client getClient() {
		return client;
	}
	
	JPanel getCardsPanel() {
		return cards_panel;
	}
	
	TacticDialog getTacticDialog() {
		return tactic_dialog;
	}
	
	void setTacticDialog(TacticDialog dialog) {
		tactic_dialog = dialog;
	}
	
	SubstitutionDialog getSubstitutionDialog() {
		return substitution_dialog;
	}
	
	void setSubstitutionDialog(SubstitutionDialog dialog) {
		substitution_dialog = dialog;
	}
	
	Listener getListener() {
		return listener;
	}
	
	SoccerFieldPane getSoccerField() {
		return fc_soccer_field_pane;
	}
	
	JComboBox getTeamsCombo() {
		return tc_combo;
	}
	
	JComboBox getFormationsCombo() {
		return fc_combo;
	}
	
	JSlider getPressingSlider() {
		return fc_slider;
	}
	
	JButton getPickButton() {
		return pc_pick_button;
	}
	
	JTable getPlayersChooserTable() {
		return pc_table;
	}
	
	PlayersChooserTableModel getPlayersChooserTableModel() {
		return pc_table_model;
	}
	
	boolean getInGame() {
		return is_ingame;
	}
	
	void setInGame(boolean flag) {
		this.is_ingame = flag;
	}
	
	void enableInteractions() {
		tc_combo.setEnabled(true);
		tc_next_button.setEnabled(true);
	}
	
	void inGameButtons(boolean state) {
		ct_tactic_button.setEnabled(state);
		ct_subst_button.setEnabled(state);
	}
	
	Vector<Team> getTeams() {
		return teams;
	}
	
	boolean getAdaptPlayersModel() {
		return adapt_players_model;
	}
	
	void setAdaptPlayersModel(boolean flag) {
		this.adapt_players_model = flag;
	}
	
	void setTeams(Vector<Team> teams) {
		this.teams = teams;
		for(int i = 0; i < teams.size(); i++)
			tc_combo.addItem(teams.elementAt(i).getName());
	}
	
	void populateTeam(int selected_team_index) {
		Team selected_team = teams.elementAt(selected_team_index);
		int rows = selected_team.getNumberOfPlayers();
		int cols = 19;
		pc_table_model.setData(rows, cols, selected_team);
		String team_name = selected_team.getName();
		//set team on panel labels and mode color (locals->red / visitors->blue)
		String mode;
		if (is_local_team) {
			mode = "<font color=\"red\">(LOCAL</font>";
			team_name =  "<font color=\"red\"> / " + team_name + ")</font>";
			fc_soccer_field_pane.getSoccerFieldStatus().setTeamColor(Color.red);
		}
		else {
			mode = "<font color=\"blue\">(VISITOR</font>";
			team_name =  "<font color=\"blue\"> / " + team_name + ")</font>";
			fc_soccer_field_pane.getSoccerFieldStatus().setTeamColor(Color.blue);
		}
		pc_label.setText("<html>" + mode + team_name + " Select Players</html>");
		fc_label.setText("<html>" + mode + team_name + " Select Formation</html>");
		ct_label.setText("<html>" + mode + team_name + " CT Panel</html>");
		setPlayersUniform();
		fc_combo.setSelectedIndex(selected_team.getDefaultFormation());
		int def_pressing = selected_team.getDefaultPressing();
		fc_slider.setValue(def_pressing);
		fc_soccer_field_pane.getSoccerFieldStatus().setTeamBalance(def_pressing);
		fc_soccer_field_pane.repaint();
	}
	
	void setPlayersUniform() {
		fc_soccer_field_pane.getSoccerFieldStatus().setLocalTeam(is_local_team);
		for (int i = 0; i < 17; i++) {
			//set shirt name
			fc_soccer_field_pane.getSoccerFieldStatus().setPlayerName(i,
					   ((String)pc_table_model.getValueAt(i, 0)).toUpperCase());
			//set shirt number
			fc_soccer_field_pane.getSoccerFieldStatus().setPlayerNumber(i,
									(Integer)pc_table_model.getValueAt(i, 1));	
		}
	}
	
	int[] getSelectedPlayers() {
		int[] players_number = new int[17];
		for (int i = 0; i < 17; i++) {
			players_number[i] = (Integer)pc_table_model.getValueAt(i, 1);
		}
		return players_number;
	}
	
	void populateInGameTable() {
		for (int i = 0; i < 17; i++) {
			ct_table_model.setValueAt(pc_table_model.getValueAt(i, 0), i, 0);
			ct_table_model.setValueAt(pc_table_model.getValueAt(i, 1), i, 1);
			for (int j = 2; j < 16; j++)
				ct_table_model.setValueAt("-", i, j);
		}
	}
	
	void updateInGameTableRow(int row, int shirt_number, String status,
								int catches, int goals, int autogoals,
								int shootings, int passings, int headings,
								int tackles_done, int tackles_suff,
								int fouls_done, int fouls_suff, double fitness,
								int yellow_cards, int red_cards) {
		String str_fitness = (int)fitness + " %";
		ct_table_model.setValueAt(ct_table_model.getValueAt(row, 0), row, 0);
		ct_table_model.setValueAt(shirt_number, row, 1);
		ct_table_model.setValueAt(status, row, 2);
		ct_table_model.setValueAt(catches, row, 3);
		ct_table_model.setValueAt(goals, row, 4);
		ct_table_model.setValueAt(autogoals, row, 5);
		ct_table_model.setValueAt(shootings, row, 6);
		ct_table_model.setValueAt(passings, row, 7);
		ct_table_model.setValueAt(headings, row, 8);
		ct_table_model.setValueAt(tackles_done, row, 9);
		ct_table_model.setValueAt(tackles_suff, row, 10);
		ct_table_model.setValueAt(fouls_done, row, 11);
		ct_table_model.setValueAt(fouls_suff, row, 12);
		ct_table_model.setValueAt(str_fitness, row, 13);
		ct_table_model.setValueAt(yellow_cards, row, 14);
		ct_table_model.setValueAt(red_cards, row, 15);
	}
	
	void updatePlayersModels(int[] shirt_numbers) {
		//get team index
		int team_index = tc_combo.getSelectedIndex();
		//get default players
		Vector<Player> def_players = teams.elementAt(team_index).getAllPlayers();
		//update the players table models
		for (int i = 0; i < 17; i++) {
			for (int j = 0; j < def_players.size(); j++) {
				if (shirt_numbers[i] == def_players.elementAt(j).
												getShirtNumber()) {
					ct_table_model.setValueAt(def_players.elementAt(j).
															   getName(), i, 0);
					pc_table_model.setValueAt(def_players.elementAt(j).
															   getName(), i, 0);
					pc_table_model.setValueAt(def_players.elementAt(j).
														getShirtNumber(), i, 1);
					if (i >= 0 && i < 11)
						pc_table_model.setValueAt("first-string", i, 2);
					else
						pc_table_model.setValueAt("substitute", i, 2);
					pc_table_model.setValueAt(def_players.elementAt(j).
															   getRole(), i, 3);
					pc_table_model.setValueAt(def_players.elementAt(j).
															  getSpeed(), i, 4);
					pc_table_model.setValueAt(def_players.elementAt(j).
															getStamina(), i, 5);
					pc_table_model.setValueAt(def_players.elementAt(j).
															 getHeight(), i, 6);
					pc_table_model.setValueAt(def_players.elementAt(j).
															   getJump(), i, 7);
					pc_table_model.setValueAt(def_players.elementAt(j).
														  getDribbling(), i, 8);
					pc_table_model.setValueAt(def_players.elementAt(j).
															 getTackle(), i, 9);
					pc_table_model.setValueAt(def_players.elementAt(j).
														  getCatching(), i, 10);
					pc_table_model.setValueAt(def_players.elementAt(j).
														 getPassPower(), i, 11);
					pc_table_model.setValueAt(def_players.elementAt(j).
													  getPassAccuracy(), i, 12);
					pc_table_model.setValueAt(def_players.elementAt(j).
														getShootPower(), i, 13);
					pc_table_model.setValueAt(def_players.elementAt(j).
													 getShootAccuracy(), i, 14);
					pc_table_model.setValueAt(def_players.elementAt(j).
														 getHeadPower(), i, 15);
					pc_table_model.setValueAt(def_players.elementAt(j).
													  getHeadAccuracy(), i, 16);
					pc_table_model.setValueAt(def_players.elementAt(j).
													getAggressiveness(), i, 17);
					pc_table_model.setValueAt(def_players.elementAt(j).
													 getTeamWorkSkill(), i, 18);
				}
			}
		}
	}
	
	void swapPlayers(int p1, int p2) {
		pc_table_model.swapPlayers(p1, p2);
	}
	
	void substitutePlayer(int p_index_in, int p_index_out) {
		pc_table_model.swapPlayers(p_index_in, p_index_out);
	}
	
	void closeConnectDialog() {
		connect_dialog.dispose();
	}
	
	void printToTextBox(String message) {
		tc_output_textarea.append("\n- " + message);
		pc_output_textarea.append("\n- " + message);
		fc_output_textarea.append("\n- " + message);
		ct_output_textarea.append("\n- " + message);
		int textbox_lenght = tc_output_textarea.getDocument().getLength();
		tc_output_textarea.setCaretPosition(textbox_lenght);
		pc_output_textarea.setCaretPosition(textbox_lenght);
		fc_output_textarea.setCaretPosition(textbox_lenght);
		ct_output_textarea.setCaretPosition(textbox_lenght);
	}
	
	void startClient() {
		is_local_team = connect_dialog.getSelectedMode();
		//set mode on panel labels
		String mode;
		if (is_local_team)
			mode = "<font color=\"red\">(LOCAL)</font>";
		else
			mode = "<font color=\"blue\">(VISITOR)</font>";
		tc_label.setText("<html>" + mode + " Select Team: </html>");
		pc_label.setText("<html>" + mode + " Select Players</html>");
		fc_label.setText("<html>" + mode + " Select Formation</html>");
		ct_label.setText("<html>" + mode + " CT Panel</html>");
		//start client with selected mode
		client.setMode(is_local_team);
		client.execute();
	}
	
	void startSubscribers() {
		response_event_subscriber.setMode(is_local_team);
		response_event_subscriber.execute();
		event_subscriber.setMode(is_local_team);
		event_subscriber.execute();
		game_status_subscriber.execute();
	}

	
	private void terminateClientAndSubscribers() {
		System.out.println("[manager gui]* Waiting Client and Subscribers " +
															 "termination...");
		if (response_event_subscriber.getState() ==
						SwingWorker.StateValue.STARTED) {
			response_event_subscriber.cancel(true);
		}
		else {
			System.out.println("[manager gui]* ResponseEventSubscriber is " +
															 	"not running");
		}
		if (event_subscriber.getState() ==
						SwingWorker.StateValue.STARTED) {
			event_subscriber.cancel(true);
		}
		else {
			System.out.println("[manager gui]* EventSubscriber is not " +
																 	"running");
		}
		if (game_status_subscriber.getState() == SwingWorker.StateValue.STARTED ||
			game_status_subscriber.getState() == SwingWorker.StateValue.DONE) {
			game_status_subscriber.cancel(true);
		}
		else {
			System.out.println("[spectator gui]* GameStatusSubscriber is not " +
																 	"running");
		}
		if (client.getState() == SwingWorker.StateValue.STARTED) {
			client.exitTask();
		}
		else
			System.out.println("[spectator gui]* Client is not running");
		waitCompletion();
	}
	
	synchronized void runningSignal() {
		running_tasks++;
	}
	
	synchronized void completitionSignal() {
		completed_tasks++;
		notify();
	}
	
	private synchronized void waitCompletion() {
		while (running_tasks != completed_tasks) {
			try {
				wait();
			}
			catch (InterruptedException ex) { }
		}
	}

    /*	Create the GUI and show it.  For thread safety,
    	this method should be invoked from the
		event-dispatching thread. */
    private static void createManagerGUI(String[] args) {
        //Create and set up the window.
        final JFrame frame = new JFrame("Manager GUI");
        frame.setMinimumSize(new Dimension(700, 500));
        frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.out.println("[manager gui]* Close ManagerGUI by the " +
																		"user");
				ManagerGUI pane = (ManagerGUI)frame.getContentPane();
				pane.terminateClientAndSubscribers();
				System.out.println("[manager gui]* Exiting ManagerGUI...");
				System.exit(0);
			}
		});

        //Set up the content pane.
		ManagerGUI content_pane = new ManagerGUI(args);
		content_pane.setOpaque(true); //content panes must be opaque
		frame.setContentPane(content_pane);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
        frame.setLocationRelativeTo(null);
        content_pane.connect_dialog.setVisible(true);
    }

    public static void main(String[] args) {
    	//read billboard addresses from configuration file
    	final String[] addresses = readConfigFile(args);
    	/* Use an appropriate Look and Feel */
        try {
            UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
        } catch (UnsupportedLookAndFeelException ex) {
            ex.printStackTrace();
        } catch (IllegalAccessException ex) {
            ex.printStackTrace();
        } catch (InstantiationException ex) {
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        }
        /* Turn off metal's use of bold fonts */
        UIManager.put("swing.boldMetal", Boolean.FALSE);
    	
        /*	Schedule a job for the event-dispatching thread:
        	creating and showing this application's GUI. */
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createManagerGUI(addresses);
            }
        });
    }
    
    private static String[] readConfigFile(String[] args) {
		//read the configuration file
    	String CONFIGURATION_FILE = "";
    	if (args.length != 0)
    		CONFIGURATION_FILE = args[0];
		Properties prop = new Properties();
		try{
			InputStream is = new FileInputStream(CONFIGURATION_FILE);
			prop.load(is);
		}
		catch (IOException e) {
			System.out.println("Configuration File error: "+ e.getMessage() +
								"\nThe correct configuration file path is " +
    										"required as the first argument.");
			System.exit(-1);
		}
		try {
			URI uri_in = new URI("tcp://" + prop.getProperty("BILLBOARD_INPUT"));
			if (uri_in.getHost() == null || uri_in.getPort() == -1) {
				throw new URISyntaxException(uri_in.toString(),
				"URI must have host and port parts");
			}
			URI uri_out = new URI("tcp://" + prop.getProperty("BILLBOARD_OUTPUT"));
			if (uri_out.getHost() == null || uri_out.getPort() == -1) {
				throw new URISyntaxException(uri_out.toString(),
				"URI must have host and port parts");
			}
		} catch (URISyntaxException ex) {
		// validation failed
		System.out.println("validation failed");
		System.exit(-1);
		}
    	String[] addresses = new String[2];
    	addresses[0] = "tcp://" + prop.getProperty("BILLBOARD_INPUT");
    	addresses[1] = "tcp://" + prop.getProperty("BILLBOARD_OUTPUT");
    	System.out.println("BILLBOARD_INPUT Server runs on: "+addresses[0]);
		System.out.println("BILLBOARD_OUTPUT Publisher runs on: "+addresses[1]);
		return addresses;  
    }
    
}

