
package manager_gui;

import javax.swing.JOptionPane;
import javax.swing.SwingWorker;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import java.awt.CardLayout;
import java.io.ByteArrayInputStream;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import com.inspirel.yami.Agent;
import com.inspirel.yami.IncomingMessage;
import com.inspirel.yami.IncomingMessageCallback;
import com.inspirel.yami.Parameters;
import com.inspirel.yami.YAMIIOException;

class ResponseEventSubscriber extends SwingWorker<Void, Void> {

 	private static ManagerGUI manager_gui;
	private String[] args;
	private boolean is_local_team;
	private String publisher_address;
	private Agent subscriber_agent;
	private Parameters params;
    private final String update_object_name = "update_handler";
	
	public ResponseEventSubscriber(String billboard_output_address,
													ManagerGUI manager_gui) {
		this.manager_gui = manager_gui;
		this.publisher_address = billboard_output_address;
		try {
			//prepare callback agent
			subscriber_agent = new Agent();
			params = new Parameters();
		}
		catch (YAMIIOException ex) {
			System.out.println("[responsevent subscriber]* error: " +
															ex.getMessage());
		}
		System.out.println("[responsevent subscriber]* " +
									"ResponseEventSubscriber is ready");
	}
	
	protected Void doInBackground() {
        System.out.println("[responsevent subscriber]* " +
        							"ResponseEventSubscriber started");
        manager_gui.runningSignal();
		try {
			Thread.sleep(150);
			try {
			    //try to subscribe to manager reply channel
			    subscriber_agent.registerObject(update_object_name,
		    								new ResponseEventHandler());
				params.setString("destination_object", update_object_name);
				if (is_local_team) {
					subscriber_agent.sendOneWay(publisher_address,
							"Local_Response_Inquiry", "subscribe", params);
					System.out.println("[responsevent subscriber]< Subscribed " +
									"to Local_Response_Inquiry channel");
				}
				else {
					subscriber_agent.sendOneWay(publisher_address,
							"Visitor_Response_Inquiry", "subscribe", params);
					System.out.println("[responsevent subscriber]< Subscribed " +
									"to Visitor_Response_Inquiry channel");
				}	
	    		manager_gui.printToTextBox("Subscribed to Response " +
	    												   "Event Channel.");
				//block and receive updates in background
				synchronized (this) {
					wait();
				}
				subscriber_agent.close();
			}
			catch (YAMIIOException ex) {
				//billboard output is not available
			    System.out.println("[responsevent subscriber]* error: " +
			    											   ex.getMessage());
			    manager_gui.printToTextBox("Error: Unable to find " +
		    								"Billboard Output. Please " +
		    								"start both Billboard Input " +
		    								"and Billboard Output and " +
		    								"restart this GUI.");
			}
        }
		catch (InterruptedException ex) { }
        System.out.println("[responsevent subscriber]* " +
        								"ResponseEventSubscriber terminates");
        subscriber_agent.unregisterObject(update_object_name);
        subscriber_agent.close();
        manager_gui.completitionSignal();
        return null;
	}
	
	void setMode(boolean is_local_team) {
		this.is_local_team = is_local_team;
	}
	
	private static class ResponseEventHandler implements IncomingMessageCallback {
		//Method that receive callbacks
		@Override
		public void call(IncomingMessage im) throws Exception {
			//receive message from the channel
			Parameters content = im.getParameters();
            //get the type of the reply, for which request it is
            String reply_type = content.getString("Reply_Type");
            System.out.println("[responsevent subscriber]< Notification " +
            					"received. \nReply_Type: " + reply_type);
            //reply for connect (initialization)
            if (reply_type.equals("connect")) {
            	//get the response of the reply
		    	boolean reply_status = content.getBoolean("Reply_Status");
		    	System.out.println("Reply_Status: " + reply_status);
            	if (reply_status) {
					String match_state = content.getString("Match_Status");
					String teams_xml = content.getString("Configuration_Xml");
					//before the beginning of the match
				    if (match_state.equals("WARM_UP")) {
				    	if (!teams_xml.equals("")) {	
				    		//initialize the gui
				    		parseTeamsXML(teams_xml);
				    		manager_gui.enableInteractions();
				    	}
				    	else {
				    		System.out.println("[responsevent subscriber]* " +
								   "Initialization failed due to invalid xml)");
							manager_gui.printToTextBox("Initialization failed " +
														"due to invalid xml.");
				    	}
				    }
				    //during the match
				    else {
				    	//get ingame parameters
						String team_id = content.getString("Team_Id");
						int team_formation = content.getInteger("Team_Formation");
						int pressing_level = content.getInteger("Pressing_Level");
				    	//initialize the gui
				    	parseTeamsXML(teams_xml);
				    	//apply configuration and go to ingame panel
						applyInGameConfiguration(team_id, team_formation,
				    											pressing_level);
				    }
				    System.out.println("[responsevent subscriber]< " +
												"Initialization received");	
					manager_gui.printToTextBox("GUI is initialized.");
            	}
            	else {
            		System.out.println("[responsevent subscriber]< " +
            										"Initialization failed");
            		manager_gui.printToTextBox("Initialization failed.");
            	}
            }
            else
            //reply for save config
            if (reply_type.equals("saveTeamConfig")) {
            	//get the response of the reply
		    	String reply_status = content.getString("Reply_Status");
		    	System.out.println("Reply_Status: " + reply_status);
            	if (reply_status.equals("SUCCESS")) {
            		manager_gui.printToTextBox("Save team accepted with the " +
            										"selected configuration.");
            		//fill ct table
					manager_gui.populateInGameTable();
					//create dialogs
					manager_gui.setTacticDialog(new TacticDialog(manager_gui));
					manager_gui.setSubstitutionDialog(
										   new SubstitutionDialog(manager_gui));
					//go to ingame panel
					CardLayout cl = (CardLayout)(manager_gui.getCardsPanel().
																   getLayout());
					cl.show(manager_gui.getCardsPanel(), "CT_Panel");
            		manager_gui.setInGame(true);
            		manager_gui.printToTextBox("Waiting start match signal.");
            	}
            	else
            	if (reply_status.equals("TEAM_ALREADY_CHOSEN")) {
            		manager_gui.printToTextBox("Team already chosen. Choose " +
            										"another team and retry.");
            		JOptionPane.showMessageDialog(manager_gui,
						"<html>This team has already been chosen by the other " +
						"manager.<br>Retry with another team.</html>",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
            		//go to team chooser panel
					CardLayout cl = (CardLayout)(manager_gui.getCardsPanel().
															   getLayout());
					cl.show(manager_gui.getCardsPanel(), "Team_Chooser");
            	}
            	else
            	if (reply_status.equals("CONFIGURATION_ALREADY_PRESENT")) {
            		manager_gui.printToTextBox("Configuration already present. " +
            										"Redirecting to ct panel.");
            		JOptionPane.showMessageDialog(manager_gui,
						"<html>The configuration has already been done. " +
						"<br>You will be redirected to ingame panel.</html>",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
            		//get ingame parameters
            		String team_id = content.getString("Team_Id");
					System.out.println("Team_Id: " + team_id);
					int team_formation = content.getInteger("Formation_Id");
					System.out.println("Team_Formation: " + team_formation);
					int pressing_level = content.getInteger("Pressing_Level");
					System.out.println("Pressing_Level: " + pressing_level);
					//apply configuration and go to ingame panel
					applyInGameConfiguration(team_id, team_formation,
				    											pressing_level);
            		
            	}
            	else
            	if (reply_status.equals("CONFIGURATION_NOT_ALLOWED")) {
            		manager_gui.printToTextBox("Configuration not allowed. " +
            									"Redirecting to ct panel.");
            		JOptionPane.showMessageDialog(manager_gui,
						"<html>The match has already been started. " +
						"<br>You will be redirected to ingame panel.</html>",
						"Warning",
						JOptionPane.WARNING_MESSAGE);
            		//get ingame parameters
            		String team_id = content.getString("Team_Id");
					int team_formation = content.getInteger("Formation_Id");
					int pressing_level = content.getInteger("Pressing_Level");
				    //apply configuration and go to ingame panel
				    applyInGameConfiguration(team_id, team_formation,
				    											pressing_level);
				}
            }
            else
            //reply for tactic change
            if (reply_type.equals("tacticChange")) {
            	//get the response of the reply
		    	boolean reply_status = content.getBoolean("Reply_Status");
		    	System.out.println("Reply_Status: " + reply_status);
            	if (reply_status) {
            		manager_gui.printToTextBox("Tactic change accepted.");
            		manager_gui.getListener().changeTacticAccepted();
            	}
            	else {
            		manager_gui.printToTextBox("Tactic change failed.");
            	}
            	manager_gui.getTacticDialog().setVisible(false);
            	manager_gui.getTacticDialog().setEnabledButtons(true);
            }
            else
            //reply for substitution (ack that the core has taken the request)
            if (reply_type.equals("playerSubstitutionAck")) {
            	//get the response of the reply
		    	boolean reply_status = content.getBoolean("Reply_Status");
		    	System.out.println("Reply_Status: " + reply_status);
            	if (reply_status) {
            		manager_gui.printToTextBox("Substitution request call " +
            											"taked over by core.");
            		manager_gui.getSubstitutionDialog().setVisible(false);
            	}
            	else {
            		manager_gui.printToTextBox("Substitution request call " +
            										"not taked over by core");
            	}
            }
            else
            //reply for substitution (result of the req application from core)
            if (reply_type.equals("playerSubstitutionReply")) {
            	//get the response of the reply
		    	boolean reply_status = content.getBoolean("Reply_Status");
		    	System.out.println("Reply_Status: " + reply_status);
            	if (reply_status) {
            		manager_gui.printToTextBox("Substitution granted.");
            		manager_gui.getListener().substitutionAccepted();
            	}
            	else {
            		manager_gui.printToTextBox("Substitution failed.");
            	}
            	manager_gui.getSubstitutionDialog().setEnabledButtons(true);
            }
		}
		
		private void applyInGameConfiguration(String team_id,
														int team_formation,
														int pressing_level) {
			//set team
			Vector<Team> teams = manager_gui.getTeams();
			for (int i = 0; i < teams.size(); i++) {
				if (teams.elementAt(i).getId().equals(team_id)) {
					manager_gui.getTeamsCombo().setSelectedIndex(i);
					manager_gui.populateTeam(i);
				}
			}
			//set formation
			manager_gui.getFormationsCombo().
							setSelectedIndex(team_formation - 1);
			//set pressing level
			manager_gui.getPressingSlider().setValue(pressing_level);
			manager_gui.getSoccerField().getSoccerFieldStatus().
									 setTeamBalance(pressing_level);
			manager_gui.getListener().setInGameTactic(
												team_formation - 1,
												pressing_level);
			manager_gui.getSoccerField().repaint();
			//players will be set by stats subscriber
			manager_gui.setAdaptPlayersModel(true);
			//create dialogs
			manager_gui.setTacticDialog(
									 new TacticDialog(manager_gui));
			manager_gui.setSubstitutionDialog(
							   new SubstitutionDialog(manager_gui));
			//go to ct pane
			CardLayout cl = (CardLayout)(manager_gui.getCardsPanel().
													   getLayout());
			cl.show(manager_gui.getCardsPanel(), "CT_Panel");
			manager_gui.setInGame(true);
			manager_gui.inGameButtons(true);
		}
		
		private void parseTeamsXML(String teams_xml) {
			//create empty Teams array
			Vector<Team> teams = new Vector<Team>();
			try {
				DocumentBuilderFactory f = DocumentBuilderFactory.newInstance();
				DocumentBuilder b = f.newDocumentBuilder();
				Document doc = b.parse(new ByteArrayInputStream(teams_xml.
															getBytes("UTF-8")));
				NodeList teams_nodes = doc.getElementsByTagName("team");
				for (int i = 0; i < teams_nodes.getLength(); i++) {
					//create empty Team
					Team t = new Team();
					Node team = teams_nodes.item(i);
					Node team_id = team.getAttributes().getNamedItem("id");
					t.setId(team_id.getTextContent());
					Node team_name = team.getAttributes().getNamedItem("name");
					t.setName(team_name.getTextContent());
					Node team_def_formation = team.getAttributes().
											   getNamedItem("defaultFormation");
					t.setDefaultFormation(Integer.parseInt(team_def_formation.
															getTextContent()) - 1);
					Node team_def_pressing = team.getAttributes().
												getNamedItem("defaultPressing");
					t.setDefaultPressing(Integer.parseInt(team_def_pressing.
															getTextContent()));
					NodeList team_players_nodes = team.getChildNodes();
					for (int j = 0; j < team_players_nodes.getLength(); j++) {
						Node child_node = team_players_nodes.item(j);
						if (child_node.getNodeName().equals("player")) {
							//create empty Player
							Player player = new Player();
							Node player_id = child_node.getAttributes().
															getNamedItem("id");
							player.setId(player_id.getTextContent());
							Node player_defstate = child_node.getAttributes().
												   getNamedItem("defaultState");
							player.setStatus(player_defstate.getTextContent());
							Node player_defrole = child_node.getAttributes().
													getNamedItem("defaultRole");
							player.setRole(Integer.parseInt(player_defrole.
															 getTextContent()));
							NodeList skills_player_nodes = child_node.
																getChildNodes();
							for (int k = 0; k < skills_player_nodes.
															getLength(); k++) {
								Node child_node2 = skills_player_nodes.item(k);
								if (child_node2.getNodeName().equals("name")) {
									player.setName(child_node2.getTextContent());
								}
								else
								if (child_node2.getNodeName().
														equals("shirtNumber")) {
									player.setShirtNumber(Integer.
														parseInt(child_node2.
														getTextContent()));
								}
								else
								if (child_node2.getNodeName().equals("speed")) {
									player.setSpeed(Double.
													parseDouble(child_node2.
													getTextContent()));
								}
								else
								if (child_node2.getNodeName().
														equals("stamina")) {
									player.setStamina(Double.
													parseDouble(child_node2.
													getTextContent()));
								}
								else
								if (child_node2.getNodeName().equals("height")) {
									player.setHeight(Double.
													parseDouble(child_node2.
													getTextContent()));
								}
								else
								if (child_node2.getNodeName().
														equals("jumpSkill")) {
									player.setJump(Double.
													parseDouble(child_node2.
													getTextContent()));
								}
								else
								if (child_node2.getNodeName().
													equals("dribbleAccuracy")) {
									player.setDribbling(Double.
													parseDouble(child_node2.
													getTextContent()));
								}
								else
								if (child_node2.getNodeName().
														equals("tackleSkill")) {
									player.setTackle(Double.
													parseDouble(child_node2.
													getTextContent()));
								}
								else
								if (child_node2.getNodeName().
												   equals("catchingAccuracy")) {
									player.setCatching(Double.
														parseDouble(child_node2.
														getTextContent()));
								}
								else
								if (child_node2.getNodeName().equals("passing")) {
									NodeList passing_skills_player_nodes =
													child_node2.getChildNodes();
									for (int z = 0;
											z < passing_skills_player_nodes.
															getLength(); z++) {
										Node child_node3 =
											passing_skills_player_nodes.item(z);
										if (child_node3.getNodeName().
															equals("power")) {
											player.setPassPower(Double.
														parseDouble(child_node3.
														getTextContent()));
										}
										if (child_node3.getNodeName().
														   equals("accuracy")) {
											player.setPassAccuracy(Double.
														parseDouble(child_node3.
														getTextContent()));
										}
									}
								}
								else
								if (child_node2.getNodeName().equals("shooting")) {
									NodeList shooting_skills_player_nodes =
													child_node2.getChildNodes();
									for (int z = 0;
											z < shooting_skills_player_nodes.
															 getLength(); z++) {
										Node child_node3 =
											shooting_skills_player_nodes.item(z);
										if (child_node3.getNodeName().
															  equals("power")) {
											player.setShootPower(Double.
														parseDouble(child_node3.
														getTextContent()));
										}
										if (child_node3.getNodeName().
														   equals("accuracy")) {
											player.setShootPower(Double.
														parseDouble(child_node3.
														getTextContent()));
										}
									}
								}
								else
								if (child_node2.getNodeName().
															equals("heading")) {
									NodeList heading_skills_player_nodes =
													child_node2.getChildNodes();
									for (int z = 0;
											z < heading_skills_player_nodes.
															getLength(); z++) {
										Node child_node3 =
											heading_skills_player_nodes.item(z);
										if (child_node3.getNodeName().
															  equals("power")) {
											player.setHeadPower(Double.
														parseDouble(child_node3.
														getTextContent()));
										}
										if (child_node3.getNodeName().
															equals("accuracy")) {
											player.setHeadAccuracy(Double.
														parseDouble(child_node3.
														getTextContent()));
										}
									}
								}
								else
								if (child_node2.getNodeName().
														equals("aggression")) {
									player.setAggressiveness(Double.
														parseDouble(child_node2.
														getTextContent()));
								}
								else
								if (child_node2.getNodeName().
													 equals("teamWorkSkill")) {
									player.setTeamWorkSkill(Double.
														parseDouble(child_node2.
														getTextContent()));
								}
							}
							//add player to the team
							t.addPlayer(player);
						}	
					}
					//add the team to the team array
					teams.add(t);
				}
				//final Teams array
				manager_gui.setTeams(teams);
			}
			catch (Exception e) {
				System.out.println(e);
			}
		}
	}
	
}

