
package manager_gui;

import javax.swing.JPanel;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Font;
import java.awt.Color;
import java.awt.RenderingHints;

class SoccerFieldPane extends JPanel {
	
	private int component_width, component_height,
				field_width, field_height,
				field_origin_w, field_origin_h;
	private double magnification_ratio;
	private boolean show_players_numbers;
	private SoccerFieldStatus soccer_field_status;
	
	public SoccerFieldPane() {
		soccer_field_status = new SoccerFieldStatus();
		magnification_ratio = 3.0;
		show_players_numbers = true;
	}
	
	protected void paintComponent(Graphics graphics) {
		//set anti-aliased
		Graphics2D g2 = (Graphics2D)graphics;
    	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
    						RenderingHints.VALUE_ANTIALIAS_ON);
		
		//get component dimension
		component_width = getWidth();
		component_height = getHeight();
	
		//draw gray background
		graphics = drawComponentBackground(graphics, new Color(177, 177, 177));
	
		//calculate soccer field dimension and position
		field_width = component_width - (int)(component_width * 0.0800);
		field_origin_w = (int)(component_width * 0.0400);
		if (field_width < 200) {
			field_width = 200;
			field_origin_w = 12; 
		}
		field_height = (int)(field_width * 0.6538);
		if (field_height + (int)(component_height * 0.1130) > component_height) {
			field_height = component_height - (int)(component_height * 0.1130);
			field_width = (int)(field_height * 1.5294);
			field_origin_w = (int)((component_width - field_width) / 2);
		}
		field_origin_h = (int)((component_height - field_height) / 2);
		
		//draw green field
		graphics = drawSoccerGreenField(graphics, new Color(64, 160, 64),
													new Color(56, 156, 56));
		
		//draw soccer field lines
		graphics = drawSoccerFieldLines(graphics, new Color(255, 255, 255));
		
		//draw team
		graphics = drawSoccerTeam(graphics, soccer_field_status.getTeamColor(),
													new Color(255, 255, 255));
	}
	
	SoccerFieldStatus getSoccerFieldStatus() {
		return soccer_field_status;
	}
	
	private int convertXPosition(double x_position) {
		return (int)(field_width * x_position / 103.0);
	}
	
	private int convertYPosition(double y_position) {
		return (int)(field_height * y_position / 67.0);
	}
	
	private Graphics drawComponentBackground(Graphics graphics, Color color) {
		graphics.setColor(color);
		graphics.fillRect(0, 0, component_width - 1, component_height - 1);
		return graphics;
	}
	
	private Graphics drawSoccerGreenField(Graphics graphics, Color color1,
																Color color2) {
		graphics.setColor(color1);
		graphics.fillRoundRect(field_origin_w - (int)(field_height * 0.0500),
						  field_origin_h - (int)(field_height * 0.0500),
						  field_width + (int)(field_height * 0.1000),
						  field_height + (int)(field_height * 0.1000),
						  (int)(field_height / 10), (int)(field_height / 10));
		graphics.setColor(color2);
		//field mower stripes
		for(int i = 0; i < 12; i++) {
			graphics.fillRect(field_origin_w + (int)(field_width / 24) +
						(int)(i * (field_width / 12)), field_origin_h,
					 	(int)(field_width / 24), field_height);
		}
		return graphics;
	}

	private Graphics drawSoccerFieldLines(Graphics graphics, Color color) {
		graphics.setColor(color);
		//outer rectangle
		graphics.drawRect(field_origin_w, field_origin_h, field_width,
																field_height);
		//half-way line
		graphics.drawLine(field_origin_w + (field_width / 2), field_origin_h,
											 field_origin_w + (field_width / 2),
											 field_origin_h + field_height );
		//centre circle
		graphics.drawOval(field_origin_w + (field_width / 2) -
							(((int)(field_height * 0.2691)) / 2),
					 		field_origin_h + (field_height / 2) -
					 		(((int)(field_height * 0.2691)) / 2),
					 		(int)(field_height * 0.2691),
					 		(int)(field_height * 0.2691));
		//centre spot
		graphics.fillOval(field_origin_w + (field_width / 2) -
							(int)(field_height * 0.0015),
							field_origin_h + (field_height / 2) -
							(int)(field_height * 0.0015),
					 		(int)(field_height * 0.0029),
					 		(int)(field_height * 0.0029));
		//upper left corner
		graphics.drawArc(field_origin_w - (((int)(field_height * 0.0294)) / 2),
						field_origin_h - (((int)(field_height * 0.0294)) / 2),
						(int)(field_height * 0.0294),
						(int)(field_height * 0.0294),
						270, 90); 
		//upper right corner
		graphics.drawArc(field_origin_w + field_width -
						(((int)(field_height * 0.0294)) / 2),
						field_origin_h - (((int)(field_height * 0.0294)) / 2),
												(int)(field_height * 0.0294),
												(int)(field_height * 0.0294),
												180, 90);
		//bottom right corner
		graphics.drawArc(field_origin_w + field_width -
							(((int)(field_height * 0.0294)) / 2),
							field_origin_h + field_height -
							(((int)(field_height * 0.0294)) / 2),
							(int)(field_height * 0.0294),
							(int)(field_height * 0.0294),
							90, 90);
		//bottom left corner
		graphics.drawArc(field_origin_w - (((int)(field_height * 0.0294)) / 2),
							field_origin_h + field_height -
							(((int)(field_height * 0.0294)) / 2),
							(int)(field_height * 0.0294),
							(int)(field_height * 0.0294),
							0, 90);
		//LEFT side
		//left area outer rect
		graphics.drawRect(field_origin_w, field_origin_h +
							(int)((field_height - (field_height * 0.5926)) / 2),
					 		(int)(field_height * 0.2426),
					 		(int)(field_height * 0.5926));
		//left area inner rect
		graphics.drawRect(field_origin_w, field_origin_h +
							(int)((field_height - (field_height * 0.2694)) / 2),
					 		(int)(field_height * 0.0808),
					 		(int)(field_height * 0.2694));
		//left goal
		graphics.drawRect(field_origin_w - (int)(field_height * 0.0358),
					 		field_origin_h + (int)((field_height -
					 		(field_height * 0.1076)) / 2),
					 		(int)(field_height * 0.0358),
					 		(int)(field_height * 0.1076));
		//left penalty kick spot
		graphics.fillOval(field_origin_w + (int)(field_height * 0.1617) -
							(int)(field_height * 0.0015),
					 		field_origin_h + (field_height / 2) -
					 		(int)(field_height * 0.0015),
					 		(int)(field_height * 0.0029),
					 		(int)(field_height * 0.0029));
		//left area arc
		graphics.drawArc(field_origin_w + (int)(field_height * 0.1617) -
						(((int)(field_height * 0.2691)) / 2),
						field_origin_h + (field_height / 2) -
						(((int)(field_height * 0.2691)) / 2),
						(int)(field_height * 0.2691),
						(int)(field_height * 0.2691),
						308, 104);
		//RIGHT side
		//right area outer rect
		graphics.drawRect(field_origin_w + field_width -
							(int)(field_height * 0.2426),
					 		field_origin_h + (int)((field_height -
					 		(field_height * 0.5926)) / 2),
					 		(int)(field_height * 0.2426),
					 		(int)(field_height * 0.5926)); 
		//right area inner rect
		graphics.drawRect(field_origin_w + field_width -
							(int)(field_height * 0.0808),
					 		field_origin_h + (int)((field_height -
					 		(field_height * 0.2694)) / 2),
					 		(int)(field_height * 0.0808),
					 		(int)(field_height * 0.2694));
		//right goal
		graphics.drawRect(field_origin_w + field_width,
					 		field_origin_h + (int)((field_height -
					 		(field_height * 0.1076)) / 2),
					 		(int)(field_height * 0.0358),
					 		(int)(field_height * 0.1076));
		//right penalty kick spot
		graphics.fillOval(field_origin_w + field_width -
							(int)(field_height * 0.1617) -
							(int)(field_height * 0.0015),
					 		field_origin_h + (field_height / 2) -
					 		(int)(field_height * 0.0015),
					 		(int)(field_height * 0.0029),
					 		(int)(field_height * 0.0029));
		//right area arc 
		graphics.drawArc(field_origin_w + field_width -
						(int)(field_height * 0.1617) -
						(((int)(field_height * 0.2691)) / 2),
						field_origin_h + (field_height / 2) -
						(((int)(field_height * 0.2691)) / 2),
						(int)(field_height * 0.2691),
						(int)(field_height * 0.2691),
						128, 104);
		return graphics;
	}
	
	private Graphics drawSoccerPlayer(Graphics graphics, Color color1,
									Color color2, int number, String name,
															int x, int y) {
		graphics.setColor(color1);
		//draw square
		graphics.fillRect(field_origin_w - (int)(field_height * 0.0147) -
						(int)((field_height * 0.0073) * magnification_ratio) + x,
					 	field_origin_h - (int)(field_height * 0.0147) -
					 	(int)((field_height * 0.0073) * magnification_ratio) + y,
					 	(int)(field_height * 0.0147 * magnification_ratio),
					 	(int)(field_height * 0.0147 * magnification_ratio));
		graphics.setColor(color2);
		int font_size = (int)(5 + ((field_width - 300) / 86) *
											((magnification_ratio - 1) / 2));
		graphics.setFont(new Font(graphics.getFont().getName(),
											Font.BOLD, font_size));
		if(show_players_numbers) {
			//draw uniform name
			graphics.drawString(name, field_origin_w -
								(int)(field_height * 0.0360) -
								(int)((field_height * 0.0073) *
								magnification_ratio) + x,
						   		field_origin_h - (int)(field_height * 0.0500) +
						   		(int)((field_height * 0.0050) *
						   		magnification_ratio) + y);
			//draw uniform number
			font_size = (int)(8 + ((field_width - 300) / 86) *
											((magnification_ratio - 1) / 2));
			graphics.setFont(new Font(graphics.getFont().getName(),
														Font.BOLD, font_size));
			graphics.drawString(Integer.toString(number),
						   		field_origin_w - (int)(field_height * 0.0147) -
						   		(int)((field_height * 0.0073) *
						   		magnification_ratio) + x,
						   		field_origin_h - (int)(field_height * 0.0147) +
						   		(int)((field_height * 0.0050) *
						   		magnification_ratio) + y);
		}
		return graphics;
	}
	
	private Graphics drawSoccerTeam(Graphics graphics, Color color1,
															Color color2) {
		//draw players
		for(int i = 0; i < 17; i++) {
			graphics = drawSoccerPlayer(graphics, color1, color2,
										soccer_field_status.getPlayerNumber(i),
										soccer_field_status.getPlayerName(i),
								   		convertXPosition(soccer_field_status.
								   			getPlayerPosition(i,
								   			SoccerFieldStatus.Axis.X)),
								   		convertYPosition(soccer_field_status.
								   			getPlayerPosition(i,
								   			SoccerFieldStatus.Axis.Y)));
		}
		return graphics;
	}
	
}

