
package manager_gui;

import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JSlider;
import javax.swing.WindowConstants;
import javax.swing.border.EmptyBorder;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

class TacticDialog extends JDialog {
	
	private ManagerGUI manager_gui;
	private SoccerFieldPane soccer_field_pane;
	private JComboBox formation_combo;
	private JSlider pressing_slider;
	private JButton apply_button;
	private JButton cancel_button;
	private boolean waiting_reply = false;
	
	public TacticDialog(final ManagerGUI manager_gui) {
		this.manager_gui = manager_gui;
		this.soccer_field_pane = manager_gui.getSoccerField();
		this.formation_combo = manager_gui.getFormationsCombo();
		this.pressing_slider = manager_gui.getPressingSlider();
		
		JLabel label = new JLabel("Select Formation");
		JPanel flow_pane2 = new JPanel(new FlowLayout(FlowLayout.CENTER,
																	20, 20));
        flow_pane2.add(label);
		
		JLabel formation_label = new JLabel("Formation");
		JPanel grid_pane2 = new JPanel(new GridLayout(0, 1, 0, 5));
		grid_pane2.add(formation_label);
		grid_pane2.add(formation_combo);
		
		JLabel pressing_level = new JLabel("Pressing Level");
		JPanel grid_pane3 = new JPanel(new GridLayout(0, 1, 0, 5));
		grid_pane3.add(pressing_level);
		grid_pane3.add(pressing_slider);
		JPanel grid_pane4 = new JPanel(new GridLayout(0, 1, 0, 15));
		grid_pane4.add(grid_pane2);
		grid_pane4.add(grid_pane3);
		JPanel flow_pane3 = new JPanel(new FlowLayout(FlowLayout.CENTER));
		flow_pane3.add(grid_pane4);
		
		apply_button = new JButton("Apply");
		apply_button.setActionCommand("apply-tactic");
        apply_button.addActionListener(manager_gui.getListener());
        cancel_button = new JButton("Cancel");
        cancel_button.setActionCommand("exit-tactic-dialog");
        cancel_button.addActionListener(manager_gui.getListener());
		JPanel flow_pane = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JPanel grid_pane = new JPanel(new GridLayout(1, 0, 5, 0));
  		grid_pane.add(cancel_button);
  		grid_pane.add(apply_button);
  		flow_pane.add(grid_pane);
		
		JPanel gridbag_pane = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 0.9; c.weighty = 1.0;
		c.gridx = 0; c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 2;
		gridbag_pane.add(soccer_field_pane, c);
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0.1; c.weighty = 0.9;
		c.gridx = 1; c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		gridbag_pane.add(flow_pane3, c);
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 0.1; c.weighty = 0.1;
		c.gridx = 1; c.gridy = 1;
		c.gridwidth = 1;
		c.gridheight = 1;
		gridbag_pane.add(flow_pane, c);
		
		JPanel pane = new JPanel(new BorderLayout());
		pane.setBorder(new EmptyBorder(0, 20, 20, 0));
		pane.add(flow_pane2, BorderLayout.NORTH);
		pane.add(gridbag_pane, BorderLayout.CENTER);
		
		setLayout(new BorderLayout());
		add(pane);
		
		setTitle("Change Tactic");
		setLocationRelativeTo(manager_gui);
        setMinimumSize(new Dimension(650, 420));
        setResizable(true);
        setModal(true);
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				setVisible(false);
				if (!waiting_reply)
					manager_gui.getListener().closeTacticDialog();
			}
		});
	}
	
	void setEnabledButtons(boolean state) {
		apply_button.setEnabled(state);
		cancel_button.setEnabled(state);
		waiting_reply = !state;
	}
}

