
package simulator_gui;

import javax.swing.SwingWorker;
import com.inspirel.yami.Agent;
import com.inspirel.yami.Parameters;
import com.inspirel.yami.OutgoingMessage;
import com.inspirel.yami.YAMIIOException;

class Client extends SwingWorker<Void, Void> {
    
    //This Class follow Client pattern
	//asynchronous properties are extended by yami4
    private String[] args;
    private SimulatorGUI simulator_gui;
    private boolean exit;
    private boolean affirmative_validation;
    private Agent client_agent;
    private Parameters params;
    private OutgoingMessage message;
    private String server_address;
    private final static String[] actions = { "simulatorGUI.connect",
                        			 		"simulatorGUI.disconnect",
                        			 		"simulatorGUI.startMatch",
                        			 		"simulatorGUI.stopMatch",
                        			 		"simulatorGUI.startSecondHalf"};
    
    public Client(String billboard_input_address, SimulatorGUI simulator_gui) {
		this.simulator_gui = simulator_gui;
		this.server_address = billboard_input_address;
		exit = false;
		affirmative_validation = false;
		try {
			//prepare client agent
			client_agent = new Agent();
			params = new Parameters();
		}
		catch (YAMIIOException ex) {
			System.out.println("[client]* error: " + ex.getMessage());
		}
		System.out.println("[client]* Client is ready");
    }
    
    protected Void doInBackground() {
		System.out.println("[client]* Client started");
		simulator_gui.runningSignal();
		try {
			Thread.sleep(1500);
			while (!exit) {
				try {
					//try to connect to Billboard after gui opening
					simulator_gui.printToTextBox("Trying to connect to " +
																"Billboard...");
					System.out.println("[client]> " + actions[0] + " was sent");
					message = client_agent.send(server_address, "controller",
															actions[0], params);
					System.out.println("[client]* Waiting validation response...");
					//wait at most 3 seconds for the reply
					message.waitForCompletion(3000);
					OutgoingMessage.MessageState state = message.getState();
					if (state == OutgoingMessage.MessageState.REPLIED) {
						System.out.println("[client]< Connected to Billboard, " +
													"SimulatorGUI is allowed.");
						simulator_gui.printToTextBox("Connected to Billboard, " +
													"SimulatorGUI is allowed.");
						affirmative_validation = true;
						//start subscribers
						simulator_gui.startSubscribers();
					}
					else
					if (state == OutgoingMessage.MessageState.REJECTED) {
						System.out.println("[client]< Rejected from Billboard");
						simulator_gui.printToTextBox("Rejected from " +
													"Billboard, only one " +
													"connected instance of " +
													"SimulatorGUI is allowed in "+
													"the system.");
					}
					else {
						System.out.println("[client]< Error while receiving " +
														"validation response.");
						simulator_gui.printToTextBox("Error while receiving " +
														"validation response.");
						exit = true;
					}
					message.close();
					//close wait dialog
					simulator_gui.closeWaitDialog();
					if (!exit) {
						synchronized (this) {
							wait();
						}
					}
				}
				catch (YAMIIOException ex) {
					System.out.println("[client]* error: " + ex.getMessage());
				}
				
				//jumped on yami exception
				//retries every 5 seconds if billboard is not available
				if (!exit)
					Thread.sleep(5000);
			}
		}
		catch (InterruptedException ex) {
			exit = true;
		}
		//deregister from Billboard on gui closure
		if (affirmative_validation) {
			System.out.println("[client]> " + actions[1] + " was sent" );
			try {
				client_agent.sendOneWay(server_address, "controller",
													actions[1], params);
			}
			catch (YAMIIOException ex) {
				System.out.println("[client]* error: " + ex.getMessage());
			}
		}
		
		try {
			Thread.sleep(1000);
		}
		catch (InterruptedException ex) { }
		
		System.out.println("[client]* Client terminates");
		client_agent.close();
		simulator_gui.completitionSignal();
        return null;
	}
	
	void exitTask() {
		exit = true;
		cancel(true);
	}
	
	void startMatch(int[] match_config) {
		System.out.println("[client]> " + actions[2] + " was sent");
		simulator_gui.printToTextBox("Start match signal was sent");
		params.setInteger("periodDuration", match_config[0]);
		params.setInteger("breakDuration", match_config[1]);
		params.setInteger("substitutionsAllowed", match_config[2]);
		try {
			client_agent.sendOneWay(server_address, "controller", actions[2],
																		params);
		}
		catch (YAMIIOException ex) {
			System.out.println("[client]* error: " + ex.getMessage());
		}
	}
	
	void abortMatch() {
		System.out.println("[client]> " + actions[3] + " was sent");
		simulator_gui.printToTextBox("Abort match signal was sent");
		try {
			client_agent.sendOneWay(server_address, "controller", actions[3],
																		params);
		}
		catch (YAMIIOException ex) {
			System.out.println("[client]* error: " + ex.getMessage());
		}
	}
	
	void startSecondHalf() {
		System.out.println("[client]> " + actions[4] + " was sent");
		simulator_gui.printToTextBox("Start 2nd half signal was sent");
		try {
			client_agent.sendOneWay(server_address, "controller", actions[4],
																		params);
		}
		catch (YAMIIOException ex) {
			System.out.println("[client]* error: " + ex.getMessage());
		}
	}
	
}

