
package simulator_gui;

import javax.swing.SwingWorker;
import com.inspirel.yami.Agent;
import com.inspirel.yami.IncomingMessage;
import com.inspirel.yami.IncomingMessageCallback;
import com.inspirel.yami.Parameters;
import com.inspirel.yami.YAMIIOException;

class GameStatusSubscriber extends SwingWorker<Void, Void> {

 	private static SimulatorGUI simulator_gui;
	private String[] args;
	private String publisher_address;
	private Agent subscriber_agent;
	private Parameters params;
    private final String update_object_name = "update_handler";
	
	public GameStatusSubscriber(String billboard_output_address,
												   SimulatorGUI simulator_gui) {
		this.simulator_gui = simulator_gui;
		this.publisher_address = billboard_output_address;
		try {
			//prepare callback agent
			subscriber_agent = new Agent();
			params = new Parameters();
		}
		catch (YAMIIOException ex) {
			System.out.println("[gamestatus subscriber]* error: " +
																ex.getMessage());
		}
		System.out.println("[gamestatus subscriber]* GameStatusSubscriber is ready");
	}
	
	protected Void doInBackground() {
        System.out.println("[gamestatus subscriber]* GameStatusSubscriber started");
        simulator_gui.runningSignal();
		try {
			Thread.sleep(200);
			try {
			    // prepare subscription update callback
			    subscriber_agent.registerObject(update_object_name,
			    								new GameStatusHandler());
				// subscribe to the Frame_Producer
				Parameters params = new Parameters();
				params.setString("destination_object", update_object_name);
				subscriber_agent.sendOneWay(publisher_address,
								"Match_Status", "subscribe", params);
				System.out.println("[gamestatus subscriber]< Subscribed to " +
														"Match_Status channel");
		    	simulator_gui.printToTextBox("Subscribed to Game Status Channel.");
			    //block and receive updates in background
	    		synchronized (this) {
					wait();
				}
			    subscriber_agent.close();
			}
			catch (YAMIIOException ex) {
				//billboard output is not available
			    System.out.println("[gamestatus subscriber]* error: " +
			    												ex.getMessage());
			}
        }
		catch (InterruptedException ex) { }
        System.out.println("[gamestatus subscriber]* GameStatusSubscriber terminates");
        subscriber_agent.unregisterObject(update_object_name);
        subscriber_agent.close();
        simulator_gui.completitionSignal();
        return null;
	}
	
	private static class GameStatusHandler implements IncomingMessageCallback {
		//Method that receive callbacks
		@Override
		public void call(IncomingMessage im) throws Exception {			
			synchronized (simulator_gui) {
				Parameters content = im.getParameters();
				String match_status = content.getString("Match_Status");
				System.out.println("[gamestatus subscriber]< Notification " +
            					"received. \nMatch_Status: " + match_status);
				//receive updates only after successful initialization
				if (simulator_gui.getSuccessfulInit()) {
					if (match_status.equals("MATCH_STARTED")) {
						//match start signal failed so, set default combo
						if (!simulator_gui.getStartMatchAccepted()) {
							simulator_gui.setDefaultComboBoxSelection();
							simulator_gui.printToTextBox("The Match has " +
													"started automatically " +
													"with the default " +
													"configuration.");	
						}
					}
					simulator_gui.enableGUIInteractions(match_status);
				
				}
			}
		}
	}
	
}

