
package simulator_gui;

import javax.swing.SwingWorker;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import java.io.ByteArrayInputStream;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import com.inspirel.yami.Agent;
import com.inspirel.yami.IncomingMessage;
import com.inspirel.yami.IncomingMessageCallback;
import com.inspirel.yami.Parameters;
import com.inspirel.yami.YAMIIOException;

class ResponseEventSubscriber extends SwingWorker<Void, Void> {

 	private static SimulatorGUI simulator_gui;
	private String[] args;
	private String publisher_address;
	private Agent subscriber_agent;
	private Parameters params;
    private final String update_object_name = "update_handler";
	
	public ResponseEventSubscriber(String billboard_output_address,
												   SimulatorGUI simulator_gui) {
		this.simulator_gui = simulator_gui;
		this.publisher_address = billboard_output_address;
		try {
			//prepare callback agent
			subscriber_agent = new Agent();
			params = new Parameters();
		}
		catch (YAMIIOException ex) {
			System.out.println("[responsevent subscriber]* error: " +
															ex.getMessage());
		}
		System.out.println("[responsevent subscriber]* " +
											"ResponseEventSubscriber is ready");
	}
	
	protected Void doInBackground() {
        System.out.println("[responsevent subscriber]* " +
        									 "ResponseEventSubscriber started");
        simulator_gui.runningSignal();
		try {
			Thread.sleep(150);
			try {
			    //try to subscribe to simulator reply channel
			    subscriber_agent.registerObject(update_object_name,
			    								new ResponseEventHandler());
				params.setString("destination_object", update_object_name);
				subscriber_agent.sendOneWay(publisher_address,
						"Simulator_Response_Inquiry", "subscribe", params);
				System.out.println("[responsevent subscriber]< Subscribed to " +
										"Simulator_Response_Inquiry channel");
		    	simulator_gui.printToTextBox("Subscribed to Response " +
		    												   "Event Channel.");
			    //block and receive updates in background
	    		synchronized (this) {
					wait();
				}
			    subscriber_agent.close();
			}
			catch (YAMIIOException ex) {
				//billboard output is not available
			    System.out.println("[responsevent subscriber]* error: " +
			    											   ex.getMessage());
			    simulator_gui.printToTextBox("Error: unable to find " +
		    								"Billboard Output. Please " +
		    								"start both Billboard Input " +
		    								"and Billboard Output components " +
		    								"and restart this GUI.");
			}
        }
		catch (InterruptedException ex) { }
        System.out.println("[responsevent subscriber]* " +
        								"ResponseEventSubscriber terminates");
        subscriber_agent.unregisterObject(update_object_name);
        subscriber_agent.close();
        simulator_gui.completitionSignal();
        return null;
	}
	
	
	private static class ResponseEventHandler
											implements IncomingMessageCallback {
		private int connected_locals;
		private int connected_visitors;
		private int connected_spectators;
		
		//Method that receive callbacks
		@Override
		public void call(IncomingMessage im) throws Exception {
			//receive message from the channel
			Parameters content = im.getParameters();
            //get the type of the reply
            String reply_type = content.getString("Reply_Type");
            //get the reply
            boolean reply_status = content.getBoolean("Reply_Status");
            System.out.println("[responsevent subscriber]< Notification " +
            					"received. \nReply_Type: " + reply_type +
            					"\nReply_Status: " + reply_status);
            //reply for connect (initialization)
            if (reply_type.equals("connect")) {
            	if (reply_status) {
					String match_state = content.getString("Match_Status");
					String simulation_xml = content.
											getString("Configuration_Xml");
					int period_duration = content.getInteger("Period_Duration");
					int break_duration = content.getInteger("Break_Duration");
					int substitutions_allowed = content.
											 getInteger("Substitution_Allowed");
					connected_locals = content.
									   getInteger("LocalManagerGUI_connecteds");
					connected_visitors = content.
									 getInteger("VisitorManagerGUI_connecteds");
					connected_spectators = content.
										  getInteger("SpectatorGUI_connecteds");
					if (!simulation_xml.equals("")) {
						synchronized (simulator_gui) {
							//initialize the gui
							//populate comboboxes and set xml default values
							parseSimulationXML(simulation_xml);
							//if during the match set ingame settings
							if (!match_state.equals("WARM_UP")) {
								simulator_gui.setInGameConfig(period_duration,
										 break_duration, substitutions_allowed);
							}
							simulator_gui.
										setConnectedLocals(connected_locals);
							simulator_gui.
									setConnectedVisitors(connected_visitors);
							simulator_gui.
								   setConnectedSpectators(connected_spectators);
							simulator_gui.enableGUIInteractions(match_state);
							//enable updates made by GameStatusSubscriber
							//to the GUI
							simulator_gui.setSuccessfulInit(true);
						}
						System.out.println("[responsevent subscriber]< " +
												"Initialization received");	
						simulator_gui.printToTextBox("GUI is initialized.");
					}
					else {
						System.out.println("[responsevent subscriber]< " +
								   "Initialization failed due to invalid xml)");
						simulator_gui.printToTextBox("Initialization failed " +
														"due to invalid xml.");
					}		
            	}
            	else {
            		System.out.println("[responsevent subscriber]< " +
            										"Initialization failed");
            		simulator_gui.printToTextBox("Initialization failed.");
            	}
            }
            else
            //reply for startMatch
            if (reply_type.equals("startMatch")) {
            	if (reply_status) {
            		//to prevent GameStatusSubscriber to set default for combo
            		synchronized (simulator_gui) {
            			simulator_gui.setStartMatchAccepted(true);
            		}
            		simulator_gui.printToTextBox("Start match accepted with " +
            									"the selected configuration.");
            	}
            	else {
            		simulator_gui.printToTextBox("Start match failed.");
            	}
            }
            else
            //reply for stopMatch
            if (reply_type.equals("stopMatch")) {
            	if (reply_status) {
            		simulator_gui.printToTextBox("Match aborted accepted.");
            	}
            	else {
            		simulator_gui.printToTextBox("Match aborted failed.");
            	}
            }
            else
            //reply for start2ndHalf
            if (reply_type.equals("startSecondHalf")) {
            	if (reply_status) {
            		simulator_gui.printToTextBox("Start 2nd half accepted");
            	}
            	else {
            		simulator_gui.printToTextBox("Start 2nd half failed.");
            	}
            }
            else
            //reply for update locals label
            if (reply_type.equals("updateLocalManagerGUI")) {
            	int new_connected_locals = content.
            							getInteger("Reply_Numeric_Detail");
            	simulator_gui.setConnectedLocals(new_connected_locals);
            	if (new_connected_locals > connected_locals)
            		simulator_gui.printToTextBox("Locals Manager has " +
            												"connected.");
            	else
            		simulator_gui.printToTextBox("Locals Manager has " +
            											"disconnected.");
            	connected_locals = new_connected_locals;
            }
            else
            //reply for update visitors label
            if (reply_type.equals("updateVisitorManagerGUI")) {
            	int new_connected_visitors = content.
            								getInteger("Reply_Numeric_Detail");
            	simulator_gui.setConnectedVisitors(new_connected_visitors);
            	if (new_connected_visitors > connected_visitors)
            		simulator_gui.printToTextBox("Visitors Manager has " +
            												"connected.");
            	else
            		simulator_gui.printToTextBox("Visitors Manager has " +
            											"disconnected.");
            	connected_visitors = new_connected_visitors;
            }
            else
            //reply for update spectators label
            if (reply_type.equals("updateSpectators")) {
            	int new_connected_spectators = content.	
            								getInteger("Reply_Numeric_Detail");
            	simulator_gui.setConnectedSpectators(new_connected_spectators);
            	if (new_connected_spectators > connected_spectators)
            		simulator_gui.printToTextBox("A new Spectator has " +
            												"connected.");
            	else
            		simulator_gui.printToTextBox("A Spectator has " +
            										"disconnected.");
            	connected_spectators = new_connected_spectators;
            }
		}
		
		private void parseSimulationXML(String simulation_xml) {
			int period_duration_def_index = 0;
			Vector<Integer> v_period_durations = new Vector<Integer>();
			int break_duration_def_index = 0;
			Vector<Integer> v_break_durations = new Vector<Integer>();
			int subst_allowed_def_index = 0;
			Vector<Integer> v_substitutions_allowed = new Vector<Integer>();
		
			//simulation xml parser
			try {
				DocumentBuilderFactory f = DocumentBuilderFactory.newInstance();
				DocumentBuilder b = f.newDocumentBuilder();
				Document doc = b.parse(new ByteArrayInputStream(simulation_xml.
															getBytes("UTF-8")));
				Node period_duration = doc.
							getElementsByTagName("periodDuration").item(0);
				Node period_duration_default = period_duration.
									getAttributes().getNamedItem("default");
				Integer period_duration_def = Integer.
							parseInt(period_duration_default.getTextContent());
				NodeList period_duration_nodes = period_duration.getChildNodes();
				for (int i = 0; i < period_duration_nodes.getLength(); i++) {
					if (period_duration_nodes.item(i).getNodeName().
															equals("value"))
						v_period_durations.add(Integer.
										parseInt(period_duration_nodes.item(i).
															getTextContent()));
				}
				period_duration_def_index = v_period_durations.
										indexOf(period_duration_def);	
				Node break_duration = doc.
						getElementsByTagName("breakDuration").item(0);
				Node break_duration_default = break_duration.
							getAttributes().getNamedItem("default");
				Integer break_duration_def = Integer.
							parseInt(break_duration_default.getTextContent());
				NodeList break_duration_nodes = break_duration.getChildNodes();
				for (int i = 0; i < break_duration_nodes.getLength(); i++) {
					if (break_duration_nodes.item(i).getNodeName().
															equals("value"))
						v_break_durations.add(Integer.
										parseInt(break_duration_nodes.item(i).
															getTextContent()));
				}
				break_duration_def_index = v_break_durations.
									indexOf(break_duration_def);
				Node substitutions_allowed = doc.
					getElementsByTagName("substitutionsAllowed").item(0);
				Node substitutions_allowed_default = substitutions_allowed.
									getAttributes().getNamedItem("default");
				Integer substitutions_allowed_def = Integer.
					parseInt(substitutions_allowed_default.getTextContent());
				NodeList substitutions_allowed_nodes = substitutions_allowed.
																getChildNodes();
				for (int i = 0; i < substitutions_allowed_nodes.
														getLength(); i++) {
					if (substitutions_allowed_nodes.item(i).
									getNodeName().equals("value"))
						v_substitutions_allowed.add(Integer.
							parseInt(substitutions_allowed_nodes.
								item(i).getTextContent()));
				}
				subst_allowed_def_index = v_substitutions_allowed.
									indexOf(substitutions_allowed_def);
			}
			catch (Exception e) {
				System.out.println(e);
			}
			simulator_gui.populateComboBoxes(period_duration_def_index,
											v_period_durations,
											break_duration_def_index,
											v_break_durations,
											subst_allowed_def_index,
											v_substitutions_allowed);
		}
	}
	
}

