
package simulator_gui;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JLabel;
import javax.swing.border.EmptyBorder;
import javax.swing.UIManager;
import javax.swing.SwingWorker;
import javax.swing.UnsupportedLookAndFeelException;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.Vector;
import java.net.URI;
import java.net.URISyntaxException;

class SimulatorGUI extends JPanel {
	
	private Listener listener;
	//tasks
	private Client client;
	private ResponseEventSubscriber response_event_subscriber;
	private GameStatusSubscriber game_status_subscriber;
	
	//ui elements
	private JComboBox match_duration_combo;
	private JComboBox break_duration_combo;
	private JComboBox substitutions_combo;	
	private JButton start_button, abort_button;
	private JTextField connected_locals_label;
	private JTextField connected_visitors_label;
	private JTextField connected_spectators_label;
	private JScrollPane scroll_pane;
	private JTextArea output_textarea;
	private WaitDialog wait_dialog;
	
	//gui internal state
	private Vector<Integer> period_durations;
	private Vector<Integer> break_durations;
	private Vector<Integer> allowed_substitutions;
	private int period_duration_def_index;
	private int break_duration_def_index;
	private int subst_allowed_def_index;
	private int[] match_config;
	private boolean break_time = false;
	private boolean successful_init = false;
	private boolean start_match_accepted = false;
    private int completed_tasks = 0;
    private int running_tasks = 0;
	
	public SimulatorGUI(String[] args) {
		System.out.println("[simulator gui]* Starting application...");
	
		//define the GUI Listener
		listener = new Listener(this);
		
		//match duration combo box and label
		JLabel combo_label_match = new JLabel("Match Duration");
		String[] period_durations = { };
		match_duration_combo = new JComboBox(period_durations);	
		//break duration combo box and label						
		JLabel combo_label_break = new JLabel("Break Duration");
		String[] break_durations = { };
		break_duration_combo = new JComboBox(break_durations);
		//substitutions combo box and label						
		JLabel combo_label_subst = new JLabel("Substitutions");
		String[] allowed_substitutions = { };
		substitutions_combo = new JComboBox(allowed_substitutions);
		JPanel flow_pane = new JPanel(new FlowLayout(FlowLayout.CENTER, 20, 20));
		JPanel grid_pane = new JPanel(new GridLayout(3, 2, 5, 5));
		grid_pane.add(combo_label_match);
		grid_pane.add(match_duration_combo);
		grid_pane.add(combo_label_break);
		grid_pane.add(break_duration_combo);
		grid_pane.add(combo_label_subst);
		grid_pane.add(substitutions_combo);
		flow_pane.add(grid_pane);
		JPanel gridbag_pane = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		gridbag_pane.add(grid_pane, c);
		
		//start button
		start_button = new JButton("Start Match");
		start_button.setAlignmentX(Component.CENTER_ALIGNMENT);
		start_button.setActionCommand("start");
		start_button.addActionListener(listener);
		//abort button
		abort_button = new JButton("Abort Match");
		abort_button.setAlignmentX(Component.CENTER_ALIGNMENT);
		abort_button.setActionCommand("abort");
		abort_button.addActionListener(listener);
        JPanel flow_pane2 = new JPanel(new FlowLayout(FlowLayout.CENTER, 20, 20));
        JPanel grid_pane2 = new JPanel(new GridLayout(0, 1, 5, 5));
  		grid_pane2.add(start_button);
  		grid_pane2.add(abort_button);
		flow_pane2.add(grid_pane2);
		c.gridx = 1;
		c.gridy = 0;
		gridbag_pane.add(flow_pane2, c);
		
		System.out.println("[simulator gui]* Listener binding complete");
		
		//connected locals label
		connected_locals_label = new JTextField("Connected Locals: 0");
		connected_locals_label.setEditable(false);
		connected_locals_label.setBorder(javax.swing.BorderFactory.
														createEmptyBorder());
		//connected visitors label
		connected_visitors_label = new JTextField("Connected Visitors: 0");
		connected_visitors_label.setEditable(false);
		connected_visitors_label.setBorder(javax.swing.BorderFactory.
														createEmptyBorder());
		//connected spectators label
		connected_spectators_label = new JTextField("Connected Spectators: 0");
		connected_spectators_label.setEditable(false);
		connected_spectators_label.setBorder(javax.swing.BorderFactory.
														   createEmptyBorder());
		JPanel flow_pane3 = new JPanel(new FlowLayout(FlowLayout.CENTER, 10, 10));
		JPanel grid_pane3 = new JPanel(new GridLayout(0, 1, 5, 5));
		grid_pane3.add(connected_locals_label);
		grid_pane3.add(connected_visitors_label);
		grid_pane3.add(connected_spectators_label);
		flow_pane3.add(grid_pane3);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 2;
		c.gridx = 0;
		c.gridy = 1;
		gridbag_pane.add(flow_pane3, c);
		gridbag_pane.setBorder(new EmptyBorder(0, 0, 8, 0));
		
		//program output text area 
		output_textarea = new JTextArea("> Simulator output:");
		output_textarea.setLineWrap(true);
		output_textarea.setWrapStyleWord(true);
		output_textarea.setEditable(false);
		scroll_pane = new JScrollPane(output_textarea);
		
		match_duration_combo.setEnabled(false);
		break_duration_combo.setEnabled(false);
		substitutions_combo.setEnabled(false);
		start_button.setEnabled(false);
		abort_button.setEnabled(false);
		
		//main layout
		setLayout(new BorderLayout());
		setBorder(new EmptyBorder(15, 15, 15, 15));
		add(gridbag_pane, BorderLayout.NORTH);
		add(scroll_pane, BorderLayout.CENTER);
		
		//create WaitDialog
		wait_dialog = new WaitDialog(this);
		
		//create ResponseEventSubscriber
		response_event_subscriber = new ResponseEventSubscriber(args[1], this);
		
		//create GameStatusSubscriber
		game_status_subscriber = new GameStatusSubscriber(args[1], this);
		
		//create and start Client
		(client = new Client(args[0], this)).execute();
	}
	
	void disableButton(String button_name) {
		if (button_name.equals("start"))
			start_button.setEnabled(false);
		else
		if (button_name.equals("abort"))
			abort_button.setEnabled(false);
	}
	
	Client getClient() {
		return client;
	}
	
	boolean getSuccessfulInit() {
		return successful_init;
	}
	
	void setSuccessfulInit(boolean state) {
		successful_init = state;
	}
	
	boolean getStartMatchAccepted() {
		return start_match_accepted;
	}
	
	void setStartMatchAccepted(boolean state) {
		start_match_accepted = state;
	}
	
	int[] getMatchConfig() {
		match_config = new int[3];
		match_config[0] = period_durations.elementAt(match_duration_combo.
															getSelectedIndex());
		match_config[1] = break_durations.elementAt(break_duration_combo.
															getSelectedIndex());
		match_config[2] = allowed_substitutions.elementAt(substitutions_combo.
															getSelectedIndex());
		return match_config;
	}
	
	void populateComboBoxes(int period_duration_def_index,
								Vector<Integer> period_durations,
								int break_duration_def_index,
								Vector<Integer> break_durations,
								int subst_allowed_def_index,
								Vector<Integer> allowed_substitutions) {
		this.period_duration_def_index = period_duration_def_index;
		this.period_durations = period_durations;	
		this.break_duration_def_index = break_duration_def_index;
		this.break_durations = break_durations;	
		this.subst_allowed_def_index = subst_allowed_def_index;
		this.allowed_substitutions = allowed_substitutions;
		for(int i = 0; i < period_durations.size(); i++)
			match_duration_combo.addItem(period_durations.elementAt(i) +
																" minutes");
		for(int i = 0; i < break_durations.size(); i++)
			break_duration_combo.addItem(break_durations.elementAt(i) +
																" minutes");
		for(int i = 0; i < allowed_substitutions.size(); i++)
			substitutions_combo.addItem(allowed_substitutions.elementAt(i));
		setDefaultComboBoxSelection();
	}
	
	void setDefaultComboBoxSelection() {
		match_duration_combo.setSelectedIndex(period_duration_def_index);
		break_duration_combo.setSelectedIndex(break_duration_def_index);
		substitutions_combo.setSelectedIndex(subst_allowed_def_index);								
	}
	
	void setInGameConfig(int period_duration, int break_duration,
													int substitutions_allowed) {
		if (period_durations.contains(period_duration) &&
				break_durations.contains(break_duration) &&
				allowed_substitutions.contains(substitutions_allowed)) {
			int index_duration = period_durations.indexOf(period_duration);
			int index_break = break_durations.indexOf(break_duration);
			int index_subst = allowed_substitutions.indexOf(substitutions_allowed);
			match_duration_combo.setSelectedIndex(index_duration);
			break_duration_combo.setSelectedIndex(index_break);
			substitutions_combo.setSelectedIndex(index_subst);
		}
	}
	
	void setConnectedLocals(int value) {
		connected_locals_label.setText("Connected Locals: " + value);
	}
	
	void setConnectedVisitors(int value) {
		connected_visitors_label.setText("Connected Visitors: " + value);
	}
	
	void setConnectedSpectators(int value) {
		connected_spectators_label.setText("Connected Spectators: " + value);
	}
	
	boolean isBrakeTime() {
		return break_time;
	}
	
	void enableGUIInteractions(String match_status) {
		//before the beginning of the match
		if (match_status.equals("WARM_UP")) {
			break_time = false;
			start_button.setText("Start Match");
			match_duration_combo.setEnabled(true);
			break_duration_combo.setEnabled(true);
			substitutions_combo.setEnabled(true);
			start_button.setEnabled(true);
			abort_button.setEnabled(false);
			printToTextBox("Before the beginning of the match. Make your " +
							"configuration and press Start Match button, " +
							"otherwise, the match will be started automatically "+
							"when core's initial timer expires.");
		}
		else
		//during first half
		if (match_status.equals("MATCH_STARTED")) {
			break_time = false;
			start_button.setText("Start Match");
			match_duration_combo.setEnabled(false);
			break_duration_combo.setEnabled(false);
			substitutions_combo.setEnabled(false);
			start_button.setEnabled(false);
			abort_button.setEnabled(true);
			printToTextBox("First half in progress...");
		}
		else
		//during break time
		if (match_status.equals("FIRST_HALF_FINISHED")) {
			break_time = true;
			start_button.setText("Start 2nd Half");
			start_button.setEnabled(true);
			abort_button.setEnabled(true);
			printToTextBox("First half is over.");
			printToTextBox("Press Start 2nd Half button to start the 2nd half " +
							"immediatly. Otherwise, the match will be started " +
							"automatically when core's break timer expires.");
		}
		else
		//during second half
		if (match_status.equals("SECOND_HALF_STARTED")) {
			break_time = false;
			start_button.setText("Start 2nd Half");
			start_button.setEnabled(false);
			abort_button.setEnabled(true);
			printToTextBox("Second half in progress...");
		}
		else
		//end match
		if (match_status.equals("MATCH_ABORTED") ||
				match_status.equals("SECOND_HALF_FINISHED")) {
			break_time = false;
			start_button.setEnabled(false);
			abort_button.setEnabled(false);
			printToTextBox("The match is over.");
			successful_init = false;
			
		}
	}
	
	void printToTextBox(String message) {
		output_textarea.append("\n- " + message);
		int textbox_lenght = output_textarea.getDocument().getLength();
		output_textarea.setCaretPosition(textbox_lenght);
	}
	
	void closeWaitDialog() {
		wait_dialog.dispose();
	}
	
	void startSubscribers() {
		response_event_subscriber.execute();
		game_status_subscriber.execute();
	}
	
	private void terminateClientAndSubscribers() {
		System.out.println("[simulator gui]* Waiting Client and Subscribers " +
															  "termination...");
		if (response_event_subscriber.getState() ==
						SwingWorker.StateValue.STARTED) {
			response_event_subscriber.cancel(true);
		}
		else
			System.out.println("[spectator gui]* ResponseEventSubscriber is " +
															 	"not running");
		if (game_status_subscriber.getState() ==
						SwingWorker.StateValue.STARTED) {
			game_status_subscriber.cancel(true);
		}
		else
			System.out.println("[spectator gui]* GameStatusSubscriber is not " +
																 	"running");
		if (client.getState() == SwingWorker.StateValue.STARTED) {
			client.exitTask();
		}
		else
			System.out.println("[spectator gui]* Client is not running");
		waitCompletion();
	}
	
	synchronized void runningSignal() {
		running_tasks++;
	}
	
	synchronized void completitionSignal() {
		completed_tasks++;
		notify();
	}
	
	private synchronized void waitCompletion() {
		while (running_tasks != completed_tasks) {
			try {
				wait();
			}
			catch (InterruptedException ex) { }
		}
	}

    /*	Create the GUI and show it.  For thread safety,
    	this method should be invoked from the
		event-dispatching thread. */
    private static void createSimulatorGUI(String[] args) {
        //Create and set up the window.
        final JFrame frame = new JFrame("Simulator GUI");
        frame.setMinimumSize(new Dimension(400, 435));
        frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.out.println("[simulator gui]* Close SimulatorGUI " +
																 "by the user");
				SimulatorGUI pane = (SimulatorGUI)frame.getContentPane();
				pane.terminateClientAndSubscribers();
				System.out.println("[simulator gui]* Exiting SimulatorGUI...");
				System.exit(0);
			}
		});

        //Set up the content pane.
		SimulatorGUI content_pane = new SimulatorGUI(args);
		content_pane.setOpaque(true); //content panes must be opaque
		frame.setContentPane(content_pane);
        
        //Display the window.
        frame.pack();
        frame.setVisible(true);
        frame.setLocationRelativeTo(null);
        content_pane.wait_dialog.setVisible(true);
    }

    public static void main(String[] args) {
    	//read billboard addresses from configuration file
    	final String[] addresses = readConfigFile(args);
    	//use an appropriate Look and Feel
    	try {
            UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
        } catch (UnsupportedLookAndFeelException ex) {
            ex.printStackTrace();
        } catch (IllegalAccessException ex) {
            ex.printStackTrace();
        } catch (InstantiationException ex) {
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        }
        //turn off metal's use of bold fonts
        UIManager.put("swing.boldMetal", Boolean.FALSE);
        /* Schedule a job for the event-dispatching thread:
           creating and showing this application's GUI. */
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createSimulatorGUI(addresses);
            }
        });
    }
    
    private static String[] readConfigFile(String[] args) {
		//read the configuration file
    	String CONFIGURATION_FILE = "";
    	if (args.length != 0)
    		CONFIGURATION_FILE = args[0];
		Properties prop = new Properties();
		try{
			InputStream is = new FileInputStream(CONFIGURATION_FILE);
			prop.load(is);
		}
		catch (IOException e) {
			System.out.println("Configuration File error: "+ e.getMessage() +
								"\nThe correct configuration file path is " +
    										"required as the first argument.");
			System.exit(-1);
		}
		try {
			URI uri_in = new URI("tcp://" + prop.getProperty("BILLBOARD_INPUT"));
			if (uri_in.getHost() == null || uri_in.getPort() == -1) {
				throw new URISyntaxException(uri_in.toString(),
				"URI must have host and port parts");
			}
			URI uri_out = new URI("tcp://" + prop.getProperty("BILLBOARD_OUTPUT"));
			if (uri_out.getHost() == null || uri_out.getPort() == -1) {
				throw new URISyntaxException(uri_out.toString(),
				"URI must have host and port parts");
			}
		} catch (URISyntaxException ex) {
		// validation failed
		System.out.println("validation failed");
		System.exit(-1);
		}
    	String[] addresses = new String[2];
    	addresses[0] = "tcp://" + prop.getProperty("BILLBOARD_INPUT");
    	addresses[1] = "tcp://" + prop.getProperty("BILLBOARD_OUTPUT");
    	System.out.println("BILLBOARD_INPUT Server runs on: "+addresses[0]);
		System.out.println("BILLBOARD_OUTPUT Publisher runs on: "+addresses[1]);
		return addresses;  
    }
}

