
package spectator_gui;

import javax.swing.SwingWorker;
import com.inspirel.yami.Agent;
import com.inspirel.yami.Parameters;
import com.inspirel.yami.YAMIIOException;

class Client extends SwingWorker<Void, Void> {
    
    //This Class follow Client pattern
	//asynchronous properties are extended by yami4
    private String[] args;
    private SpectatorGUI spectator_gui;
    private boolean exit;
    private boolean is_connected;
    private Agent client_agent;
    private Parameters params;
    private String server_address;
    private final static String[] actions = { "spectatorGUI.connect",
                        			 		"spectatorGUI.disconnect" };
    
    public Client(String billboard_input_address, SpectatorGUI spectator_gui) {
		this.spectator_gui = spectator_gui;
		this.server_address = billboard_input_address;
		exit = false;
		is_connected = false;
		try {
			//prepare client agent
			client_agent = new Agent();
			params = new Parameters();
			params.setString("action", actions[0]);
		}
		catch (YAMIIOException ex) {
			System.out.println("[client]* error: " + ex.getMessage());
		}
		System.out.println("[client]* Client is ready");
    }
    
    protected Void doInBackground() {
		System.out.println("[client]* Client started");
		spectator_gui.runningSignal();
		try {
			Thread.sleep(1500);
			while (!exit) {
				try {
					//try to connect to Billboard after gui opening
					spectator_gui.printToTextBox("Trying to connect to " +
																"Billboard...");
					System.out.println("[client]> " + actions[0] + " was sent");
					client_agent.sendOneWay(server_address, "controller",
														actions[0], params);
					System.out.println("[client]* Waiting response...");
					//trigger actions on the gui and starts subscribers
					is_connected = true;
					System.out.println("[client]< Connected to Billboard, " +
													"SpectatorGUI is allowed.");
					spectator_gui.printToTextBox("Connected to Billboard, " +
													"SpectatorGUI is allowed.");
			    	spectator_gui.startSubscribers();
			    	//close wait dialog
					spectator_gui.closeWaitDialog();
					spectator_gui.printToTextBox("GUI is ready, waiting " +
												"for incoming fotograms.");
					if (!exit) {
						synchronized (this) {
							wait();
						}
					}
				}
				catch (YAMIIOException ex) {
					System.out.println("[client]* error: " + ex.getMessage());
				}
				//jumped on yami exception
				//retry every 5 seconds if billboard is not available
				if (!exit)
					Thread.sleep(5000);
			}
		}
		catch (InterruptedException ex) {
			exit = true;
		}
		//deregister from Billboard on gui closure
		if (is_connected) {
			try {
				System.out.println("[client]> " + actions[1] + " was sent" );
				client_agent.sendOneWay(server_address, "controller",
														actions[1], params);
			}
			catch (YAMIIOException ex) {
				System.out.println("[client]* error: " + ex.getMessage());
			}
		}
		//wait a second to let disconnect signal's come out
		try {
			Thread.sleep(1000);
		}
		catch (InterruptedException ex) { }
		
		System.out.println("[client]* Client terminates");
		client_agent.close();
		spectator_gui.completitionSignal();
        return null;
	}
	
	void exitTask() {
		exit = true;
		cancel(true);
	}
	
}

