
package spectator_gui;

import javax.swing.SwingWorker;
import com.inspirel.yami.Agent;
import com.inspirel.yami.IncomingMessage;
import com.inspirel.yami.IncomingMessageCallback;
import com.inspirel.yami.Parameters;
import com.inspirel.yami.YAMIIOException;

class EventSubscriber extends SwingWorker<Void, Void> {

 	private static SpectatorGUI spectator_gui;
	private String[] args;
	private String publisher_address;
	private Agent stats_subscriber_agent;
	private Parameters stats_params;
	private Agent msg_subscriber_agent;
	private Parameters msg_params;
    private final String update_object_name = "update_handler";
	
	public EventSubscriber(String billboard_output_address,
												SpectatorGUI spectator_gui) {
		this.spectator_gui = spectator_gui;
		this.publisher_address = billboard_output_address;
		try {
			//prepare callback agents
			stats_subscriber_agent = new Agent();
		    stats_params = new Parameters();
		    stats_params.setString("destination_object", update_object_name);
			msg_subscriber_agent = new Agent();
		    msg_params = new Parameters();
		    msg_params.setString("destination_object", update_object_name);
			
		}
		catch (YAMIIOException ex) {
			System.out.println("[event subscriber]* error: " + ex.getMessage());
		}
		System.out.println("[event subscriber]* EventSubscriber is ready");
	}
	
	protected Void doInBackground() {
        System.out.println("[event subscriber]* EventSubscriber started");
        spectator_gui.runningSignal();
		try {
			Thread.sleep(200);
			try {
			    //try to subscribe to teams statistics channel
			    stats_subscriber_agent.registerObject(update_object_name,
		    									new StatisticHandler());
			    stats_subscriber_agent.sendOneWay(publisher_address,
			    								"Teams_Statistics",
			    								"subscribe", stats_params);
			    System.out.println("[event subscriber]< Subscribed to " +
			    								"Teams_Statistics channel");
			    spectator_gui.printToTextBox("Subscribed to Game " +
			    									"Statistics Channel.");
			    //try to subscribe to match facts messages channel
			    msg_subscriber_agent.registerObject(update_object_name,
			    								  new TextMessageHandler());
			    msg_subscriber_agent.sendOneWay(publisher_address,
			    								"Spectator_Messages",
			    								"subscribe", msg_params);
			    System.out.println("[event subscriber]< Subscribed to " +
			    							  "Spectator_Messages channel");
			    spectator_gui.printToTextBox("Subscribed to Game Facts " +
			    										"Messages Channel.");
			    //wait for gui closure and receive updates in background
	        	synchronized (this) {
					wait();
				}
			}
			catch (YAMIIOException ex) {
			    //billboard output is not available
			    System.out.println("[event subscriber]* error: " +
			    										ex.getMessage());
			}
		}
		catch (InterruptedException ex) { }
        System.out.println("[event subscriber]* EventSubscriber terminates");
        stats_subscriber_agent.unregisterObject(update_object_name);
        msg_subscriber_agent.unregisterObject(update_object_name);
        stats_subscriber_agent.close();
		msg_subscriber_agent.close();
        spectator_gui.completitionSignal();
        return null;
	}
	

	private static class StatisticHandler implements IncomingMessageCallback {
		//Method that receive callbacks
		public void call(IncomingMessage im) throws Exception {
			//receive message from channel
			Parameters content = im.getParameters();
			//get locals stats from message
			String local_name = new String(content.getString("LOCAL_Name"));
			int local_score = content.getInteger("LOCAL_Score");
			int local_catches = content.getInteger("LOCAL_Catches_Number");
			int local_autogoals = content.getInteger("LOCAL_Autogoals_Number");
			int local_shootings = content.getInteger("LOCAL_Shootings_Number");
			int local_subst = content.getInteger("LOCAL_Substitutions_Number");
			int local_throws = content.getInteger("LOCAL_Throw_In_Number");
			int local_freekicks = content.getInteger("LOCAL_Free_Kicks_Number");
			int local_penalykicks = content.getInteger("LOCAL_Penalty_Kicks_Number");
			int local_goalkicks = content.getInteger("LOCAL_Goal_Kicks_Number");
			int local_kick_off = content.getInteger("LOCAL_Kick_Off_Number");
			int local_corners = content.getInteger("LOCAL_Corner_Kicks_Number");
			int local_passings = content.getInteger("LOCAL_Passings_Number");
			int local_headings = content.getInteger("LOCAL_Headings_Number");
			int local_tacklesdone = content.getInteger("LOCAL_Tackles_Done_Number");
			int local_tacklessuff = content.getInteger("LOCAL_Tackles_Suffered_Number");
			int local_folusdone = content.getInteger("LOCAL_Fouls_Done_Number");
			int local_foulssuff = content.getInteger("LOCAL_Fouls_Suffered_Number");
			int local_yellowcards = content.getInteger("LOCAL_Yellow_Cards_Number");
			int local_redcards = content.getInteger("LOCAL_Red_Cards_Number");
			//get visitors stats from message
			String visitor_name = new String(content.getString("VISITOR_Name"));
			int visitor_score = content.getInteger("VISITOR_Score");
			int visitor_catches = content.getInteger("VISITOR_Catches_Number");
			int visitor_autogoals = content.getInteger("VISITOR_Autogoals_Number");
			int visitor_shootings = content.getInteger("VISITOR_Shootings_Number");
			int visitor_subst = content.getInteger("VISITOR_Substitutions_Number");
			int visitor_throws = content.getInteger("VISITOR_Throw_In_Number");
			int visitor_freekicks = content.getInteger("VISITOR_Free_Kicks_Number");
			int visitor_penalykicks = content.getInteger("VISITOR_Penalty_Kicks_Number");
			int visitor_goalkicks = content.getInteger("VISITOR_Goal_Kicks_Number");
			int visitor_kick_off = content.getInteger("VISITOR_Kick_Off_Number");
			int visitor_corners = content.getInteger("VISITOR_Corner_Kicks_Number");
			int visitor_passings = content.getInteger("VISITOR_Passings_Number");
			int visitor_headings = content.getInteger("VISITOR_Headings_Number");
			int visitor_tacklesdone = content.getInteger("VISITOR_Tackles_Done_Number");
			int visitor_tacklessuff = content.getInteger("VISITOR_Tackles_Suffered_Number");
			int visitor_folusdone = content.getInteger("VISITOR_Fouls_Done_Number");
			int visitor_foulssuff = content.getInteger("VISITOR_Fouls_Suffered_Number");
			int visitor_yellowcards = content.getInteger("VISITOR_Yellow_Cards_Number");
			int visitor_redcards = content.getInteger("VISITOR_Red_Cards_Number");	
			//update score label
			spectator_gui.updateScore(local_name, local_score,
										visitor_name, visitor_score);
			//update statistics dialog
			spectator_gui.getStatsDialog().updateStats(local_name.trim(),
											local_score, local_shootings,
											local_catches, local_corners,
											local_folusdone, local_foulssuff,
											local_yellowcards, local_redcards,
											local_freekicks, local_penalykicks,
											local_autogoals, local_passings,
											local_tacklesdone, local_tacklessuff,
											local_throws, local_headings,
											local_goalkicks, local_kick_off,
											visitor_name.trim(), visitor_score,
											visitor_shootings, visitor_catches,
											visitor_corners, visitor_folusdone,
											visitor_foulssuff, visitor_yellowcards,
											visitor_redcards, visitor_freekicks,
											visitor_penalykicks, visitor_autogoals,
											visitor_passings, visitor_tacklesdone,
											visitor_tacklessuff, visitor_throws,
											visitor_headings, visitor_goalkicks,
											visitor_kick_off);
		}
	}
	
	
	private static class TextMessageHandler implements IncomingMessageCallback {
		//Method that receive callbacks
		public void call(IncomingMessage im) throws Exception {
			//receive message from channel
			Parameters content = im.getParameters();
			//update text box with the new match fact message
			spectator_gui.printToTextBox(content.getString("Message"));
		}
	}

}

