
package spectator_gui;

import javax.swing.SwingWorker;
import com.inspirel.yami.Agent;
import com.inspirel.yami.IncomingMessage;
import com.inspirel.yami.IncomingMessageCallback;
import com.inspirel.yami.Parameters;
import com.inspirel.yami.YAMIIOException;

class FotogramSubscriber extends SwingWorker<Void, Void> {

 	private static SpectatorGUI spectator_gui;
	private String[] args;
	private boolean is_connected;
	private int selected_fotogram_index;
	private String publisher_address;
	private Agent subscriber_agent;
	private Parameters params;
    private final String update_object_name = "update_handler";
    private final static String[] fotogram_quality = { "Fotogram_Frame",
													"Regular_Fotogram_Frame",
													"Sporadic_Fotogram_Frame" };
    	
	public FotogramSubscriber(String billboard_output_address,
												SpectatorGUI spectator_gui) {
		this.spectator_gui = spectator_gui;
		this.publisher_address = billboard_output_address;
		is_connected = false;
		selected_fotogram_index = 0;
		try {
			//prepare callback agent
			subscriber_agent = new Agent();
			subscriber_agent.registerObject(update_object_name,
												new FotogramHandler());
			params = new Parameters();
			params.setString("destination_object", update_object_name);	
		}
		catch (YAMIIOException ex) {
			System.out.println("[fotogram subscriber]* error: " +
														ex.getMessage());
		}
		System.out.println("[fotogram subscriber]* FotogramSubscriber is ready");
	}
	
	protected Void doInBackground() {
        System.out.println("[fotogram subscriber]* FotogramSubscriber started");
        spectator_gui.runningSignal();
		try {
			Thread.sleep(150);
			try {
			    //try to subscribe to fotogram frame channel
			    subscriber_agent.sendOneWay(publisher_address,
									fotogram_quality[selected_fotogram_index],
									"subscribe", params);
				System.out.println("[fotogram subscriber]< Subscribed to " +
								fotogram_quality[selected_fotogram_index] +
																" channel");
				spectator_gui.printToTextBox("Subscribed to Game " +
													"Snapshots Channel.");
		    	is_connected = true;
			    //wait for gui closure and receive updates in background
	    		synchronized (this) {
					wait();
				}
			}
			catch (YAMIIOException ex) {
				//billboard output is not available
			    System.out.println("[fotogram subscriber]* error: " +
			    										ex.getMessage());
			    spectator_gui.printToTextBox("Error: Unable to find " +
			    								"Billboard Output. Please " +
			    								"start both Billboard Input " +
			    								"and Billboard Output and " +
			    								"restart this GUI.");
			}
        }
		catch (InterruptedException ex) { }
        System.out.println("[fotogram subscriber]* FotogramSubscriber terminates");
        subscriber_agent.unregisterObject(update_object_name);
        subscriber_agent.close();
        spectator_gui.completitionSignal();
        return null;
	}
	
    void registerToFotogram(int fotogram_quality_index) throws YAMIIOException {
		selected_fotogram_index = fotogram_quality_index;
		if (is_connected) {
			//unsubscribe from the old fotogram frame channel
			subscriber_agent.unregisterObject(update_object_name);
			subscriber_agent.close();
			//subscribe to the selected fotogram frame channel
			subscriber_agent = new Agent();
			subscriber_agent.registerObject(update_object_name,
												new FotogramHandler());
			subscriber_agent.sendOneWay(publisher_address,
									fotogram_quality[selected_fotogram_index],
									"subscribe", params);
			System.out.println("[fotogram subscriber]< Subscribed to " +
							fotogram_quality[selected_fotogram_index] +
																" channel");
			spectator_gui.printToTextBox("Subscribed to Game Snapshots Channel.");
		}
	}
	
	
	private static class FotogramHandler implements IncomingMessageCallback {
		private static boolean incoming = false;
		//Method that receive callbacks
		public void call(IncomingMessage im) throws Exception {
			//if it is the first callback call, then start the Renderer task
			if (!incoming) {
				System.out.println("[fotogram subscriber]< Incoming fotograms");
				spectator_gui.startRender();
				incoming = true;
			}
			//receive message from the channel
			Parameters content = im.getParameters();
			//get fotogram content
			int match_time_m = content.getInteger("Minutes");
			int match_time_s = content.getInteger("Seconds");
			int[] ball_position = content.getIntegerArray("Ball_Position");
			int[][] local_team = new int[11][3];
			int[][] visitor_team = new int[11][3];
			for (int i = 1; i <= 11; i++) {
				local_team[i-1] = content.getIntegerArray("LOCAL_Player" +
										Integer.toString(i) + "_Frame");
				visitor_team[i-1] = content.getIntegerArray("VISITOR_Player" +
										Integer.toString(i) + "_Frame");
			}
			//update game status, renderer will takes care to repaint it 
			spectator_gui.setMatchTime(match_time_m, match_time_s);
				SoccerFieldStatus soccer_field_status = spectator_gui.
									getSoccerField().getSoccerFieldStatus();
				synchronized (soccer_field_status) {
				soccer_field_status.setBallPosition(SoccerFieldStatus.Axis.X,
															ball_position[0]);
				soccer_field_status.setBallPosition(SoccerFieldStatus.Axis.Y,
															ball_position[1]);
				soccer_field_status.setBallPosition(SoccerFieldStatus.Axis.Z,
															ball_position[2]);
				for(int i = 0; i < 11; i++) {
					soccer_field_status.setPlayerNumber("team1", i,
														local_team[i][0]);
					soccer_field_status.setPlayerPosition("team1", i,
								SoccerFieldStatus.Axis.X, local_team[i][1]);
					soccer_field_status.setPlayerPosition("team1", i,
								SoccerFieldStatus.Axis.Y, local_team[i][2]);
					soccer_field_status.setPlayerNumber("team2", i,
														visitor_team[i][0]);
					soccer_field_status.setPlayerPosition("team2", i,
								SoccerFieldStatus.Axis.X, visitor_team[i][1]);
					soccer_field_status.setPlayerPosition("team2", i,
								SoccerFieldStatus.Axis.Y, visitor_team[i][2]);
				}
			}
		}
	}
	
}

