
package spectator_gui;

import javax.swing.JPanel;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Font;
import java.awt.Color;
import java.awt.RenderingHints;

class SoccerFieldPane extends JPanel {
	
	private int component_width, component_height,
				field_width, field_height,
				field_origin_w, field_origin_h;
	private double magnification_ratio;
	private boolean show_players_numbers;
	private boolean show_ball, show_players;
	private SoccerFieldStatus soccer_field_status;
	
	public SoccerFieldPane() {
		soccer_field_status = new SoccerFieldStatus();
		magnification_ratio = 1.0;
		show_players_numbers = true;
		show_ball = true;
		show_players = true;	
	}
	
	protected void paintComponent(Graphics graphics) {
		//set anti-aliased
		Graphics2D g2 = (Graphics2D)graphics;
    	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
    									RenderingHints.VALUE_ANTIALIAS_ON);
		
		//get component dimension
		component_width = getWidth();
		component_height = getHeight();
	
		//draw gray background
		graphics = drawComponentBackground(graphics, new Color(177, 177, 177));
	
		//calculate soccer field dimension and position
		field_width = component_width - (int)(component_width * 0.0800);
		field_origin_w = (int)(component_width * 0.0400);
		if (field_width < 300) {
			field_width = 300;
			field_origin_w = 12; 
		}
		field_height = (int)(field_width * 0.6538);
		if (field_height + (int)(component_height * 0.1130) > component_height) {
			field_height = component_height - (int)(component_height * 0.1130);
			field_width = (int)(field_height * 1.5294);
			field_origin_w = (int)((component_width - field_width) / 2);
		}
		field_origin_h = (int)((component_height - field_height) / 2);
		
		//draw green field
		graphics = drawSoccerGreenField(graphics, new Color(64, 160, 64),
													new Color(56, 156, 56));
		
		//draw soccer field lines
		graphics = drawSoccerFieldLines(graphics, new Color(255, 255, 255));
		
		synchronized (soccer_field_status) {
			if (show_players) {
				//draw team1 (red team)
				graphics = drawSoccerTeam(graphics, "team1", new Color(255, 30, 30),
														new Color(255, 255, 255));
		
				//draw team2 (blue team)
				graphics = drawSoccerTeam(graphics, "team2", new Color(30, 30, 255),
															new Color(255, 255, 255));
			}
		
			if (show_ball) {
				//draw the ball
				graphics = drawSoccerBall(graphics,
									new Color(255, 255, 255),
									new Color(0, 0, 0),
									 // 0 < x < 105
									 convertXPosition(soccer_field_status.
									 	getBallPosition(SoccerFieldStatus.Axis.X)),
									 // 0 < y < 69
									 convertYPosition(soccer_field_status.
									 	getBallPosition(SoccerFieldStatus.Axis.Y)),
									 // 0 < z < 1000
									 (int)soccer_field_status.
									 	getBallPosition(SoccerFieldStatus.Axis.Z));
			}
		}
	}
	
	SoccerFieldStatus getSoccerFieldStatus() {
		return soccer_field_status;
	}
	
	boolean getShowPlayersNumbers() {
		return show_players_numbers;
	}
	
	void setShowPlayersNumbers(boolean flag) {
		show_players_numbers = flag;
	}
	
	private double getMagnificationRatio() {
		return magnification_ratio;
	}
	
	void setMagnificationRatio(double ratio) {
		magnification_ratio = ratio;
	}
	
	private int convertXPosition(double x_position) {
		return (int)(field_width * x_position / 103.0);
	}
	
	private int convertYPosition(double y_position) {
		return (int)(field_height * y_position / 67.0);
	}
	
	private void showBall(boolean flag) {
		show_ball = flag;
	}
	
	private void showPlayers(boolean flag) {
		show_players = flag;
	}
	
	private Graphics drawComponentBackground(Graphics graphics, Color color) {
		graphics.setColor(color);
		graphics.fillRect(0, 0, component_width - 1, component_height - 1);
		return graphics;
	}
	
	private Graphics drawSoccerGreenField(Graphics graphics,
											Color color1, Color color2) {
		//field
		graphics.setColor(color1);
		graphics.fillRoundRect(field_origin_w - (int)(field_height * 0.0500),
						  field_origin_h - (int)(field_height * 0.0500),
						  field_width + (int)(field_height * 0.1000),
						  field_height + (int)(field_height * 0.1000),
						  (int)(field_height / 10), (int)(field_height / 10));
		//field mower stripes
		graphics.setColor(color2);
		for(int i = 0; i < 12; i++) {
			graphics.fillRect(field_origin_w + (int)(field_width / 24) +
						(int)(i * (field_width / 12)), field_origin_h,
					 	(int)(field_width / 24), field_height);
		}
		return graphics;
	}

	private Graphics drawSoccerFieldLines(Graphics graphics, Color color) {
		graphics.setColor(color);
		//outer rectangle
		graphics.drawRect(field_origin_w, field_origin_h,
									field_width, field_height);
		//half-way line
		graphics.drawLine(field_origin_w + (field_width / 2), field_origin_h,
											 field_origin_w + (field_width / 2),
											 field_origin_h + field_height ); 
		//centre circle
		graphics.drawOval(field_origin_w + (field_width / 2) -
								(((int)(field_height * 0.2691)) / 2),
						 		field_origin_h + (field_height / 2) -
						 		(((int)(field_height * 0.2691)) / 2),
						 		(int)(field_height * 0.2691),
						 		(int)(field_height * 0.2691)); 
		//centre spot
		graphics.fillOval(field_origin_w + (field_width / 2) -
							(int)(field_height * 0.0015),
							field_origin_h + (field_height / 2) -
							(int)(field_height * 0.0015),
						 	(int)(field_height * 0.0029),
						 	(int)(field_height * 0.0029)); 
		//upper left corner
		graphics.drawArc(field_origin_w - (((int)(field_height * 0.0294)) / 2),
						field_origin_h -(((int)(field_height * 0.0294)) / 2),
						(int)(field_height * 0.0294),
						(int)(field_height * 0.0294),
						270, 90); 
		//upper right corner
		graphics.drawArc(field_origin_w + field_width -
						(((int)(field_height * 0.0294)) / 2),
						field_origin_h - (((int)(field_height * 0.0294)) / 2),
						(int)(field_height * 0.0294),
						(int)(field_height * 0.0294),
						180, 90); 
		//bottom right corner
		graphics.drawArc(field_origin_w + field_width -
						(((int)(field_height * 0.0294)) / 2),
						field_origin_h + field_height -
						(((int)(field_height * 0.0294)) / 2),
						(int)(field_height * 0.0294),
						(int)(field_height * 0.0294),
						90, 90); 
		//bottom left corner
		graphics.drawArc(field_origin_w - (((int)(field_height * 0.0294)) / 2),
							field_origin_h + field_height -
							(((int)(field_height * 0.0294)) / 2),
							(int)(field_height * 0.0294),
							(int)(field_height * 0.0294),
							0, 90);
		//LEFT side
		//left area outer rect
		graphics.drawRect(field_origin_w,
						field_origin_h + (int)((field_height -
						(field_height * 0.5926)) / 2),
					 	(int)(field_height * 0.2426),
					 	(int)(field_height * 0.5926));
		//left area inner rect
		graphics.drawRect(field_origin_w,
							field_origin_h +
							(int)((field_height - (field_height * 0.2694)) / 2),
					 		(int)(field_height * 0.0808),
					 		(int)(field_height * 0.2694));
		//left goal
		graphics.drawRect(field_origin_w - (int)(field_height * 0.0358),
					 		field_origin_h + (int)((field_height -
					 		(field_height * 0.1076)) / 2),
					 		(int)(field_height * 0.0358),
					 		(int)(field_height * 0.1076));
		//left penalty kick spot
		graphics.fillOval(field_origin_w + (int)(field_height * 0.1617) -
							(int)(field_height * 0.0015),
					 		field_origin_h + (field_height / 2) -
					 		(int)(field_height * 0.0015),
					 		(int)(field_height * 0.0029),
					 		(int)(field_height * 0.0029));
		//left area arc
		graphics.drawArc(field_origin_w + (int)(field_height * 0.1617) -
						(((int)(field_height * 0.2691)) / 2),
						field_origin_h + (field_height / 2) -
						(((int)(field_height * 0.2691)) / 2),
						(int)(field_height * 0.2691),
						(int)(field_height * 0.2691),
						308, 104);
		//RIGHT side
		//right area outer rect
		graphics.drawRect(field_origin_w + field_width -
							(int)(field_height * 0.2426),
					 		field_origin_h + (int)((field_height -
					 		(field_height * 0.5926)) / 2),
					 		(int)(field_height * 0.2426),
					 		(int)(field_height * 0.5926)); 
		//right area inner rect
		graphics.drawRect(field_origin_w + field_width -
							(int)(field_height * 0.0808),
					 		field_origin_h + (int)((field_height -
					 		(field_height * 0.2694)) / 2),
						 	(int)(field_height * 0.0808),
						 	(int)(field_height * 0.2694)); 
		//right goal
		graphics.drawRect(field_origin_w + field_width,
					 		field_origin_h + (int)((field_height -
					 		(field_height * 0.1076)) / 2),
					 		(int)(field_height * 0.0358),
					 		(int)(field_height * 0.1076)); 
		//right penalty kick spot
		graphics.fillOval(field_origin_w + field_width -
							(int)(field_height * 0.1617) -
							(int)(field_height * 0.0015),
					 		field_origin_h + (field_height / 2) -
					 		(int)(field_height * 0.0015),
					 		(int)(field_height * 0.0029),
					 		(int)(field_height * 0.0029)); 
		//right area arc
		graphics.drawArc(field_origin_w + field_width -
						(int)(field_height * 0.1617) -
						(((int)(field_height * 0.2691)) / 2),
						field_origin_h + (field_height / 2) -
						(((int)(field_height * 0.2691)) / 2),
						(int)(field_height * 0.2691),
						(int)(field_height * 0.2691),
						128, 104); 
		return graphics;
	}

	private Graphics drawSoccerBall(Graphics graphics, Color color1,
											Color color2, int x, int y, int z) {
		double ball_magnification_ratio;
		if (z < 800.0)
			ball_magnification_ratio = (z / 800.0) + 0.6;
		else
			ball_magnification_ratio = 2.0;
		//outer circle
		graphics.setColor(color1);
		graphics.fillOval(field_origin_w - (int)(field_height * 0.0147) -
							(int)(field_height * 0.0073 *
							ball_magnification_ratio * magnification_ratio) + x,
					 		field_origin_h - (int)(field_height * 0.0147) -
					 		(int)(field_height * 0.0073 *
					 		ball_magnification_ratio * magnification_ratio) + y,
					 		(int)(field_height * 0.0147 *
					 		ball_magnification_ratio * magnification_ratio),
					 		(int)(field_height * 0.0147 *
					 		ball_magnification_ratio * magnification_ratio));
		//inner center circle
		graphics.setColor(color2);
		graphics.fillOval(field_origin_w - (int)(field_height * 0.0147) -
							(int)(field_height * 0.0025 *
							ball_magnification_ratio * magnification_ratio) + x,
							field_origin_h - (int)(field_height * 0.0147) -
							(int)(field_height * 0.0025 *
							ball_magnification_ratio * magnification_ratio) + y,
							(int)(field_height * 0.0050 *
							ball_magnification_ratio * magnification_ratio),
					 		(int)(field_height * 0.0050 *
					 		ball_magnification_ratio * magnification_ratio)); 
		//inner left circle
		graphics.fillOval(field_origin_w - (int)(field_height * 0.0147) -
							(int)(field_height * 0.0055 *
							ball_magnification_ratio * magnification_ratio) + x,
					 		field_origin_h - (int)(field_height * 0.0147) -
					 		(int)(field_height * 0.0055 *
					 		ball_magnification_ratio * magnification_ratio) + y,
					 		(int)(field_height * 0.0030 *
					 		ball_magnification_ratio * magnification_ratio),
					 		(int)(field_height * 0.0030 *
					 		ball_magnification_ratio * magnification_ratio)); 
		//inner right circle
		graphics.fillOval(field_origin_w - (int)(field_height * 0.0147) +
							(int)(field_height * 0.0043 *
							ball_magnification_ratio * magnification_ratio) + x,
					 		field_origin_h - (int)(field_height * 0.0147) -
					 		(int)(field_height * 0.0025 *
					 		ball_magnification_ratio * magnification_ratio) + y,
					 		(int)(field_height * 0.0030 *
					 		ball_magnification_ratio * magnification_ratio),
					 		(int)(field_height * 0.0030 *
					 		ball_magnification_ratio * magnification_ratio)); 
		//inner bottom circle		
		graphics.fillOval(field_origin_w - (int)(field_height * 0.0147) -
							(int)(field_height * 0.0032 *
							ball_magnification_ratio * magnification_ratio) + x,
					 		field_origin_h - (int)(field_height * 0.0147) +
					 		(int)(field_height * 0.0042 *
					 		ball_magnification_ratio * magnification_ratio) + y,
					 		(int)(field_height * 0.0030 *
					 		ball_magnification_ratio * magnification_ratio),
					 		(int)(field_height * 0.0030 *
					 		ball_magnification_ratio * magnification_ratio)); 
		return graphics;
	}
	
	private Graphics drawSoccerPlayer(Graphics graphics, Color color1,
									Color color2, int number, int x, int y) {
		graphics.setColor(color1);
		//draw square
		graphics.fillRect(field_origin_w - (int)(field_height * 0.0147) -
							(int)((field_height * 0.0073) *
							magnification_ratio) + x,
					 		field_origin_h - (int)(field_height * 0.0147) -
					 		(int)((field_height * 0.0073) *
					 		magnification_ratio) + y,
					 		(int)(field_height * 0.0147 * magnification_ratio),
					 		(int)(field_height * 0.0147 * magnification_ratio)); 
		graphics.setColor(color2);
		int font_size = (int)(8 + ((field_width - 300) / 86) *
										((magnification_ratio - 1) / 2));
		graphics.setFont(new Font(graphics.getFont().getName(),
												Font.BOLD, font_size));
		//draw uniform numbers
		if(show_players_numbers) {
			graphics.drawString(Integer.toString(number),
						   field_origin_w - (int)(field_height * 0.0147) -
						   (int)((field_height * 0.0073) *
						   magnification_ratio) + x,
						   field_origin_h - (int)(field_height * 0.0147) +
						   (int)((field_height * 0.0050) *
						   magnification_ratio) + y);
		}
		return graphics;
	}
	
	private Graphics drawSoccerTeam(Graphics graphics, String team,
												Color color1, Color color2) {
		//draw players
		for(int i = 0; i < 11; i++) {
			graphics = drawSoccerPlayer(graphics, color1, color2,
										soccer_field_status.
										getPlayerNumber(team, i),
										convertXPosition(soccer_field_status.
										getPlayerPosition(team, i,
										SoccerFieldStatus.Axis.X)),
										convertYPosition(soccer_field_status.
										getPlayerPosition(team, i,
										SoccerFieldStatus.Axis.Y)));
		}
		return graphics;
	}
	
}

