
package spectator_gui;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JSlider;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.GroupLayout;
import javax.swing.LayoutStyle;
import javax.swing.BorderFactory;
import javax.swing.UIManager;
import javax.swing.SwingConstants;
import javax.swing.SwingWorker;
import javax.swing.UnsupportedLookAndFeelException;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Dimension;
import java.awt.Font;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.concurrent.ExecutionException;
import com.inspirel.yami.YAMIIOException;
import java.net.URI;
import java.net.URISyntaxException;


class SpectatorGUI extends JPanel {
	
	private Listener listener;
	//tasks
	private Renderer renderer;
	private Client client;
	private FotogramSubscriber fotogram_subscriber;
	private EventSubscriber event_subscriber;

	//ui elements
	private SoccerFieldPane soccer_field_pane;
	private JComboBox render_quality_combo;
	private JSlider magnification_slider;
	private JCheckBox players_numbers_checkbox;
	private JTextArea output_textarea;
	private JLabel teams_score;
	private JLabel match_time;
	private JButton stats_button;
	private StatisticsDialog stats_dialog;
	private WaitDialog wait_dialog;
	
	//gui internal state
	private static final int RATIO_MIN = 100;
    private static final int RATIO_MAX = 300;
    private static final int RATIO_INIT = 100;
    private int completed_tasks = 0;
    private int running_tasks = 0;
	
	public SpectatorGUI(String[] args) {
		System.out.println("[spectator gui]* Starting application...");
		
		//define the GUI Listener
		listener = new Listener(this);
		
		//soccer field panel
		soccer_field_pane = new SoccerFieldPane();
		
		//magnification label and slider
		JLabel magnification_label = new JLabel("Players magnification (%)");
		magnification_slider = new JSlider(JSlider.HORIZONTAL, RATIO_MIN,
													RATIO_MAX, RATIO_INIT);
        magnification_slider.setBorder(BorderFactory.
        								createEmptyBorder(0, 0, 10, 0));
        Font font = new Font("Arial", Font.ITALIC, 9);
        magnification_slider.setFont(font);
		magnification_slider.addChangeListener(listener);
		
		//players numbers checkbox
		players_numbers_checkbox = new JCheckBox("Players numbers");
		players_numbers_checkbox.setActionCommand("players_numbers");
		players_numbers_checkbox.setSelected(true);
		players_numbers_checkbox.addActionListener(listener);
		
		//render quality
		JLabel render_label = new JLabel("Render quality");
		String[] available_modes = { "High", "Medium", "Low" };
		render_quality_combo = new JComboBox(available_modes);
		render_quality_combo.addActionListener(listener);
		
		//teams and score
		teams_score = new JLabel("Initializing...", SwingConstants.CENTER);
		teams_score.setFont(new Font("Arial", Font.BOLD, 16));
		
		//match time
		match_time = new JLabel("--:--");
		match_time.setFont(new Font("Arial", Font.BOLD, 12));
		
		//statistics button
		stats_button = new JButton("Show Statistics");
		stats_button.setActionCommand("stats");
		stats_button.addActionListener(listener);
		
		System.out.println("[spectator gui]* Listener binding complete");
		
		//statistics text area and scroll pane
		output_textarea = new JTextArea("> Live match:");
		output_textarea.setLineWrap(true);
		output_textarea.setWrapStyleWord(true);
		output_textarea.setEditable(false);
		JScrollPane scroll_pane = new JScrollPane(output_textarea);
		
		//tools pane
		JPanel tools_pane = new JPanel();
		GroupLayout tools_pane_group_layout = new GroupLayout(tools_pane);
		tools_pane.setLayout(tools_pane_group_layout);
		tools_pane_group_layout.setAutoCreateGaps(true);
		tools_pane_group_layout.setAutoCreateContainerGaps(true);
		tools_pane_group_layout.setHorizontalGroup(
   			tools_pane_group_layout.createSequentialGroup()
   				.addGroup(tools_pane_group_layout.
   						createParallelGroup(GroupLayout.Alignment.CENTER)
   					.addGroup(tools_pane_group_layout.createSequentialGroup()
   						.addComponent(render_label)
						.addComponent(render_quality_combo,
									GroupLayout.PREFERRED_SIZE,
									GroupLayout.DEFAULT_SIZE,
									GroupLayout.PREFERRED_SIZE)
   					)
   					.addGroup(tools_pane_group_layout.createSequentialGroup()
		   				.addGroup(tools_pane_group_layout.
		   					createParallelGroup(GroupLayout.Alignment.CENTER)
		   					.addComponent(magnification_label)
		   					.addComponent(magnification_slider,
					   					GroupLayout.PREFERRED_SIZE,
										GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE)
		   				)
		   				.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
				             					   GroupLayout.DEFAULT_SIZE, 15)
		   				.addComponent(players_numbers_checkbox)
		   			)
	   			)
   		);
		tools_pane_group_layout.setVerticalGroup(
			tools_pane_group_layout.createSequentialGroup()
				.addGroup(tools_pane_group_layout.
					createParallelGroup(GroupLayout.Alignment.CENTER)
					.addComponent(render_label)
					.addComponent(render_quality_combo, 
								GroupLayout.PREFERRED_SIZE,
								GroupLayout.DEFAULT_SIZE,
								GroupLayout.PREFERRED_SIZE)
				)
				.addGroup(tools_pane_group_layout.
					createParallelGroup(GroupLayout.Alignment.CENTER)
					.addGroup(tools_pane_group_layout.createSequentialGroup()
						.addComponent(magnification_label)
						.addComponent(magnification_slider,
									GroupLayout.PREFERRED_SIZE,
									GroupLayout.DEFAULT_SIZE,
									GroupLayout.PREFERRED_SIZE)
					)
					.addComponent(players_numbers_checkbox)
				)
		);
		
		//inner left panel
		JPanel left_pane = new JPanel();
		GroupLayout left_pane_group_layout = new GroupLayout(left_pane);
		left_pane.setLayout(left_pane_group_layout);
		left_pane_group_layout.setHorizontalGroup(
   			left_pane_group_layout.createSequentialGroup()
   				.addGroup(left_pane_group_layout.
   					createParallelGroup(GroupLayout.Alignment.CENTER)
   					.addComponent(soccer_field_pane)
					.addComponent(tools_pane)
   				)
     	);
     	left_pane_group_layout.setVerticalGroup(
			left_pane_group_layout.createSequentialGroup()
				.addComponent(soccer_field_pane)
				.addComponent(tools_pane)
		);
		
		//inner right panel
		JPanel right_pane = new JPanel();
		GroupLayout right_pane_group_layout = new GroupLayout(right_pane);
		right_pane.setLayout(right_pane_group_layout);
		right_pane_group_layout.setAutoCreateGaps(true);
		right_pane_group_layout.setAutoCreateContainerGaps(true);
		right_pane_group_layout.setHorizontalGroup(
   			right_pane_group_layout.createSequentialGroup()
   				.addGroup(right_pane_group_layout.
   					createParallelGroup(GroupLayout.Alignment.CENTER)
					.addComponent(match_time)
					.addComponent(teams_score)
					.addComponent(scroll_pane)
					.addComponent(stats_button)
				)
     	);
		right_pane_group_layout.setVerticalGroup(
			right_pane_group_layout.createSequentialGroup()
				.addComponent(match_time)
				.addComponent(teams_score)
				.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
                     				GroupLayout.DEFAULT_SIZE, 15)
				.addComponent(scroll_pane)
				.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
                     				GroupLayout.DEFAULT_SIZE, 15)
				.addComponent(stats_button)
		);
		
		//split panel
		JSplitPane split_pane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
													left_pane, right_pane);
		split_pane.setDividerLocation(430);
		split_pane.setOneTouchExpandable(true);
		split_pane.setContinuousLayout(true);

		//add components to the group layout and define layout criteria
		GroupLayout group_layout = new GroupLayout(this);
 		setLayout(group_layout);
		group_layout.setAutoCreateGaps(true);
		group_layout.setAutoCreateContainerGaps(true);
        group_layout.setHorizontalGroup(
   			group_layout.createSequentialGroup()
				.addComponent(split_pane)
     	);
		group_layout.setVerticalGroup(
			group_layout.createSequentialGroup()
				.addComponent(split_pane)
		);
		
		//create StatisticsDialog
		stats_dialog = new StatisticsDialog(this);
		
		//create WaitDialog
		wait_dialog = new WaitDialog(this);
		
		//create Renderer
		renderer = new Renderer(this);
		
		//create FotogramSubscriber
		fotogram_subscriber = new FotogramSubscriber(args[1], this);
		
		//create EventSubscriber
		event_subscriber = new EventSubscriber(args[1], this);
		
		//create and start Client
		(client = new Client(args[0], this)).execute();
	}
	
	SoccerFieldPane getSoccerField() {
		return soccer_field_pane;
	}
	
	StatisticsDialog getStatsDialog() {
		return stats_dialog;
	}
	
	JComboBox getRenderQualityCombo() {
		return render_quality_combo;
	}
	
	void updateScore(String local_name, int local_score,
					 String visitor_name, int visitor_score) {
		String score = "<html><font color=\"red\">" + local_name.trim() +
						" " + local_score + "</font> - <font color=\"blue\">" +
						visitor_score + " " + visitor_name.trim() +
						"</font></html>";
		teams_score.setText(score);
	}
	
	void setMatchTime(int minutes, int seconds) {
		if (seconds >= 0 && seconds <= 9)
			match_time.setText(minutes + ":0" + seconds);
		else
			match_time.setText(minutes + ":" + seconds);
	}
	
	void setChannelEventQuality(int value) {
		try {
			fotogram_subscriber.registerToFotogram(value);
		}
		catch (YAMIIOException ex) {
			System.out.println("[fotogram subscriber]* error: " +
													ex.getMessage());
		}
	}
	
	void printToTextBox(String message) {
		output_textarea.append("\n- " + message);
		int textbox_lenght = output_textarea.getDocument().getLength();
		output_textarea.setCaretPosition(textbox_lenght);
		output_textarea.repaint();
	}
	
	void closeWaitDialog() {
		wait_dialog.dispose();
	}
	
	void setRenderRefresh(int ms) {
		renderer.setRefresh(ms);
	}
	
	void startRender() {
		renderer.execute();
	}
	
	void startSubscribers() {
		fotogram_subscriber.execute();
		event_subscriber.execute();
	}
	
	private void terminateRender() {
		System.out.println("[spectator gui]* Waiting Renderer termination...");
		if (renderer.getState() == SwingWorker.StateValue.STARTED) {
			renderer.terminateRender();
			try {
				renderer.get();
			}
			catch (InterruptedException ex) { }
			catch (ExecutionException ex) { }
		}
		else
			System.out.println("[spectator gui]* Renderer is not running yet");
	}
	
	private void terminateClientAndSubscribers() {
		System.out.println("[spectator gui]* Waiting Client and Subscribers " +
															  "termination...");
		if (fotogram_subscriber.getState() == SwingWorker.StateValue.STARTED) {
			fotogram_subscriber.cancel(true);
		}
		else
			System.out.println("[spectator gui]* FotogramSubscriber is not " +
																 	"running");
		if (event_subscriber.getState() == SwingWorker.StateValue.STARTED) {
			event_subscriber.cancel(true);
		}
		else
			System.out.println("[spectator gui]* EventSubscriber is not " +
																	"running");
		if (client.getState() == SwingWorker.StateValue.STARTED) {
			client.exitTask();
		}
		else
			System.out.println("[spectator gui]* Client is not running");
		waitCompletion();
	}
	
	synchronized void runningSignal() {
		running_tasks++;
	}
	
	synchronized void completitionSignal() {
		completed_tasks++;
		notify();
	}
	
	private synchronized void waitCompletion() {
		while (running_tasks != completed_tasks) {
			try {
				wait();
			}
			catch (InterruptedException ex) { }
		}
	}

    /*	Create the GUI and show it.  For thread safety,
    	this method should be invoked from the
		event-dispatching thread. */
    private static void createSpectatorGUI(String[] args) {
        //Create and set up the window.
        final JFrame frame = new JFrame("Spectator GUI");
        frame.setMinimumSize(new Dimension(750, 435));
        frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.out.println("[spectator gui]* Close SpectatorGUI by " +
																	"the user");
				SpectatorGUI pane = (SpectatorGUI)frame.getContentPane();
				pane.terminateRender();
				pane.terminateClientAndSubscribers();
				System.out.println("[spectator gui]* Exiting SpectatorGUI...");
				System.exit(0);
			}
		});
        
        //Set up the content pane.
		SpectatorGUI content_pane = new SpectatorGUI(args);
		content_pane.setOpaque(true); //content panes must be opaque
		frame.setContentPane(content_pane);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
        frame.setLocationRelativeTo(null);
        content_pane.wait_dialog.setVisible(true);
    }

    public static void main(String[] args) {
    	//read billboard addresses from configuration file
    	final String[] addresses = readConfigFile(args);
    	//use an appropriate Look and Feel
    	try {
            UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
        } catch (UnsupportedLookAndFeelException ex) {
            ex.printStackTrace();
        } catch (IllegalAccessException ex) {
            ex.printStackTrace();
        } catch (InstantiationException ex) {
            ex.printStackTrace();
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        }
        //turn off metal's use of bold fonts
        UIManager.put("swing.boldMetal", Boolean.FALSE);
    	
        /* Schedule a job for the event-dispatching thread:
           creating and showing this application's GUI. */
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createSpectatorGUI(addresses);
            }
        });
    }
    
    private static String[] readConfigFile(String[] args) {
		//read the configuration file
    	String CONFIGURATION_FILE = "";
    	if (args.length != 0)
    		CONFIGURATION_FILE = args[0];
		Properties prop = new Properties();
		try{
			InputStream is = new FileInputStream(CONFIGURATION_FILE);
			prop.load(is);
		}
		catch (IOException e) {
			System.out.println("Configuration File error: "+ e.getMessage() +
								"\nThe correct configuration file path is " +
    										"required as the first argument.");
			System.exit(-1);
		}
		try {
			URI uri_in = new URI("tcp://" + prop.getProperty("BILLBOARD_INPUT"));
			if (uri_in.getHost() == null || uri_in.getPort() == -1) {
				throw new URISyntaxException(uri_in.toString(),
				"URI must have host and port parts");
			}
			URI uri_out = new URI("tcp://" + prop.getProperty("BILLBOARD_OUTPUT"));
			if (uri_out.getHost() == null || uri_out.getPort() == -1) {
				throw new URISyntaxException(uri_out.toString(),
				"URI must have host and port parts");
			}
		} catch (URISyntaxException ex) {
		// validation failed
		System.out.println("validation failed");
		System.exit(-1);
		}
    	String[] addresses = new String[2];
    	addresses[0] = "tcp://" + prop.getProperty("BILLBOARD_INPUT");
    	addresses[1] = "tcp://" + prop.getProperty("BILLBOARD_OUTPUT");
    	System.out.println("BILLBOARD_INPUT Server runs on: "+addresses[0]);
		System.out.println("BILLBOARD_OUTPUT Publisher runs on: "+addresses[1]);
		return addresses;  
    }
}

